
extern "C"{
#include "libmac6903.h"
#include "graph.h"
}
#include "PointSet.h"
#include <cstdlib>
#include <ctime>


typedef enum {
    AD_AVERAGE = 0, AD_MAX, AD_MIN, ADN_AVERAGE, ADN_MAX
}   ArcDen;

typedef enum {
    SGD_SUM = 0, SGD_MAX, SGDN_SUM, SGDN_MAX
} SubGraphDen;

typedef enum {
    AW_DIVISION = 0, AW_LINEAR, AW_EXP, AW_SUM
}  ArcW;

typedef enum {
    GABRIEL_GRAPH=0, GABRIEL_KNN_GRAPH
} GraphType;

float *GenerateWArray( Graph *graph, PSet::DataList input, float sqrsigma)
{
    int i, j, n;
    float *Wnodes = NULL;
    n = input.NumberOfPoints();
    Wnodes = AllocFloatArray(n);
    for(i=0;i<n;i++)
    {
    //    Wnodes[i] = input.Density(input.pt(i), graph->nodes[i].outdegree, input.StdDevSqr());
        for(j=0;j<graph->nodes[i].outdegree;j++)
            Wnodes[i] += exp(-(input.pt(i) - input.pt(graph->nodes[i].adjList[j])).sqrabs()/(2*sqrsigma));
         Wnodes[i] /= (graph->nodes[i].outdegree*sqrt(2*PI*sqrsigma));
    }
    return Wnodes;
}

void ConquestAdjNodes(Graph *graph, int *label, int i, int *currentlabel)
{
    int j;
    bool newlabel;
    if(label[i] == -1)
    {
        newlabel = true;
        for(j=0;j<graph->nodes[i].outdegree;j++) {
            if(label[graph->nodes[i].adjList[j]] != -1) {
                label[i] = label[graph->nodes[i].adjList[j]];
                newlabel = false;  }

            else
                ConquestAdjNodes(graph, label, graph->nodes[i].adjList[j], currentlabel);
        }
        if(newlabel) {
            *currentlabel++;
            label[i] = *currentlabel;
            }
    }
}

Graph *CreateGabrielGraph(PSet::DataList DL, int outdegree, float sigma)
{

    Graph *graph;
    graph = NULL;
    int i, j, k, n = DL.NumberOfPoints();
    float dij, dik, djk;
    bool ConnectEdges;
    graph = CreateGraph(n, outdegree);

    for(i=0;i<n-1;i++)
        for(j=i+1;j<n;j++) {
            ConnectEdges = true;
            dij = (DL.pt(i) - DL.pt(j)).sqrabs();
            for(k=0;k<n;k++) {
                if(k == i || k == j) continue;
                dik = (DL.pt(i) - DL.pt(k)).sqrabs();
                djk = (DL.pt(j) - DL.pt(k)).sqrabs();
                if(dij >= (dik + djk)) {
                    ConnectEdges = false;
                    break;
                }
            }
            if(ConnectEdges)
                addEdge(graph, i, j, 1);
        }

    graph->Wnodes = GenerateWArray(graph, DL, sigma);

    return graph;
}
/*

Graph *CreateGabrielGraph(PSet::DataList DL, int outdegree, float sigma)
{

    Graph *graph;
    graph = NULL;
    int i, j, k, n = DL.NumberOfPoints();
    float dij, dik, djk;
    bool ConnectEdges;
    graph = CreateGraph(n, outdegree);

    for(i=0;i<n-1;i++)
        for(j=i+1;j<n;j++) {
            ConnectEdges = true;
            dij = (DL.pt(i) - DL.pt(j)).sqrabs();
            for(k=0;k<n;k++) {
                if(k == i || k == j) continue;
                dik = (DL.pt(i) - DL.pt(k)).sqrabs();
                djk = (DL.pt(j) - DL.pt(k)).sqrabs();
                if(dij >= (dik + djk))
                    ConnectEdges = false;
            }
            if(ConnectEdges)
                addEdge(graph, i, j, 1);
        }

    graph->Wnodes = GenerateWArray(graph, DL, sigma);

    return graph;
}*/


Graph *CreateGabrielGraphFromWnodes(int nnodes, int outdegree, float *Wnodes)
{
    Graph *graph = NULL;
    int i, j, k;
    float dij, dik, djk;
    bool ConnectEdges;
    graph = CreateGraph(nnodes, outdegree);
    graph->Wnodes = Wnodes;

    for(i=0;i<nnodes;i++)
        for(j=i+1;j<nnodes;j++) {
            ConnectEdges = true;
            dij = abs(Wnodes[i] - Wnodes[j]);
            for(k=0;k<nnodes;k++) {
                if(k == i || k == j) continue;
                dik = abs(Wnodes[i] - Wnodes[k]);
                djk = abs(Wnodes[i] - Wnodes[k]);
                if(dij >= dik + djk)
                    ConnectEdges = false;
            }
            if(ConnectEdges)
                addEdge(graph, i, j, 1);
        }

    return graph;
}

Graph *CreateKnnGabrielGraph(PSet::DataList DL, int K, float sigma){
    Graph *graph;
    int i, j, k, *bestj, m=0, kdmax=0, KMinDistSize;
    float d, *KMinDist, aux;
    bool AlreadyAdj;

    KMinDist = AllocFloatArray(K);
    bestj = AllocIntArray(K);
    graph = CreateGabrielGraph(DL, K, sigma);

    for(i=0;i<graph->nnodes;i++) {
            for(j=0;j<K;j++)
                KMinDist[j] = 0;
            KMinDistSize = 0;
            for(j=0;j<graph->nnodes;j++)
            {
                if(i == j) continue;

                d = (DL.pt(i) - DL.pt(j)).sqrabs();

                if(KMinDistSize < K) {
                    KMinDist[KMinDistSize] = d;
                    KMinDistSize++;
                    bestj[KMinDistSize] = j;
                    continue;
                }

                /*Neste ponto o vetor já está preenchido*/

                for(k=0;k<K;k++)
                    if(d < KMinDist[k])
                    {
                        aux = 0;
                        for(m=0;m<K;m++)
                            if(KMinDist[m] > aux) {
                                aux = KMinDist[m];
                                kdmax = m;
                            }
                        KMinDist[kdmax] = d;
                        bestj[kdmax] = j;
                        break;
                    }
            }
        for(k=0;k<K;k++)
        {
            AlreadyAdj = false;
            for(j=0;j<graph->nodes[i].outdegree;j++)
                if(graph->nodes[i].adjList[j] == bestj[k]) {
                    AlreadyAdj = true;
                    break;
                }
        if(!AlreadyAdj)
            addEdge(graph,i,bestj[k],1);
        }
    }

    graph->Wnodes = GenerateWArray(graph, DL, sigma);


    return graph;
}



int GetNodeClass(int *label, int *pred, int i)
{
    if(label[i] != -1)
        return label[i];
    else
        return GetNodeClass(label, pred, pred[i]);
}

int *GenerateLabelArray(int *pred, int nnodes){
    int *label;
    int i,l = 0;

    label = AllocIntArray(nnodes);

    for(i=0; i<nnodes; i++)
    {
        if(pred[i] == NIL) {
            label[i] = l;
            l++; }
        else label[i] = -1;
    }



    for(i=0; i<nnodes; i++)
            label[i] = GetNodeClass(label, pred, i);

    //for(i=0;i<nnodes;i++)
    //    printf("label[%d] = %d\n", i, label[i]);
    return label;


}

float ArcDensity(float Wa, float Wb, int type, Graph *graph)
{
    int i;
    float Wmax;
    if(type == AD_AVERAGE)
        return ((Wa+Wb)/2);
    else if(type == AD_MAX)
    {
        if(Wa > Wb) return Wa;
        else return Wb;
    }

    else if(type == ADN_AVERAGE)
    {
        Wmax = 0;
        for(i=0;i<graph->nnodes;i++)
            if(graph->Wnodes[i] > Wmax)
                Wmax = graph->Wnodes[i];
        return ((Wa+Wb)/(2*Wmax));
    }

    else if(type == ADN_MAX)
    {
        Wmax = 0;
        for(i=0;i<graph->nnodes;i++)
            if(graph->Wnodes[i] >Wmax)
                Wmax = graph->Wnodes[i];
        if(Wa > Wb) return Wa/Wmax;
        else return Wb/Wmax;
    }

    else if(type == AD_MIN)
    {
        if(Wa < Wb) return Wa;
        else return Wb;
    }

    else {
    printf("\nError in GFT ArcDensity - Invalid type\n");
    return (-1);
    }
}

float SubGraphDen_Sum(int r, Graph *graph, int *pred, int i){
  int *P;
  int topo = 0,a,b,k;
  float soma = 0;

  P = AllocIntArray(graph->nnodes);
  P[topo] = r;
  topo++;

  while(topo > 0){
    topo--;
    a = P[topo];

    soma += graph->Wnodes[a];

    for(k=0; k < graph->nodes[a].outdegree; k++){
      b = graph->nodes[a].adjList[k];
      if(a == pred[b] && b != i){
	P[topo] = b;
	topo++;
      }
    }
  }
  free(P);
  return soma;
}


float SubGraphDen_Max(int r, Graph *graph, int *pred, int i){
  int *P;
  int topo = 0,a,b,k;
  float max = 0;

  P = AllocIntArray(graph->nnodes);
  P[topo] = r;
  topo++;

  while(topo > 0){
    topo--;
    a = P[topo];

    if(graph->Wnodes[a] > max)
      max = graph->Wnodes[a];

    for(k=0; k < graph->nodes[a].outdegree; k++){
      b = graph->nodes[a].adjList[k];
      if(a == pred[b] && b != i){
	P[topo] = b;
	topo++;
      }
    }
  }
  free(P);
  return max;
}

float SubGraphDen_Sum2(int r, Graph *graph, int *pred, int i, float AD){
  int *P;
  int topo = 0,a,b,k;
  float soma = 0;

  P = AllocIntArray(graph->nnodes);
  P[topo] = r;
  topo++;

  while(topo > 0){
    topo--;
    a = P[topo];

    if(graph->Wnodes[a] > AD)
        soma += (graph->Wnodes[a] - AD);

    for(k=0; k < graph->nodes[a].outdegree; k++){
      b = graph->nodes[a].adjList[k];
      if(a == pred[b] && b != i){
	P[topo] = b;
	topo++;
      }
    }
  }
  free(P);
  return soma;
}


float SubGraphDensity(Graph *graph, int *pred, int i, SubGraphDen type)
{
    int k, r;
    float S, maxW, somai, somar, maxi, maxr;
    if(type == SGD_SUM) {
        somai = SubGraphDen_Sum(i, graph, pred, -1);
        r = i;
        while(pred[r] != NIL){
            r = pred[r];
        }
        somar = SubGraphDen_Sum(r, graph, pred, i);
        if(somai < somar)
            return somai;
        else
            return somar;
    }

    else if(type == SGD_MAX) {
        maxi = SubGraphDen_Max(i, graph, pred, -1);
        r = i;
        while(pred[r] != NIL){
            r = pred[r];
        }
        maxr = SubGraphDen_Max(r, graph, pred, i);
        if(maxi < maxr)
            return maxi;
        else
            return maxr;
    }

    else if(type == SGDN_SUM) {

        somai = SubGraphDen_Sum(i, graph, pred, -1);
        r = i;
        while(pred[r] != NIL){
            r = pred[r];
        }
        somar = SubGraphDen_Sum(r, graph, pred, i);

        for(k=0,S=0;k<graph->nnodes;k++)
            S += graph->Wnodes[k];

        if(somai < somar)
            return somai/S;
        else
            return somar/S;
    }

    else if(type == SGDN_MAX) {

        maxi = SubGraphDen_Max(i, graph, pred, -1);
        r = i;
        while(pred[r] != NIL){
            r = pred[r];
        }
        maxr = SubGraphDen_Max(r, graph, pred, i);
        for(k=0,maxW=0;k<graph->nnodes;k++)
            if(graph->Wnodes[i] > maxW)
                maxW = graph->Wnodes[i];

        if(maxi < maxr)
            return maxi/maxW;
        else
            return maxr/maxW;
    }

    else {
        printf("\nError in GFT SubGraphDensity - Invalid Type");
        return -1;
    }
}

float ArcWeight(float AD, float SGD, ArcW type, float *param)
{
    if(type == AW_DIVISION)
        return AD/SGD;
    else if(type == AW_LINEAR)
        return param[0]*AD - param[1]*SGD;
    else if(type == AW_EXP)
        return SGD*exp(-param[0]*AD);
    else {
        printf("\nError in GFT ArcWeight - %d is an invalid type", type);
        return (-1);
    }
}

float ArcWeight(Graph *graph, int *pred, int i, float *param, ArcDen ADtype, SubGraphDen SGDtype, ArcW AWtype)
{
    if(AWtype == AW_SUM)
    {
        int r;
        float somai, somar;

        somai = SubGraphDen_Sum2(i, graph, pred, -1, ArcDensity(graph->Wnodes[i],
                graph->Wnodes[pred[i]], ADtype, graph));
        r = i;
        while(pred[r] != NIL){
            r = pred[r];
        }
        somar = SubGraphDen_Sum2(r, graph, pred, i, ArcDensity(graph->Wnodes[i],
                graph->Wnodes[pred[i]], ADtype, graph));

        if(somai < somar)
            return -somai;
        else
            return -somar;
    }

    else
    return ArcWeight(
                    ArcDensity(graph->Wnodes[i], graph->Wnodes[pred[i]], ADtype, graph),
                    SubGraphDensity(graph, pred, i, SGDtype),
                    AWtype, param);
}

int *BestCut(Graph *graph, int *pred, int nclasses, float *param, ArcDen ADtype, SubGraphDen SGDtype, ArcW AWtype, bool CorrectWeights)
{
    int i, k, besti=0, *label, *cuts, *backup, it = 1;
    float W, Wmin = 0;
    bool FirstW, change;
    cuts = new int[nclasses - 1];
    backup = new int[nclasses - 1];

    for(k=0;k < nclasses - 1;k++)
        cuts[k] = -1;

    do
    {
        printf("GFT: Iteration %d\n", it);
        if(it > 1000)
        {
            printf("Iteration did not converge!\n");
            break;
        }

        change = false;
        for(k=0; k < nclasses-1; k++)
        {
            FirstW = true;
            if(cuts[k] != -1)
                pred[cuts[k]] = backup[k];

            for(i=0; i<graph->nnodes; i++)
            {
                if(pred[i] == NIL) continue;
                W = ArcWeight(graph, pred, i, param, ADtype, SGDtype, AWtype);         //Cuts (a,b) with minimum ArcWeight(Wa,Wb)
                if(W < Wmin || FirstW)
                {
                    Wmin = W;
                    besti = i;
                    FirstW = false;
                }
            }

            if(CorrectWeights)
            {
                graph->Wnodes[besti] *= ((float) graph->nodes[besti].outdegree)/(graph->nodes[besti].outdegree - 1);
                graph->Wnodes[pred[besti]] *= ((float) graph->nodes[pred[besti]].outdegree)/(graph->nodes[besti].outdegree - 1);
            }

            if(besti != cuts[k])
                change = true;

            backup[k] = pred[besti];
            cuts[k] = besti;
            pred[besti] = NIL;

        }

        it++;
    }
        while(change);



    //-------------------------------------------------------------//





    label = GenerateLabelArray(pred, graph->nnodes);

    return label;


}


int *GFT(Graph *graph, int seed){
  float *cost1 = NULL, *cost2 = NULL;
  int *pred = NULL;
  Heap_lex *Q;
  float c1,c2;
  int i,p,q;
  pred = AllocIntArray(graph->nnodes);
  cost1 = AllocFloatArray(graph->nnodes);
  cost2 = AllocFloatArray(graph->nnodes);
  for(i = 0; i < graph->nnodes; i++){
    pred[i] = NIL;
    cost1[i] = FLT_MIN;
    cost2[i] = FLT_MIN;
  }
  Q = CreateHeap_lex(graph->nnodes, cost1, cost2);
  SetRemovalPolicyHeap_lex(Q, MAXVALUE);

  cost1[seed] = graph->Wnodes[seed];
  cost2[seed] = graph->Wnodes[seed];
  InsertHeap_lex(Q, seed);

  while(!IsEmptyHeap_lex(Q)){
    RemoveHeap_lex(Q, &p);

    for(i=0; i < graph->nodes[p].outdegree; i++){
      q = graph->nodes[p].adjList[i];
      if(Q->color[q] != BLACK){
	c1 = graph->Wnodes[q];
	c2 = graph->Wnodes[p];
	if(c1 > cost1[q] ||
	   (c1 == cost1[q] && c2 > cost2[q])){
	  UpdateHeap_lex(Q, q, c1, c2);
	  pred[q] = p;
	}
      }
    }
  }
  free(cost1);
  free(cost2);
  DestroyHeap_lex(&Q);
  return pred;
}


int *GFT_Clustering(Graph *graph,
		    int nclasses, float *param, ArcDen ADtype, SubGraphDen SGDtype, ArcW AWtype, bool CorrectWeights){
  int *label = NULL;
  int *pred = NULL;
  float Wmax;
  int seed,i;

  seed = 0;
  Wmax = graph->Wnodes[0];
  for(i = 1; i < graph->nnodes; i++){
    if(graph->Wnodes[i] > Wmax){
      Wmax = graph->Wnodes[i];
      seed = i;
    }
  }

  pred = GFT(graph, seed);

  /*for(i=0;i<graph->nnodes;i++)
    printf("pred[%d] = %d\n", i, pred[i]);*/

/*
    FILE *file;
    file = fopen("log.txt","w");

    for(i=0; i<graph->nnodes; i++)
        fprintf(file, "%f ", graph->Wnodes[i]);
    fprintf(file, "\n");
    for(i=0;i<graph->nnodes;i++)
        fprintf(file, "%d ", pred[i]);
    fclose(file);
*/

  /*procura aresta de remocao que minimiza energia:*/
  label = BestCut(graph, pred, nclasses, param, ADtype, SGDtype, AWtype, CorrectWeights);

  return label;
}

float CalculateOPFsqrsigma(PSet::DataList *DL, Graph *graph)
{
    int i, j;
    float sqrd, sqrdmax = 0;
    for(i=0;i<graph->nnodes;i++)
        for(j=0;j<graph->nodes[i].outdegree;j++)
        {
            sqrd = (DL->pt(i) - DL->pt(graph->nodes[j].adjList[j])).sqrabs();
            if(sqrd > sqrdmax)
                sqrdmax = sqrd;
        }

    return sqrdmax/9;
}


void GFT_Clustering(PSet::DataList *DL, int outdegree, int nclasses, float *param, ArcDen ADtype, SubGraphDen SGDtype, ArcW AWtype,
                    GraphType Gtype, float sigma, bool CorrectWeights, bool OPFsigma)
{
    int i, n;
    Graph *g;
    int *label;
    label = NULL;
    n = DL->NumberOfPoints();
    if(Gtype == GABRIEL_GRAPH)
        g = CreateGabrielGraph(*DL, outdegree, sigma);
    else if(Gtype == GABRIEL_KNN_GRAPH)
        g = CreateKnnGabrielGraph(*DL, outdegree, sigma);
    if(OPFsigma)
        g->Wnodes = GenerateWArray(g, *DL, CalculateOPFsqrsigma(DL, g));
    label = GFT_Clustering(g, nclasses, param, ADtype, SGDtype, AWtype, CorrectWeights);
    for(i=0;i<n;i++) {
        DL->pt(i).classifcl = label[i];
        //printf("%f\n", g->Wnodes[i]);
    }



    free(label);
    DestroyGraph(&g);
}


void PrintGraph(Graph *graph)
{
    int i, j;
    for(i=0;i<graph->nnodes;i++) {
        printf("%d: ", i);
        for(j=0;j<graph->nodes[i].outdegree;j++)
            printf("%d ", graph->nodes[i].adjList[j]);
        printf("| %d %d\n", graph->nodes[i].outdegree, graph->nodes[i].arraysize);
    }
}

int main(int argc, char* argv[])    //argv[1] = input file name
                                    //argv[2] = output file name
                                    //argv[3] = Number Of Classes
                                    //argv[4] = outdegree
                                    //argv[5] = param[0]
                                    //argv[6] = param[1]
                                    //argv[7] = ArcDen
                                    //argv[8] = SubGraphDen
                                    //argv[9] = ArcW
                                    //argv[10] = GraphType
                                    //argv[11] = sigma
                                    //argv[12] = CorrectWeights (boolean)
                                    //argv[13] = OPFsigma (boolean) - Choose true to use sigma like in OPF.
{
  printf("\nGFT: Running...");
  float param[2];
  param[0] = atof(argv[5]); param[1] = atof(argv[6]);

  PSet::DataList DL;
  DL.ImportFile(argv[1],PSet::SIMPLESTD);
  GFT_Clustering(&DL, atoi(argv[4]) , atoi(argv[3]), param, (ArcDen) atoi(argv[7]), (SubGraphDen) atoi(argv[8]), (ArcW) atoi(argv[9]),
                (GraphType) atoi(argv[10]), atof(argv[11]), (bool) atoi(argv[12]), (bool) atoi(argv[13]));
  DL.ExportFile(argv[2], PSet::SIMPLESTD);
  printf("\nGFT: Done!\n", atoi(argv[3]));
  return 0;
}
