
#include "common.h"
#include "image.h"
#include "image64.h"
#include "cimage.h"
#include "adjacency.h"
#include "morphology.h"
#include "ift.h"
#include "filelist.h"
#include "highlight.h"


/*Calcula imagem da soma das diferencas 
de intensidade entre fatias consecutivas.*/
Image *ComputeAccDiff(FileList *FL){
  Image *T,*I,*D;
  int fi, p;

  fi = 0;
  T = ReadImage(GetFile(FL, fi));
  D = CloneImage(T);
  SetImage(D, 0);
  //printf("%s\n", GetFile(FL, fi));
  for(fi = 1; fi < FL->n; fi++){
    //printf("%s\n", GetFile(FL, fi));
    I = ReadImage(GetFile(FL, fi));
    for(p=0; p<D->ncols*D->nrows; p++){
      D->data[p] += abs(I->data[p] - T->data[p]);
    }
    DestroyImage(&T);
    T = I;
  }
  DestroyImage(&T);
  return D;
}


/*A imagem acumulada das intensidades de todas as fatias.*/
Image *ComputeAccSum(FileList *FL){
  Image *I,*S;
  int fi, p;

  fi = 0;
  S = ReadImage(GetFile(FL, fi));
  for(fi = 1; fi < FL->n; fi++){
    I = ReadImage(GetFile(FL, fi));
    for(p=0; p<S->ncols*S->nrows; p++){
      S->data[p] += I->data[p];
    }
    DestroyImage(&I);
  }
  return S;
}


Image *ComputeSeedsByFabio(Image *AccDiff, Image *AccSum){
  Image *sum_min,*sum_max,*diff_max,*dil_diff_max;
  Image *S;
  AdjRel *A = Circular(4.0);
  int Smax = MaximumValue(AccSum);
  int Dmax = MaximumValue(AccDiff);
  int p;
  sum_min  = Threshold(AccSum, 0, (int)(0.10*Smax + 0.5));
  sum_max  = Threshold(AccSum, (int)(0.70*Smax + 0.5), Smax);
  diff_max = Threshold(AccDiff, (int)(0.70*Dmax + 0.5), Dmax);
  dil_diff_max = Dilate(diff_max, A);

  WriteImage(sum_min,  "sum_min.pgm");
  WriteImage(sum_max,  "sum_max.pgm");
  WriteImage(diff_max, "diff_max.pgm");
  WriteImage(dil_diff_max, "dil_diff_max.pgm");

  S = CloneImage(AccDiff);
  SetImage(S, 0);
  for(p=0; p<S->ncols*S->nrows; p++){
    if(dil_diff_max->data[p] > 0 && sum_min->data[p] > 0)
      S->data[p] = 2;
    if(sum_max->data[p] > 0)
      S->data[p] = 1;
  }

  DestroyAdjRel(&A);
  DestroyImage(&sum_min);
  DestroyImage(&sum_max);
  DestroyImage(&diff_max);
  DestroyImage(&dil_diff_max);
  return S;
}


int *GetArrayOfPixelsInSubImage(Image *img,
				int x1, int y1,
				int x2, int y2){
  int i,j,p,n,k;
  int *A;
  if(y1 < 0) y1 = 0;
  if(x1 < 0) x1 = 0;
  if(y2 >= img->nrows) y2 = img->nrows-1;
  if(x2 >= img->ncols) x2 = img->ncols-1;

  n = (x2-x1+1)*(y2-y1+1);
  A = (int *)malloc(sizeof(int)*(n+1));
  A[0] = n;
  k = 1;
  for(i=y1; i<=y2; i++){
    for(j=x1; j<=x2; j++){
      p = j + i*img->ncols;
      A[k] = p;
      k++;
    }
  }
  return A;
}


void SortArrayOfPixels(Image *img, int *A){
  int i,j,m,n,tmp;
  n = A[0];
  for(i = n; i > 1; i--) {
    /* Coloca em dados[i] o maximo de [1,i]. */
    m = 1;
    for(j = 2; j <= i; j++) {
      if(img->data[A[j]] > img->data[A[m]])
	m = j;
    }
    tmp = A[i];
    A[i] = A[m];
    A[m] = tmp;
  }
}

/* Finds the best plane of symmetry. */
int Bisection(Image *I){
  int bx,i,j,jj,bx_min = 0;
  int p,pp;
  float diff,diff_min = FLT_MAX;
  for(bx=0; bx<I->ncols; bx++){
    diff = 0.0;
    for(i=0; i<I->nrows; i++){
      for(j=0; j<I->ncols; j++){
	p = j+i*I->ncols;
	jj = bx - (j-bx);
	if(ValidPixel(I, jj, i)){
	  pp = jj+i*I->ncols;
	  diff += abs(I->data[p]-I->data[pp]); 
	}
	else
	  diff += I->data[p];
      }
    }
    if(diff < diff_min){
      diff_min = diff;
      bx_min = bx;
    }
  }
  return bx_min;
}


Image *ComputeSeeds(Image *AccDiff, Image *img){
  Image *C,*S;
  int p,q, tam_x, tam_y, ncols, nrows;
  int px,py,qx,qy,dd,ddmax;
  int r,l,i,bx;
  int *Ar,*Al,*Ac;
  nrows = AccDiff->nrows;
  ncols = AccDiff->ncols;
  C = CreateImage(ncols, nrows);
  S = CreateImage(ncols, nrows);

  tam_x = (int)(ncols*0.35 + 0.5);
  tam_y = tam_x/2;
  p = GetPosWindowMaxSum(AccDiff, tam_x, tam_y);
  DrawRectangle(C, p%ncols, p/ncols, p%ncols+tam_x-1, p/ncols+tam_y-1, 128);
  p += (ncols*(tam_y/2) + tam_x/2);
  px = p%ncols;
  py = p/ncols;
  C->data[p] = 255;
  //WriteImage(C, "C.pgm");
  
  SetImage(S, 0);
  ddmax = (int)(ncols*0.45 + 0.5);
  ddmax = ddmax*ddmax;
  for(q=0; q<ncols*nrows; q++){
    qx = q%ncols;
    qy = q/ncols;
    dd = (qx-px)*(qx-px) + (qy-py)*(qy-py);
    if(dd >= ddmax)
      S->data[q] = 1;
  }

  l = p - tam_x/2;
  r = p + tam_x/2;

  Al = GetArrayOfPixelsInSubImage(img,
				  l%ncols - tam_x/4, l/ncols - tam_x/3,
				  l%ncols + tam_x/4, l/ncols + tam_x/3);
  Ar = GetArrayOfPixelsInSubImage(img,
				  r%ncols - tam_x/4, r/ncols - tam_x/3,
				  r%ncols + tam_x/4, r/ncols + tam_x/3);
  SortArrayOfPixels(img, Al);
  SortArrayOfPixels(img, Ar);

  for(i=1; i<=Al[0]/4; i++){
    S->data[Al[i]] = 2;
  }
  for(i=1; i<=Ar[0]/4; i++){
    S->data[Ar[i]] = 2;
  }

  Ac = GetArrayOfPixelsInSubImage(img,
				  px - tam_x/4, py - tam_x/4,
				  px + tam_x/4, py + tam_x/4);
  SortArrayOfPixels(img, Ac);

  for(i=Ac[0]; i >= (3*Ac[0])/4; i--){
    S->data[Ac[i]] = 1;
  }

  bx = Bisection(img);
  for(i=py; i >= 0; i--){
    S->data[bx+i*ncols] = 1;
  }

  free(Ac);
  free(Al);
  free(Ar);
  DestroyImage(&C);
  return S;
}



/*Converte imagem de label para um vetor
contendo a posicao de todos os pixels com
valor igual a lb. A posicao zero do vetor
guarda o numero de elementos do vetor.*/
int *LabelImage2Seeds(Image *label, int lb){
  int *S;
  int p,cont,i;
  cont = 0;
  for(p=0; p<label->ncols*label->nrows; p++){
    if(label->data[p] == lb)
      cont++;
  }
  S = (int *)malloc(sizeof(int)*(cont+1));
  S[0] = cont;
  i = 1;
  for(p=0; p<label->ncols*label->nrows; p++){
    if(label->data[p] == lb){
      S[i] = p;
      i++;
    }
  }
  return S;
}

void SubFromImage(Image *I, int val){
  int p;
  for(p=0; p<I->ncols*I->nrows; p++){
    I->data[p] -= val;
  }
}


Image *ComputeWeightImage(Image *img, AdjRel *A){
  Image *W;
  int p,q,i,sum;
  int p_x,p_y,q_x,q_y;
  W = CloneImage(img);
  for(p=0; p<img->n; p++){
    p_x = p%img->ncols;
    p_y = p/img->ncols;
    sum = 0;
    for(i=1; i<A->n; i++){
      q_x = p_x + A->dx[i];
      q_y = p_y + A->dy[i];
      if(ValidPixel(img, q_x, q_y)){
	q = q_x + q_y*img->ncols;
	sum += abs(img->data[p] - img->data[q]);
      }
    }
    W->data[p] = sum;
  }
  return W;
}


Image *BinModeFilter(Image *bin, AdjRel *A){
  Image *B;
  int p,q,i,cont0,cont1;
  int p_x,p_y,q_x,q_y;
  B = CloneImage(bin);
  for(p=0; p<bin->n; p++){
    p_x = p%bin->ncols;
    p_y = p/bin->ncols;
    cont0 = 0;
    cont1 = 0;
    for(i=0; i<A->n; i++){
      q_x = p_x + A->dx[i];
      q_y = p_y + A->dy[i];
      if(ValidPixel(bin, q_x, q_y)){
	q = q_x + q_y*bin->ncols;
	if(bin->data[q] > 0)
	  cont1++;
	else
	  cont0++;
      }
    }
    if(cont1 > cont0)
      B->data[p] = 1;
    else if(cont0 > cont1)
      B->data[p] = 0;
  }
  return B;
}



int main(){
  Image *I,*G,*L,*Lf,*Lfh,*AccDiff,*AccSum;
  AdjRel *A;
  int *Si=NULL,*Se=NULL;
  float alpha,beta;
  int isHigh2Low;
  SparseGraph *sg,*dsg;
  Image *P_sum;
  char filename[512];
  char dir[512];
  FileList *FL;
  int fi;

  printf("Enter the shape constraints control parameter (beta):\n");
  printf("\tbeta = 0.0 (Star Convexity) or higher e.g., 4.0 (Geodesic Star Convexity)\n");
  scanf("%f", &beta);

  printf("Enter the oriented edges strength (alpha):\n");
  printf("\talpha = -1.0 (extremely oriented (dark to bright)),\n");
  printf("\talpha =  0.0 (non-oriented),\n");
  printf("\talpha =  1.0 (extremely oriented (bright to dark)).\n");
  scanf("%f", &alpha);
  isHigh2Low = MAX(SIGN(alpha), 0);
  alpha = fabsf(alpha);

  printf("Enter the input directory: ");
  scanf("%s", dir);

  FL = CreateFileList(30);
  AddFilesInDir(FL, dir);

  AccDiff = ComputeAccDiff(FL);
  AccSum  = ComputeAccSum(FL);

  WriteImage(AccDiff, "accdiff.pgm");
  WriteImage(AccSum,  "accsum.pgm");

  A = Circular(1.5);

  for(fi = 0; fi < FL->n; fi++){
    I = ReadImage(GetFile(FL, fi));
    //G = MorphGrad(I, A);
    G = ComputeWeightImage(I, A);

    L = ComputeSeeds(AccDiff, I);
    sprintf(filename,"./out/seeds_%03d.pgm",fi+1);
    WriteImage(L, filename);

    SubFromImage(L, 1);
    Si = LabelImage2Seeds(L, 1);
    Se = LabelImage2Seeds(L, 0);

    sg = WeightImage2SparseGraph(G, 1.5);
    ChangeSparseGraphType(sg, DISSIMILARITY);
    dsg = CloneSparseGraph(sg);
    Orient2Digraph(dsg, I, alpha*100.0);
    SuppressZeroWeightedArcs(dsg);

    if(!isHigh2Low){
      InvertArcsDigraph(dsg);
      P_sum = pred_IFTSUM(dsg, Si, NULL, L, beta);
      InvertArcsDigraph(dsg);
    }
    else{
      P_sum = pred_IFTSUM(dsg, Si, NULL, L, beta);
    }
    SetImage(L, NIL);

    if(isHigh2Low){
      Orient2DigraphOuter(dsg, P_sum);
      method_IFTW_FIFO_OuterCut(dsg, Si, Se, L);
    }
    else{
      Orient2DigraphInner(dsg, P_sum);
      method_IFTW_FIFO_InnerCut(dsg, Si, Se, L);
    }

    Lf = BinModeFilter(L, A);
    sprintf(filename,"./out/bin_%03d.pgm",fi+1);
    WriteImage(Lf, filename);

    Lfh = WideHighlight(I, Lf, 2.0, MaximumValue(I), false);
    sprintf(filename,"./out/seg_%03d.pgm",fi+1);
    WriteImage(Lfh, filename);

    DestroySparseGraph(&dsg);
    DestroySparseGraph(&sg);
    DestroyImage(&P_sum);
    DestroyImage(&I);
    DestroyImage(&G);
    DestroyImage(&L);
    DestroyImage(&Lf);
    DestroyImage(&Lfh);
    free(Si);
    free(Se);
  }

  DestroyAdjRel(&A);
  DestroyImage(&AccDiff);
  DestroyImage(&AccSum);
  DestroyFileList(&FL);

  return 0;
}

