
#include "common.h"
#include "image.h"
#include "cimage.h"
#include "adjacency.h"
#include "queue.h"
#include "gpqueue_by_Falcao.h"
#include "morphology.h"

/* Devolve o representante de p */
int Find(Image *R, int p){
  if(R->data[p] == p)
    return p;
  else{
    R->data[p] = Find(R, R->data[p]);
    return R->data[p];
  }
}


/*Removes all background connected components from the stack
of binary images of I whose area (number of pixels) is less than 
a threshold and outputs a simplified image.*/
Image *AreaClosing(AdjRel *A, Image *I, int T){
  Image *J,*V;
  Image *Ar; /* Areas para os representantes */
  Image *R; /* Imagem de representantes */
  Image *P; /* Imagem de predecessores */
  GQueue *Q;
  int p,q,n,Imax,rp,rq,tmp;
  int xp,yp,xq,yq,i;
  //Image *teste;

  Imax = MaximumValue(I);
  n  = I->ncols*I->nrows;
  V  = CreateImage(I->ncols, I->nrows);
  J  = CreateImage(I->ncols, I->nrows);
  Ar = CreateImage(I->ncols, I->nrows);
  R  = CreateImage(I->ncols, I->nrows);
  P  = CreateImage(I->ncols, I->nrows);

  /*teste  = CreateImage(I->ncols, I->nrows);*/

  Q  = CreateGQueue(Imax+2, n, V->data);
  SetTieBreak(Q, LIFOBREAK);

  for(p = 0; p < n; p++){
    P->data[p] = NIL;
    V->data[p] = I->data[p] + 1;
    R->data[p] = p;
    Ar->data[p] = 0;
    J->data[p] = I->data[p];
    InsertGQueue(&Q, p);
  }

  while(!EmptyGQueue(Q)){
    p = RemoveGQueue(Q);
    rp = Find(R, p);

    if(Ar->data[rp] <= T && J->data[rp] < I->data[p])
      J->data[rp] = I->data[p];

    Ar->data[rp]++;
    xp = p%I->ncols;
    yp = p/I->ncols;
    for(i = 1; i < A->n; i++){
      xq = xp + A->dx[i];
      yq = yp + A->dy[i];
      if(ValidPixel(I, xq, yq)){
	q = xq + yq*I->ncols;
	/*
	if(V->data[q] > V->data[p]){
	*/
	if(Q->L.elem[q].color != BLACK){
	  tmp = MAX(V->data[p], I->data[q]);
	  if(tmp <= V->data[q]){
	    RemoveGQueueElem(Q, q);
	    V->data[q] = tmp;
	    P->data[q] = p;
	    R->data[q] = rp;
	    InsertGQueue(&Q, q);
	  }
	}
	else{/* if(Q->L.elem[q].color != GRAY){*/
	  rq = Find(R, q);
	  if(rp != rq){
	    if(Ar->data[rq] <= T && J->data[rq] < I->data[p])
	      J->data[rq] = I->data[p];
	    if(Ar->data[rp] < Ar->data[rq]){
	      tmp = rp;
	      rp = rq;
	      rq = tmp;
	    }
	    R->data[rq] = rp;
	    Ar->data[rp] += Ar->data[rq];
	    /*
	    if(P->data[p] == NIL && P->data[q] == NIL)
	      P->data[rq] = rp;
	    */
	  }
	}
      }
    }
  }
  ResetGQueue(Q);
  SetRemovalPolicy(Q, MAXVALUE);

  for(p = 0; p < n; p++){
    V->data[p] = J->data[p];
    if(P->data[p] == NIL){
      InsertGQueue(&Q, p);
      //teste->data[p] = 1;
    }
  }
  while(!EmptyGQueue(Q)){
    p = RemoveGQueue(Q);    
    xp = p%I->ncols;
    yp = p/I->ncols;
    for(i = 1; i < A->n; i++){
      xq = xp + A->dx[i];
      yq = yp + A->dy[i];
      if(ValidPixel(I, xq, yq)){
	q = xq + yq*I->ncols;
	if(V->data[p] > V->data[q]){
	  if(Q->L.elem[q].color == GRAY)
	    RemoveGQueueElem(Q, q);
	  V->data[q] = V->data[p];
	  InsertGQueue(&Q, q);
	}
      }
    }
  }

  //WriteImage(teste, "teste.pgm");

  DestroyImage(&J);
  DestroyImage(&R);
  DestroyImage(&P);
  DestroyImage(&Ar);
  DestroyGQueue(&Q);
  return V;
}



int main(){
  Image *I,*AC;
  AdjRel *A;
  int T;

  A = Circular(1.0);
  printf("Entre com o valor de T: ");
  scanf("%d",&T);
  I = ReadImage("./dat/cheese.pgm");

  AC = AreaClosing(A, I, T);

  WriteImage(AC, "AC.pgm");

  DestroyImage(&I);
  DestroyImage(&AC);
  DestroyAdjRel(&A);

  return 0;
}

