
#include "common.h"
#include "image.h"
#include "cimage.h"
#include "adjacency.h"
#include "morphology.h"
#include "ift.h"


int *LabelImage2Seeds(Image *label, int lb){
  int *S;
  int p,cont,i;
  cont = 0;
  for(p=0; p<label->ncols*label->nrows; p++){
    if(label->data[p] == lb)
      cont++;
  }
  S = (int *)malloc(sizeof(int)*(cont+1));
  S[0] = cont;
  i = 1;
  for(p=0; p<label->ncols*label->nrows; p++){
    if(label->data[p] == lb){
      S[i] = p;
      i++;
    }
  }
  return S;
}

void SubFromImage(Image *I, int val){
  int p;
  for(p=0; p<I->ncols*I->nrows; p++){
    I->data[p] -= val;
  }
}


int main(){
  Image *I,*G,*L;
  AdjRel *A;
  int *Si=NULL,*Se=NULL;
  float alpha,beta;
  int isHigh2Low;
  SparseGraph *sg,*dsg;
  Image *P_sum; 
  char filename[512];

  printf("Enter the shape constraints control parameter (beta):\n");
  printf("\tbeta = 0.0 (Star Convexity) or higher e.g., 4.0 (Geodesic Star Convexity)\n");
  scanf("%f", &beta);

  printf("Enter the oriented edges strength (alpha):\n");
  printf("\talpha = -1.0 (extremely oriented (dark to bright)),\n");
  printf("\talpha =  0.0 (non-oriented),\n");
  printf("\talpha =  1.0 (extremely oriented (bright to dark)).\n");
  scanf("%f", &alpha);
  isHigh2Low = MAX(SIGN(alpha), 0);
  alpha = fabsf(alpha);

  printf("Enter the image file name: ");
  scanf("%s", filename);
  A = Circular(1.5);
  I = ReadImage(filename);
  G = MorphGrad(I, A);

  printf("Enter the seeds file name: ");
  scanf("%s", filename);
  L = ReadImage(filename);
  SubFromImage(L, 1);
  Si = LabelImage2Seeds(L, 1);
  Se = LabelImage2Seeds(L, 0);

  sg = WeightImage2SparseGraph(G, 1.5);

  ChangeSparseGraphType(sg, DISSIMILARITY);
  dsg = CloneSparseGraph(sg);

  Orient2Digraph(dsg, I, alpha*100.0);
  SuppressZeroWeightedArcs(dsg);

  if(!isHigh2Low){
    InvertArcsDigraph(dsg);
    P_sum = pred_IFTSUM(dsg, Si, NULL, L, beta);
    InvertArcsDigraph(dsg);
  }
  else{
    P_sum = pred_IFTSUM(dsg, Si, NULL, L, beta);
  }
  SetImage(L, NIL);

  if(isHigh2Low){
    Orient2DigraphOuter(dsg, P_sum);
    method_IFTW_FIFO_OuterCut(dsg, Si, Se, L);
  }
  else{
    Orient2DigraphInner(dsg, P_sum);
    method_IFTW_FIFO_InnerCut(dsg, Si, Se, L);
  }

  WriteImage(L, "L.pgm");

  DestroySparseGraph(&dsg);
  DestroySparseGraph(&sg);
  DestroyImage(&P_sum);
  DestroyImage(&I);
  DestroyImage(&G);
  DestroyImage(&L);
  DestroyAdjRel(&A);
  free(Si);
  free(Se);

  return 0;
}

