
#include "common.h"
#include "image.h"
#include "cimage.h"
#include "adjacency.h"
#include "morphology.h"
#include "ift.h"
#include "color.h"

#include <time.h>


Image *GetLabelFromCyclicEDT(Image *bin, float r){
  Image *Dx=NULL, *Dy=NULL, *cost, *label;
  PriorityQueue *Q=NULL;
  int i,p,q,u_x,u_y,v_x,v_y,dx,dy,tmp,lb;
  AdjRel *A;

  A = Circular(r);
  Dx = CreateImage(bin->ncols, bin->nrows);
  Dy = CreateImage(bin->ncols, bin->nrows);
  cost  = CreateImage(bin->ncols, bin->nrows);
  label = CreateImage(bin->ncols, bin->nrows);
  Q = CreatePQueue(bin->ncols+bin->nrows, bin->n, cost->data);

  lb = 0;
  for(p = 0; p < bin->n; p++){
    if(bin->data[p] > 0){
      lb++;
      label->data[p] = lb;
      cost->data[p] = 0;
      InsertElemPQueue(&Q, p);
    }
    else
      cost->data[p] = INT_MAX;
  }
  while(!IsEmptyPQueue(Q)) {
    p = RemoveMinFIFOPQueue(Q);
    u_x = p%bin->ncols;
    u_y = p/bin->ncols;
    for(i=1; i<A->n; i++){
      v_x = u_x + A->dx[i];
      v_y = u_y + A->dy[i];
      dx  = Dx->data[p] + abs(v_x - u_x);
      dy  = Dy->data[p] + abs(v_y - u_y);
      if(!ValidPixel(bin,v_x,v_y)){
	if(v_x >= bin->ncols) v_x %= bin->ncols;
	if(v_y >= bin->nrows) v_y %= bin->nrows;
	if(v_x < 0) v_x += bin->ncols;
	if(v_y < 0) v_y += bin->nrows;
      }
      q = v_x + bin->ncols*v_y;

      if(cost->data[p] < cost->data[q]){
	tmp = dx*dx + dy*dy;
	if (tmp < cost->data[q]){
	  if(Q->L.elem[q].color == GRAY)
	    RemoveElemPQueue(Q, q);
	  label->data[q] = label->data[p];
	  cost->data[q] = tmp;
	  Dx->data[q] = dx;
	  Dy->data[q] = dy;
	  InsertElemPQueue(&Q, q);
	}
      }
    }
  }
  DestroyPQueue(&Q);
  DestroyAdjRel(&A);
  DestroyImage(&Dx);
  DestroyImage(&Dy);
  DestroyImage(&cost);
  return label;
}


void CyclicDrawAdjRelCImage(CImage *cimg, AdjRel *A, 
			    int x, int y,int color){
  int r,g,b,i,u,v;

  r = CHANNEL0(color);
  g = CHANNEL1(color);
  b = CHANNEL2(color);
  for(i = 0; i < A->n; i++){
    u = x + A->dx[i];
    v = y + A->dy[i];
    if(!ValidPixel(cimg->C[0],u,v)){
      if(u >= cimg->C[0]->ncols) u %= cimg->C[0]->ncols;
      if(v >= cimg->C[0]->nrows) v %= cimg->C[0]->nrows;
      if(u < 0) u += cimg->C[0]->ncols;
      if(v < 0) v += cimg->C[0]->nrows;
    }
    cimg->C[0]->array[v][u] = r;
    cimg->C[1]->array[v][u] = g;
    cimg->C[2]->array[v][u] = b;
  }
}


void CyclicDrawLineCImage(CImage *cimg,
			  int x1, int y1,
			  int x2, int y2,
			  int color){
  float x,y,dx,dy;
  int xi,yi,xs,ys;
  int r,g,b;
  r = CHANNEL0(color);
  g = CHANNEL1(color);
  b = CHANNEL2(color);
  dx = (x2-x1);
  dy = (y2-y1);
  if(fabsf(dx) > fabsf(dy)){
    dy = dy/(fabsf(dx));
    dx = dx/(fabsf(dx));
  }
  else{
    dx = dx/(fabsf(dy));
    dy = dy/(fabsf(dy));
  }
  x = (float)x1;
  y = (float)y1;
  xi = x1;
  yi = y1;
  while(xi != x2 || yi != y2){
    xs = xi;
    ys = yi;
    if(xs >= cimg->C[0]->ncols) xs %= cimg->C[0]->ncols;
    if(ys >= cimg->C[0]->nrows) ys %= cimg->C[0]->nrows;
    if(xs < 0) xs += cimg->C[0]->ncols;
    if(ys < 0) ys += cimg->C[0]->nrows;
    cimg->C[0]->array[ys][xs] = r;
    cimg->C[1]->array[ys][xs] = g;
    cimg->C[2]->array[ys][xs] = b;
    x += dx;
    y += dy;
    xi = ROUND(x);
    yi = ROUND(y);
  }
  xs = xi;
  ys = yi;
  if(xs >= cimg->C[0]->ncols) xs %= cimg->C[0]->ncols;
  if(ys >= cimg->C[0]->nrows) ys %= cimg->C[0]->nrows;
  if(xs < 0) xs += cimg->C[0]->ncols;
  if(ys < 0) ys += cimg->C[0]->nrows;
  cimg->C[0]->array[ys][xs] = r;
  cimg->C[1]->array[ys][xs] = g;
  cimg->C[2]->array[ys][xs] = b;
}



int main(){
  CImage *cimg;
  Image *bin, *label;
  AdjRel *A;
  int nrows,ncols,nreg;
  int x,y,p,q,i,j,u,v,uu,vv,d,dmin;
  int px,py,qx,qy,rx,ry,qqx,qqy,ppx,ppy;
  int *Label2Point;

  printf("Enter the image size in X: ");
  scanf("%d", &ncols);
  printf("Enter the image size in Y: ");
  scanf("%d", &nrows);
  printf("Enter the number of regions: ");
  scanf("%d", &nreg);
 
  bin = CreateImage(ncols, nrows);
  SetImage(bin, 0);

  srand(time(NULL));
  for(i = 1; i <= nreg; i++){
    x = rand()%ncols;
    y = rand()%nrows;
    bin->array[y][x] = 1;
  }
  /*WriteImage(bin, "bin.pgm");*/
  label = GetLabelFromCyclicEDT(bin, 1.5);
  cimg = ColorizeLabel(label);

  Label2Point = (int *)malloc(sizeof(int)*(nreg+1));
  for(p = 0; p < bin->n; p++){
    if(bin->data[p] > 0){
      Label2Point[label->data[p]] = p;
    }
  }

  /*WriteImage(label, "label.pgm");*/
  SetImage(cimg->C[0], 0);
  A = Circular(4.5);
  for(y = 0; y < nrows; y++){
    for(x = 0; x < ncols; x++){
      if(bin->array[y][x] > 0)
	CyclicDrawAdjRelCImage(cimg, A, x, y, 0x000000);
    }
  }
  DestroyAdjRel(&A);

  A = Circular(1.5);
  for(y = 0; y < nrows; y++){
    for(x = 0; x < ncols; x++){
      for(i = 1; i < A->n; i++){
	u = x + A->dx[i];
	v = y + A->dy[i];
	uu = u; vv = v;
	if(u >= ncols) u %= ncols;
	if(v >= nrows) v %= nrows;
	if(u < 0) u += ncols;
	if(v < 0) v += nrows;

	if(label->array[y][x] != label->array[v][u]){
	  p = Label2Point[label->array[y][x]];
	  q = Label2Point[label->array[v][u]];
	  //ClosestCyclicPoint(p,q,ncols,nrows);
	  px = p%ncols;
	  py = p/ncols;
	  qx = q%ncols;
	  qy = q/ncols;

	  dmin = INT_MAX;
	  qqx = qx; qqy = qy;
	  for(j = 0; j < A->n; j++){
	    rx = qx + A->dx[j]*ncols;
	    ry = qy + A->dy[j]*nrows;
	    d = (uu-rx)*(uu-rx) + (vv-ry)*(vv-ry);
	    if(d < dmin){
	      dmin = d;
	      qqx = rx;
	      qqy = ry;
	    }
	  }
	  //----------------------------------
	  dmin = INT_MAX;
	  ppx = px; ppy = py;
	  for(j = 0; j < A->n; j++){
	    rx = px + A->dx[j]*ncols;
	    ry = py + A->dy[j]*nrows;
	    d = (x-rx)*(x-rx) + (y-ry)*(y-ry);
	    if(d < dmin){
	      dmin = d;
	      ppx = rx;
	      ppy = ry;
	    }
	  }
	  //----------------------------------
	  CyclicDrawLineCImage(cimg, ppx, ppy, qqx, qqy, 0x000000);
	}
      }
    }
  }
  DestroyAdjRel(&A);

  WriteCImage(cimg, "delaunay.ppm");

  free(Label2Point);
  DestroyImage(&label);
  DestroyImage(&bin);
  DestroyCImage(&cimg);

  return 0;
}
