
#ifndef _SPARSEGRAPH_H_
#define _SPARSEGRAPH_H_

#include "common.h"
#include "image.h"
#include "cimage.h"
#include "adjacency.h"

#define DISSIMILARITY 0
#define CAPACITY      1

// Image Sparse Graph:
// n-links connect pairs of neighboring pixels.
typedef struct _sparseGraph {
  int type;
  int Wmax;
  int **n_link;
  int ncols, nrows;
  AdjRel *A;
} SparseGraph;


SparseGraph *AbsDiff2SparseGraph(Image *img, float r);
SparseGraph *WeightImage2SparseGraph(Image *W, float r);

//------ SparseGraph Functions ------------

int            get_edge_index(int p, int q, 
			      SparseGraph *g);

void           DestroySparseGraph(SparseGraph **g);
SparseGraph   *CreateSparseGraph(int ncols, int nrows, 
				 AdjRel *A);
SparseGraph   *CloneSparseGraph(SparseGraph *g);

void           ChangeSparseGraphType(SparseGraph *g, 
				     int type);

void           PowSparseGraph(SparseGraph *sg, int power, int max);
SparseGraph   *Convert2SymmetricSparseGraph(SparseGraph *g,
					    int method);

void  SuppressZeroWeightedArcs(SparseGraph *sg);

void InvertArcsDigraph(SparseGraph *sg);

Image *ArcWeightImage(SparseGraph *sg);

void Orient2Digraph(SparseGraph *sg, Image *img, float per);

void Orient2DigraphInner(SparseGraph *sg, 
			 Image *P_sum);
void Orient2DigraphOuter(SparseGraph *sg, 
			 Image *P_sum);

#endif

