#include "heap.h"

void Change(int *a, int *b){
  int tmp;
  tmp = *a;
  *a = *b;
  *b = tmp;
}

void SetRemovalPolicyHeap(Heap *H, char policy){
  if(H->removal_policy != policy){
    H->removal_policy = policy;
  }
}

void GoUpHeap(Heap *H, int i) {
  int j = HEAP_DAD(i);

  if(H->removal_policy == MINVALUE){

    while ((i > 0) && (H->cost[H->pixel[j]] > H->cost[H->pixel[i]])) {
      Change(&H->pixel[j], &H->pixel[i]);
      H->pos[H->pixel[i]] = i;
      H->pos[H->pixel[j]] = j;
      i = j;
      j = HEAP_DAD(i);
    }
  }
  else{ /* removal_policy == MAXVALUE */

    while ((i > 0) && (H->cost[H->pixel[j]] < H->cost[H->pixel[i]])) {
      Change(&H->pixel[j], &H->pixel[i]);
      H->pos[H->pixel[i]] = i;
      H->pos[H->pixel[j]] = j;
      i = j;
      j = HEAP_DAD(i);
    }
  }
}

void GoDownHeap(Heap *H, int i) {
  int j, left = HEAP_LEFTSON(i), right = HEAP_RIGHTSON(i);

  j = i;
  if(H->removal_policy == MINVALUE){

    if ((left <= H->last) && 
	(H->cost[H->pixel[left]] < H->cost[H->pixel[i]]))
      j = left;
    if ((right <= H->last) && 
	(H->cost[H->pixel[right]] < H->cost[H->pixel[j]]))
      j = right;
  }
  else{ /* removal_policy == MAXVALUE */

    if ((left <= H->last) && 
	(H->cost[H->pixel[left]] > H->cost[H->pixel[i]]))
      j = left;
    if ((right <= H->last) && 
	(H->cost[H->pixel[right]] > H->cost[H->pixel[j]]))
      j = right;
  }

  if(j != i) {
    Change(&H->pixel[j], &H->pixel[i]);
    H->pos[H->pixel[i]] = i;
    H->pos[H->pixel[j]] = j;
    GoDownHeap(H, j);
 }
}

char IsFullHeap(Heap *H) {
  if (H->last == (H->n - 1))
    return 1;
  else
    return 0;
}

char IsEmptyHeap(Heap *H) {
  if (H->last == -1){
    ResetHeap(H); 
    return 1;
  }else
    return 0;
}

Heap *CreateHeap(int n, float *cost) {
  Heap *H = NULL;
  int i;

  if (cost == NULL) {
    fprintf(stdout,"Cannot create heap without cost map in CreateHeap");
    return NULL;
  }

  H = (Heap *) malloc(sizeof(Heap));
  if (H != NULL) {
    H->n       = n;
    H->cost    = cost;
    H->color   = (char *) malloc(sizeof(char) * n);
    H->pixel   = (int *) malloc(sizeof(int) * n);
    H->pos     = (int *) malloc(sizeof(int) * n);
    H->last    = -1;
    H->removal_policy = MINVALUE;
    if (H->color == NULL || H->pos == NULL || H->pixel == NULL)
      Error(MSG1,"CreateHeap");
    for (i = 0; i < H->n; i++) {
      H->color[i] = WHITE;
      H->pos[i]   = -1;
      H->pixel[i] = -1;
    }    
  } 
  else
    Error(MSG1,"CreateHeap");

  return H;
}

void DestroyHeap(Heap **H) {
  Heap *aux = *H;
  if (aux != NULL) {
    if (aux->pixel != NULL) free(aux->pixel);
    if (aux->color != NULL) free(aux->color);
    if (aux->pos != NULL)   free(aux->pos);
    free(aux);
    *H = NULL;
  }
}

char InsertHeap(Heap *H, int pixel) {
  if (!IsFullHeap(H)) {
    H->last++;
    H->pixel[H->last] = pixel;
    H->color[pixel]   = GRAY;
    H->pos[pixel]     = H->last;
    GoUpHeap(H, H->last); 
    return 1;
  } else 
    return 0;
}

char RemoveHeap(Heap *H, int *pixel) {
  if (!IsEmptyHeap(H)) {
    *pixel = H->pixel[0];
    H->pos[*pixel]   = -1;
    H->color[*pixel] = BLACK;
    H->pixel[0]      = H->pixel[H->last];
    H->pos[H->pixel[0]] = 0;
    H->pixel[H->last] = -1;
    H->last--;
    GoDownHeap(H, 0);
    return 1;
  } else 
    return 0;
}


void UpdateHeap(Heap *H, int p, float value){
  H->cost[p] = value;

  if (H->color[p] == BLACK) printf("ferrou\n");

  if(H->color[p] == WHITE)
    InsertHeap(H, p);
  else
    GoUpHeap(H, H->pos[p]);
}

void ResetHeap(Heap *H)
{
  int i;

  for (i=0; i < H->n; i++) {
    H->color[i] = WHITE;
    H->pos[i]   = -1;
    H->pixel[i] = -1;
  }
  H->last = -1;
}



