#include "heap_lex.h"

void SetRemovalPolicyHeap_lex(Heap_lex *H, char policy){
  if(H->removal_policy != policy){
    H->removal_policy = policy;
  }
}

void GoUpHeap_lex(Heap_lex *H, int i) {
  int j = HEAP_DAD(i);

  if(H->removal_policy == MINVALUE){
	//if ()
    while ((i > 0) && (H->cost1[H->pixel[j]] > H->cost1[H->pixel[i]])) {
      Change(&H->pixel[j], &H->pixel[i]);
      H->pos[H->pixel[i]] = i;
      H->pos[H->pixel[j]] = j;
      i = j;
      j = HEAP_DAD(i);
    }


  }
  else{ /* removal_policy == MAXVALUE */

    while ((i > 0) && (H->cost1[H->pixel[j]] < H->cost1[H->pixel[i]])) {
      Change(&H->pixel[j], &H->pixel[i]);
      H->pos[H->pixel[i]] = i;
      H->pos[H->pixel[j]] = j;
      i = j;
      j = HEAP_DAD(i);
    }
  }
}

void GoDownHeap_lex(Heap_lex *H, int i) {
  int j, left = HEAP_LEFTSON(i), right = HEAP_RIGHTSON(i);

  j = i;
  if(H->removal_policy == MINVALUE){

    if ((left <= H->last) && 
	(H->cost1[H->pixel[left]] < H->cost1[H->pixel[i]]))
      j = left;
    if ((right <= H->last) && 
	(H->cost1[H->pixel[right]] < H->cost1[H->pixel[j]]))
      j = right;
  }
  else{ /* removal_policy == MAXVALUE */

    if ((left <= H->last) && 
	(H->cost1[H->pixel[left]] > H->cost1[H->pixel[i]]))
      j = left;
    if ((right <= H->last) && 
	(H->cost1[H->pixel[right]] > H->cost1[H->pixel[j]]))
      j = right;
  }

  if(j != i) {
    Change(&H->pixel[j], &H->pixel[i]);
    H->pos[H->pixel[i]] = i;
    H->pos[H->pixel[j]] = j;
    GoDownHeap_lex(H, j);
 }
}

char IsFullHeap_lex(Heap_lex *H) {
  if (H->last == (H->n - 1))
    return 1;
  else
    return 0;
}

char IsEmptyHeap_lex(Heap_lex *H) {
  if (H->last == -1){
    ResetHeap_lex(H); 
    return 1;
  }else
    return 0;
}

Heap_lex *CreateHeap_lex(int n, float *cost1, float *cost2) {
  Heap_lex *H = NULL;
  int i;

  if (cost1 == NULL || cost2 == NULL) {
    fprintf(stdout,"Cannot create heap without cost map in CreateHeap_lex");
    return NULL;
  }

  H = (Heap_lex *) malloc(sizeof(Heap_lex));

  if (H != NULL) {
    H->n       = n;
    H->cost1   = cost1;
    H->cost2   = cost2;
    H->color   = (char *) malloc(sizeof(char) * n);
    H->pixel   = (int *) malloc(sizeof(int) * n);
    H->pos     = (int *) malloc(sizeof(int) * n);
    H->last    = -1;
    H->removal_policy = MINVALUE;
    if (H->color == NULL || H->pos == NULL || H->pixel == NULL)
      Error(MSG1,"CreateHeap_lex");
    for (i = 0; i < H->n; i++) {
      H->color[i] = WHITE;
      H->pos[i]   = -1;
      H->pixel[i] = -1;
    }    
  } 
  else
    Error(MSG1,"CreateHeap_lex");
  
  return H;
}

void DestroyHeap_lex(Heap_lex **H) {
  Heap_lex *aux = *H;
  if (aux != NULL) {
    if (aux->pixel != NULL) free(aux->pixel);
    if (aux->color != NULL) free(aux->color);
    if (aux->pos != NULL)   free(aux->pos);
    free(aux);
    *H = NULL;
  }
}

char InsertHeap_lex(Heap_lex *H, int pixel) {
  if (!IsFullHeap_lex(H)) {
    H->last++;
    H->pixel[H->last] = pixel;
    H->color[pixel]   = GRAY;
    H->pos[pixel]     = H->last;
    GoUpHeap_lex(H, H->last); 
    return 1;
  } else 
    return 0;
}

char RemoveHeap_lex(Heap_lex *H, int *pixel) {
  if (!IsEmptyHeap_lex(H)) {
    *pixel = H->pixel[0];
    H->pos[*pixel]   = -1;
    H->color[*pixel] = BLACK;
    H->pixel[0]      = H->pixel[H->last];
    H->pos[H->pixel[0]] = 0;
    H->pixel[H->last] = -1;
    H->last--;
    GoDownHeap_lex(H, 0);
    return 1;
  } else 
    return 0;
}


void UpdateHeap_lex(Heap_lex *H, int p, float value1, float value2){
  H->cost1[p] = value1;
  H->cost2[p] = value2;
  if (H->color[p] == BLACK) printf("ferrou\n");

  if(H->color[p] == WHITE)
    InsertHeap_lex(H, p);
  else
    GoUpHeap_lex(H, H->pos[p]);
}

void ResetHeap_lex(Heap_lex *H){
  int i;

  for (i=0; i < H->n; i++) {
    H->color[i] = WHITE;
    H->pos[i]   = -1;
    H->pixel[i] = -1;
  }
  H->last = -1;
}



