
#include "highlight.h"


CImage *CWideHighlight(CImage *cimg, Image *label, float radius, int color, bool fill){
  CImage *hcimg;
  int p,q,i;
  AdjRel *A=NULL;
  int u_x,u_y,v_x,v_y;

  hcimg = CloneCImage(cimg);
  A    = Circular(radius);
  for (u_y=0; u_y < hcimg->C[0]->nrows; u_y++){
    for (u_x=0; u_x < hcimg->C[0]->ncols; u_x++){
      p = u_x + u_y*hcimg->C[0]->ncols;
      if(fill){
	if(label->data[p] > 0){
	  hcimg->C[0]->data[p] = ROUND(0.3*CHANNEL0(color) + 0.7*hcimg->C[0]->data[p]);
	  hcimg->C[1]->data[p] = ROUND(0.3*CHANNEL1(color) + 0.7*hcimg->C[1]->data[p]);
	  hcimg->C[2]->data[p] = ROUND(0.3*CHANNEL2(color) + 0.7*hcimg->C[2]->data[p]);
	}
      }

      for (i=1; i < A->n; i++){
	v_x = u_x + A->dx[i];
	v_y = u_y + A->dy[i];
	if (ValidPixel(hcimg->C[0],v_x,v_y)){
	  q = v_x + v_y*hcimg->C[0]->ncols;
	  if (label->data[p] > label->data[q]){
	    hcimg->C[0]->data[p] = CHANNEL0(color);
	    hcimg->C[1]->data[p] = CHANNEL1(color);
	    hcimg->C[2]->data[p] = CHANNEL2(color);
	    break;
	  }
	}
      }
    }
  }
  DestroyAdjRel(&A);
  return(hcimg);
}


Image *WideHighlight(Image *img, Image *label, float radius, int value, bool fill){
  Image *himg;
  int p,q,i;
  AdjRel *A=NULL;
  int u_x,u_y,v_x,v_y;

  himg = CloneImage(img);
  A    = Circular(radius);
  for (u_y=0; u_y < himg->nrows; u_y++){
    for (u_x=0; u_x < himg->ncols; u_x++){
      p = u_x + u_y*himg->ncols;
      if(fill){
	if(label->data[p] > 0)
	  himg->data[p] = ROUND(0.3*value + 0.7*himg->data[p]);
      }

      for (i=1; i < A->n; i++){
	v_x = u_x + A->dx[i];
	v_y = u_y + A->dy[i];
	if (ValidPixel(himg,v_x,v_y)){
	  q = v_x + v_y*himg->ncols;
	  if (label->data[p] > label->data[q]){
	    himg->data[p] = value;
	    break;
	  }
	}
      }
    }
  }
  DestroyAdjRel(&A);
  return(himg);
}



CImage *CWideHighlightLabels(CImage *cimg, Image *label, float radius, int *colormap, bool fill){
  CImage *hcimg;
  int p,q,i,lb;
  AdjRel *A=NULL;
  int u_x,u_y,v_x,v_y;

  hcimg = CloneCImage(cimg);
  A    = Circular(radius);
  for (u_y=0; u_y < hcimg->C[0]->nrows; u_y++){
    for (u_x=0; u_x < hcimg->C[0]->ncols; u_x++){
      p = u_x + u_y*hcimg->C[0]->ncols;
      if(fill){
        lb = label->data[p];
	if(lb> 0){
	  hcimg->C[0]->data[p] = ROUND(0.3*CHANNEL0(colormap[lb]) + 0.7*hcimg->C[0]->data[p]);
	  hcimg->C[1]->data[p] = ROUND(0.3*CHANNEL1(colormap[lb]) + 0.7*hcimg->C[1]->data[p]);
	  hcimg->C[2]->data[p] = ROUND(0.3*CHANNEL2(colormap[lb]) + 0.7*hcimg->C[2]->data[p]);
	}
      }

      for (i=1; i < A->n; i++){
	v_x = u_x + A->dx[i];
	v_y = u_y + A->dy[i];
	if (ValidPixel(hcimg->C[0],v_x,v_y)){
	  q = v_x + v_y*hcimg->C[0]->ncols;
          lb = label->data[p];
	  if (lb > label->data[q]){
	    hcimg->C[0]->data[p] = CHANNEL0(colormap[lb]);
	    hcimg->C[1]->data[p] = CHANNEL1(colormap[lb]);
	    hcimg->C[2]->data[p] = CHANNEL2(colormap[lb]);
	    break;
	  }
	}
      }
    }
  }
  DestroyAdjRel(&A);
  return(hcimg);
}


bool    HStripedTexture(int x, int y, int w, int h){
  x %= w;
  y %= h;
  if(y==h/2) return 1;
  else       return 0;
}

bool    VStripedTexture(int x, int y, int w, int h){
  x %= w;
  y %= h;
  if(x==w/2) return 1;
  else       return 0;
}

bool    BackslashTexture(int x, int y, int w, int h){
  x %= w;
  y %= h;
  if(x==y) return 1;
  else     return 0;
}

bool    SlashTexture(int x, int y, int w, int h){
  x %= w;
  y %= h;
  if(y==-x+h-1) return 1;
  else          return 0;
}

bool    GridTexture(int x, int y, int w, int h){
  x %= w;
  y %= h;
  return(VStripedTexture(x,y,w,h) ||
	 HStripedTexture(x,y,w,h));
}

bool    RGridTexture(int x, int y, int w, int h){
  x %= w;
  y %= h;
  return(BackslashTexture(x,y,w,h) ||
	 SlashTexture(x,y,w,h));
}


Image  *TextureHighlight(Image *img, Image *label, 
			 float radius, int value, bool fill,
			 bool (*texture)(int,int,int,int), int w, int h){
  Image *himg;
  int p,q,i;
  AdjRel *A=NULL;
  int u_x,u_y,v_x,v_y;
  bool b;

  himg = CloneImage(img);
  A    = Circular(radius);
  for (u_y=0; u_y < himg->nrows; u_y++){
    for (u_x=0; u_x < himg->ncols; u_x++){
      p = u_x + u_y*himg->ncols;

      if(label->data[p] > 0){
	b = (*texture)(u_x,u_y,w,h);
	if(b)
	  himg->data[p] = value;
	else if(fill)
	  himg->data[p] = ROUND(0.3*value + 0.7*himg->data[p]);
      }

      for (i=1; i < A->n; i++){
	v_x = u_x + A->dx[i];
	v_y = u_y + A->dy[i];
	if (ValidPixel(himg,v_x,v_y)){
	  q = v_x + v_y*himg->ncols;
	  if (label->data[p] > label->data[q]){
	    himg->data[p] = value;
	    break;
	  }
	}
      }
    }
  }
  DestroyAdjRel(&A);
  return(himg);
}



CImage *CTextureHighlight(CImage *cimg, Image *label, 
			  float radius, int color, bool fill,
			  bool (*texture)(int,int,int,int), int w, int h){
  CImage *hcimg;
  int p,q,i;
  AdjRel *A=NULL;
  int u_x,u_y,v_x,v_y;
  bool b;

  hcimg = CloneCImage(cimg);
  A    = Circular(radius);
  for (u_y=0; u_y < hcimg->C[0]->nrows; u_y++){
    for (u_x=0; u_x < hcimg->C[0]->ncols; u_x++){
      p = u_x + u_y*hcimg->C[0]->ncols;

      if(label->data[p] > 0){
	b = (*texture)(u_x,u_y,w,h);
	if(b){
	  hcimg->C[0]->data[p] = CHANNEL0(color);
	  hcimg->C[1]->data[p] = CHANNEL1(color);
	  hcimg->C[2]->data[p] = CHANNEL2(color);
	}
	else if(fill){
	  hcimg->C[0]->data[p] = ROUND(0.3*CHANNEL0(color) + 0.7*hcimg->C[0]->data[p]);
	  hcimg->C[1]->data[p] = ROUND(0.3*CHANNEL1(color) + 0.7*hcimg->C[1]->data[p]);
	  hcimg->C[2]->data[p] = ROUND(0.3*CHANNEL2(color) + 0.7*hcimg->C[2]->data[p]);
	}
      }

      for (i=1; i < A->n; i++){
	v_x = u_x + A->dx[i];
	v_y = u_y + A->dy[i];
	if (ValidPixel(hcimg->C[0],v_x,v_y)){
	  q = v_x + v_y*hcimg->C[0]->ncols;
	  if (label->data[p] > label->data[q]){
	    hcimg->C[0]->data[p] = CHANNEL0(color);
	    hcimg->C[1]->data[p] = CHANNEL1(color);
	    hcimg->C[2]->data[p] = CHANNEL2(color);
	    break;
	  }
	}
      }
    }
  }
  DestroyAdjRel(&A);
  return(hcimg);
}


