
#include "ift.h"


void DrawSeeds(Image *img, int *S, int val){
  int nS,i;
  if(S == NULL) 
    return;
  nS = S[0];
  for(i=1; i<=nS; i++){
    img->data[S[i]] = val;
  }
}



//Weighted Distance Transform.
Image *pred_IFTSUM(SparseGraph *sg,
		   int *Si, 
		   int *Se, 
		   Image *label,
		   float power){
  Heap *Q=NULL;
  Image *pred;
  int i,p,q,n;
  float edge,tmp;
  float *cost=NULL;
  int u_x,u_y,v_x,v_y;
  AdjRel *A;
  float *Dpq;

  n    = sg->ncols*sg->nrows;
  pred = CreateImage(sg->ncols, sg->nrows);
  cost = AllocFloatArray(n);
  Q = CreateHeap(n, cost);
  SetRemovalPolicyHeap(Q, MINVALUE);
  A = sg->A;

  //--------------------
  Dpq = AllocFloatArray(A->n);
  for(i=1; i<A->n; i++){
    Dpq[i] = sqrtf(A->dx[i]*A->dx[i] + A->dy[i]*A->dy[i]);
  }
  //--------------------

  DrawSeeds(label, Si, 1);
  DrawSeeds(label, Se, 0);
 
  SetImage(pred, NIL);
  for(p=0; p<n; p++){
    if(label->data[p]==NIL) cost[p] = FLT_MAX;
    else                   cost[p] = 0.0;
  }

  if(Si != NULL){
    for(i=1; i<=Si[0]; i++){
      InsertHeap(Q, Si[i]);
    }
  }
  if(Se != NULL){
    for(i=1; i<=Se[0]; i++){
      InsertHeap(Q, Se[i]);
    }
  }

  while(!IsEmptyHeap(Q)){
    RemoveHeap(Q, &p);
    u_x = p%label->ncols; 
    u_y = p/label->ncols; 

    for(i=1; i<A->n; i++){
      v_x = u_x + A->dx[i];
      v_y = u_y + A->dy[i];
      if(ValidPixel(label,v_x,v_y)){
	q = v_x + label->ncols*v_y;
	if(Q->color[q] != BLACK){

	  edge = (sg->n_link[p])[i];
          
	  tmp  = cost[p] + powf(edge, power) - 1.0 + Dpq[i];

	  if(tmp < cost[q]){
	    UpdateHeap(Q, q, tmp);
	   // label->data[q] = label->data[p];
	    pred->data[q] = p;

	  }
	}
      }
    }
  }
  free(Dpq);
  free(cost);
  DestroyHeap(&Q);
  return pred;
}


void method_IFTW_FIFO_InnerCut(SparseGraph *sg,
			       int *Si, 
			       int *Se,
			       Image *label){
  PriorityQueue *Q=NULL;
  int i,j,p,q,n;
  int edge,tmp;
  Image *value;
  int u_x,u_y,v_x,v_y;
  AdjRel *A;

  value = CreateImage(sg->ncols,
		      sg->nrows);
  n = label->ncols*label->nrows;
  Q = CreatePQueue(sg->Wmax+2,n,value->data);
  A = sg->A;

  DrawSeeds(label, Si, 1);
  DrawSeeds(label, Se, 0);

  for(p=0; p<n; p++){
    if(label->data[p]==NIL) value->data[p] = INT_MAX;
    else                   value->data[p] = 0;
  }

  for(i=1; i<=Si[0]; i++){
    FastInsertElemPQueue(Q, Si[i]);
  }
  for(i=1; i<=Se[0]; i++){
    FastInsertElemPQueue(Q, Se[i]);
  }

  while(!IsEmptyPQueue(Q)) {
    p = FastRemoveMinFIFOPQueue(Q);
    u_x = p%label->ncols; //PixelX(label, p);
    u_y = p/label->ncols; //PixelY(label, p);

    for(i=1; i<A->n; i++){
      v_x = u_x + A->dx[i];
      v_y = u_y + A->dy[i];
      if(ValidPixel(label,v_x,v_y)){
	q = v_x + label->ncols*v_y;
	if(Q->L.elem[q].color != BLACK){

	  if(label->data[p]==1){
	    j = get_edge_index(q, p, sg);
	    edge = (sg->n_link[q])[j] + 1;
	  }
	  else
	    edge = (sg->n_link[p])[i] + 1;

	  tmp  = edge;

	  if(tmp < value->data[q]){
	    if(Q->L.elem[q].color == GRAY)
	      FastRemoveElemPQueue(Q, q);
	    value->data[q] = tmp;
	    label->data[q] = label->data[p];
	    FastInsertElemPQueue(Q, q);
	  }
	}
      }
    }
  }
  DestroyImage(&value);
  DestroyPQueue(&Q);
}


void method_IFTW_FIFO_OuterCut(SparseGraph *sg,
			       int *Si, 
			       int *Se,
			       Image *label){
  PriorityQueue *Q=NULL;
  int i,j,p,q,n;
  int edge,tmp;
  Image *value;
  int u_x,u_y,v_x,v_y;
  AdjRel *A;

  value = CreateImage(sg->ncols,
		      sg->nrows);
  n = label->ncols*label->nrows;
  Q = CreatePQueue(sg->Wmax+2,n,value->data);
  A = sg->A;

  DrawSeeds(label, Si, 1);
  DrawSeeds(label, Se, 0);

  for(p=0; p<n; p++){
    if(label->data[p]==NIL) value->data[p] = INT_MAX;
    else                   value->data[p] = 0;
  }

  for(i=1; i<=Si[0]; i++){
    FastInsertElemPQueue(Q, Si[i]);
  }
  for(i=1; i<=Se[0]; i++){
    FastInsertElemPQueue(Q, Se[i]);
  }

  while(!IsEmptyPQueue(Q)) {
    p = FastRemoveMinFIFOPQueue(Q);
    u_x = p%label->ncols; //PixelX(label, p);
    u_y = p/label->ncols; //PixelY(label, p);

    for(i=1; i<A->n; i++){
      v_x = u_x + A->dx[i];
      v_y = u_y + A->dy[i];
      if(ValidPixel(label,v_x,v_y)){
	q = v_x + label->ncols*v_y;
	if(Q->L.elem[q].color != BLACK){

	  if(label->data[p]==0){
	    j = get_edge_index(q, p, sg);
	    edge = (sg->n_link[q])[j] + 1;
	  }
	  else
	    edge = (sg->n_link[p])[i] + 1;

	  tmp  = edge;

	  if(tmp < value->data[q]){
	    if(Q->L.elem[q].color == GRAY)
	      FastRemoveElemPQueue(Q, q);
	    value->data[q] = tmp;
	    label->data[q] = label->data[p];
	    FastInsertElemPQueue(Q, q);
	  }
	}
      }
    }
  }
  DestroyImage(&value);
  DestroyPQueue(&Q);
}


//----------------------------------------


Image *EDT(Image *bin, float r){
  Image *Dx=NULL, *Dy=NULL, *cost;
  PriorityQueue *Q=NULL;
  int i,p,q,u_x,u_y,v_x,v_y,dx,dy,tmp;
  AdjRel *A;

  A = Circular(r);
  Dx = CreateImage(bin->ncols, bin->nrows);
  Dy = CreateImage(bin->ncols, bin->nrows);
  cost = CreateImage(bin->ncols, bin->nrows);
  Q = CreatePQueue(bin->ncols+bin->nrows, bin->n, cost->data);

  for(p = 0; p < bin->n; p++){
    if(bin->data[p] > 0){
      cost->data[p] = 0;
      InsertElemPQueue(&Q, p);
    }
    else
      cost->data[p] = INT_MAX;
  }
  while(!IsEmptyPQueue(Q)) {
    p = RemoveMinFIFOPQueue(Q);
    u_x = p%bin->ncols;
    u_y = p/bin->ncols;
    for(i=1; i<A->n; i++){
      v_x = u_x + A->dx[i];
      v_y = u_y + A->dy[i];
      if(ValidPixel(bin,v_x,v_y)){
	q = v_x + bin->ncols*v_y;
	if(cost->data[p] < cost->data[q]){
	  dx  = Dx->data[p] + abs(v_x-u_x);
	  dy  = Dy->data[p] + abs(v_y-u_y);
	  tmp = dx*dx + dy*dy;
	  if (tmp < cost->data[q]){
	    if(Q->L.elem[q].color == GRAY)
	      RemoveElemPQueue(Q, q);
	    cost->data[q] = tmp;
	    Dx->data[q] = dx;
	    Dy->data[q] = dy;
	    InsertElemPQueue(&Q, q);
	  }
	}
      }
    }
  }
  DestroyPQueue(&Q);
  DestroyAdjRel(&A);
  DestroyImage(&Dx);
  DestroyImage(&Dy);
  return cost;
}

