
#include "common.h"
#include "image.h"
#include "cimage.h"
#include "adjacency.h"
#include "morphology.h"
#include "ift.h"

#include <time.h>


Image *GetLabelFromCyclicEDT(Image *bin, float r){
  Image *Dx=NULL, *Dy=NULL, *cost, *label;
  PriorityQueue *Q=NULL;
  int i,p,q,u_x,u_y,v_x,v_y,dx,dy,tmp,lb;
  AdjRel *A;

  A = Circular(r);
  Dx = CreateImage(bin->ncols, bin->nrows);
  Dy = CreateImage(bin->ncols, bin->nrows);
  cost  = CreateImage(bin->ncols, bin->nrows);
  label = CreateImage(bin->ncols, bin->nrows);
  Q = CreatePQueue(bin->ncols+bin->nrows, bin->n, cost->data);

  lb = 0;
  for(p = 0; p < bin->n; p++){
    if(bin->data[p] > 0){
      lb++;
      label->data[p] = lb;
      cost->data[p] = 0;
      InsertElemPQueue(&Q, p);
    }
    else
      cost->data[p] = INT_MAX;
  }
  while(!IsEmptyPQueue(Q)) {
    p = RemoveMinFIFOPQueue(Q);
    u_x = p%bin->ncols;
    u_y = p/bin->ncols;
    for(i=1; i<A->n; i++){
      v_x = u_x + A->dx[i];
      v_y = u_y + A->dy[i];
      dx  = Dx->data[p] + abs(v_x - u_x);
      dy  = Dy->data[p] + abs(v_y - u_y);
      if(!ValidPixel(bin,v_x,v_y)){
	if(v_x >= bin->ncols) v_x %= bin->ncols;
	if(v_y >= bin->nrows) v_y %= bin->nrows;
	if(v_x < 0) v_x += bin->ncols;
	if(v_y < 0) v_y += bin->nrows;
      }
      q = v_x + bin->ncols*v_y;

      if(cost->data[p] < cost->data[q]){
	tmp = dx*dx + dy*dy;
	if (tmp < cost->data[q]){
	  if(Q->L.elem[q].color == GRAY)
	    RemoveElemPQueue(Q, q);
	  label->data[q] = label->data[p];
	  cost->data[q] = tmp;
	  Dx->data[q] = dx;
	  Dy->data[q] = dy;
	  InsertElemPQueue(&Q, q);
	}
      }
    }
  }
  DestroyPQueue(&Q);
  DestroyAdjRel(&A);
  DestroyImage(&Dx);
  DestroyImage(&Dy);
  DestroyImage(&cost);
  return label;
}



int main(){
  CImage *cimg;
  Image *bin, *label;
  int nrows,ncols,nreg;
  int x,y,i;

  printf("Enter the image size in X: ");
  scanf("%d", &ncols);
  printf("Enter the image size in Y: ");
  scanf("%d", &nrows);
  printf("Enter the number of regions: ");
  scanf("%d", &nreg);
 
  bin = CreateImage(ncols, nrows);
  SetImage(bin, 0);

  srand(time(NULL));
  for(i = 1; i <= nreg; i++){
    x = rand()%ncols;
    y = rand()%nrows;
    bin->array[y][x] = 1;
  }
  /*WriteImage(bin, "bin.pgm");*/
  label = GetLabelFromCyclicEDT(bin, 1.5);
  cimg = ColorizeLabel(label);

  /*WriteImage(label, "label.pgm");*/
  SetImage(cimg->C[0], 0);
  WriteCImage(cimg, "voronoi.ppm");

  DestroyImage(&label);
  DestroyImage(&bin);
  DestroyCImage(&cimg);

  return 0;
}
