#ifndef _PointSet_h_
#define _PointSet_h_

#include <iostream>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <ctime>
#include <cmath>
#include <vector>
#include <fstream>
#include <algorithm>

#define PI acos(-1)

namespace PSet  {

using namespace std;

enum
{
    STD=0, SIMPLESTD, LIBOPF, OPFDAT, MATLAB, CLUTO, GNUPLOT, SVM
};

enum {CONST, ID, SIN, COS, TAN, ASIN, ACOS, ATAN, SINH, COSH, TANH, ASINH, ACOSH, ATANH, LN, EXP, POW, PLUS, MINUS, TIMES,
        DIV, ENTER, SQR, SQRT};

enum{GAUSSIAN, SPLINEGAUSSIAN, LINEAR, CIRC_RC, CIRC_CP, CIRC_C2P, CIRC_3P, CIRC_NCP, LINE, CUBIC, BEZIER, SPIRAL_CRNV, SPIRAL_CAV, SPIRAL_CAB, POLY};

enum{LABEL, TRUELABEL, DATA};                   //Veja esses nomes!!!

float Operation(float u, float v, unsigned int type, vector<float> arg);

int Binomial(int n, int k);

class Point
{
    int DIM;
    std::vector <float> p;
    public:
    int cl, classifcl;            //cl = truelabel, while classifcl is the label provided by a classification method.
    Point();
    Point(int D);
    void setdim(int n);
    int dim();
    Point operator - (Point P2);
    Point operator + (Point P2);
    Point operator * (float x);
    float operator * (Point P2);                    //Dot Product
    Point operator ^ (Point P2);                    //Cross Product
    void operator = (Point P2);
    bool operator == (Point P2);
    void operator += (Point P2);
    void operator *= (float f);
    float& operator [] (int i);                 //With automatic alocation..
    Point versor();
    void free();
    float sqrabs();
    float abs();
    void print();                   //Para testes
    void clear();
    float maxcoord();
    void DeleteDimension(int dim);
};


struct fNode
{
    unsigned int type;
    vector <float> arg;
    fNode *left;
    fNode *right;
    fNode();
    float operator () (float t);
    float value(float t);
    void operator = (fNode fN);
};


class AlgFunction
{
    vector <fNode> fN;

public:
    void insert (unsigned int type, vector<float> arg);
    void insert (unsigned int type, float arg);
    void insert (unsigned int type);
    float operator () (float t);
    void append(unsigned int x, unsigned int y, bool b);            //Appends y in x. b=0: left. b=1:right.
    void append(unsigned int x, AlgFunction F, bool b);             //Appends F in x.
    void linsert(unsigned int type);
    void rinsert(unsigned int type);
    void operator = (AlgFunction F);
};


class Spline : public vector<AlgFunction>
{
    public:
    unsigned int type;
    int id[2];                                                     //id[0] = cl, id[1] = m
    vector<float> Ctrl;
    vector<Point> CtrlPt;
    vector< vector<float> > CtrlVector;
    float a, b;
    Point operator () (float t);
    Point value(float t);
    float length();
    float length(float u, float v);
    Spline();

    /*  Default Spline Configuration Functions   */

    void SetAsCircumference(Point center, float radius);
    void SetAsCircumference(Point center, Point A);
    void SetAsCircumference(Point C, Point P, Point Q);
    void SetAsCircumference(Point center, float radius, vector<float> normal);
    void SetAsLine(Point A, Point B);
    void SetAsCubicSpline (Point A, Point B);
    void SetAsCubicSpline (Point A, Point B, Point C, Point D);
    void SetAsBezierSpline (vector<Point> V);
    void SetAsSpiral(Point center, float radius, vector<float> normal, float velocity);
    void SetAsSpiral(Point center, Point A, float velocity);
    void SetAsSpiral(Point center, Point A, Point B, float velocity, int nloops);
    void SetAsSpiral(Point center, Point A, Point B);
    void SetAsPolynomial(vector<Point> V);
    void SetAsCyclicPolynomial(vector<Point> V);
    /* Default Spline Transformations */

    void Rotation(Point Center, float Angle);               //Implementar
    void Rotation(Point Center, Point P);                   //Implementar
    void Translation(Point Original, Point New);

    void SetAsCustomSpline(char *dir);

};

class PointSet
{
    int DIM, NUM;
    std::vector<Point> p;
    public:
    Spline *S;
    int cl, type, role, visual_dim;
    std::vector <Point> ctrl;
    float ctrlfloat[10];
    bool visible;
    Point ctrlback(unsigned int i);          //Acessa a posio i de ctrl de trs pra frente.
    PointSet();
    void setdim(int n);
    void setnum(int n);
    void addnum();
    int dim();
    int num();
    void addPoint(Point PT);
    void erasePoint(Point PT);
    void erasePoint(unsigned int index);
    void setcl(int c);
    void operator = (PointSet PS);
    PointSet operator + (Point P);
    void operator += (Point P);
    void operator *= (float f);
    Point& operator [] (unsigned int i);                     //Com alocao automtica.


    void AddGaussianDistribution(int nsamples, float stddev, Point x);
    void AddGaussianDistribution(int nsamples, float stddev, Point x, Point PMAX, Point PMIN);
    void AddGaussianDistributionAlongLine(int nsamples, float stddev, Point x1, Point x2);
    void AddGaussianDistributionAlongLine(int nsamples, float stddev, Point x1, Point x2, Point PMAX, Point PMIN);
    void AddGaussianDistributionAlongSpline(int nsamples, float stddev, Spline s, float a, float b, int N);
    void AddGaussianDistributionAlongSpline(int nsamples, float stddev, Spline s);
    void AddSimpleGaussianDistributionAlongSpline(int nsamples, float stddev, Spline s);

    void AddLinearDistribution(int nsamples, float radius, Point x);
    //void AddLinearDistributionAlongSpline(int nsamples, float radius, Spline s int distribution_type);

    void DeleteDimension(int dim);
    void AddSpline(Spline S, float VoxelSize);
    void AddSplineSimple(Spline S);
    void ImportFile(char name[], int EXT);
    void ExportFile(char str[], int EXT);
    void free();
    void clear();
    void print();               //Para testes.
};

class DataList: public vector<PointSet>
{
    int DIM;
    public:
    int role;
    int CurrentDim[3];
    vector<Spline> SList;
    vector<Point> temp;
    DataList();
    void setdim(int dim);
    int dim();
    unsigned int NumberOfClasses();
    unsigned int NumberOfPoints();
    void AddPoint(Point PT);
    void AddPoint(Point PT, int cl);
    void AddPointSet(PointSet PS);
    void AddGaussianDistribution(int nsamples, float stddev, Point x);
    void AddGaussianDistribution(int nsamples, float stddev, Point x, int cl);
    void AddGaussianDistribution(int nsamples, float stddev, Point x, Point PMAX, Point PMIN);
    void AddGaussianDistributionAlongLine(int nsamples, float stddev, Point x1, Point x2);
    void AddGaussianDistributionAlongLine(int nsamples, float stddev, Point x1, Point x2, Point PMAX, Point PMIN);
    void AddGaussianDistributionAlongSpline(int nsamples, float stddev, Spline s, float a, float b, int N);
    void AddGaussianDistributionAlongSpline(int nsamples, float stddev, Spline s);
    void AddGaussianDistributionAlongSpline(int nsamples, float stddev, Spline s, int cl);
    void AddSimpleGaussianDistributionAlongSpline(int nsamples, float stddev, Spline s, int cl);

    Point& pt(int n);                                                                    //returns the (n+1)th point

    void AddLinearDistribution(int nsamples, float radius, Point x, int cl);

    void EraseClass(int cl);
    void DeleteDimension(int dim);

    unsigned int PointSetIndex(int cl, unsigned int k);                                  //retrieves the index of the (k+1)-th PointSet whose class is cl.
    void AddSpline(Spline S, float VoxelSize);
    void operator = (DataList DL);
    void operator *= (float f);
    void operator += (Point P);
    void operator += (DataList DL);
    PointSet FindPoint(Point P);
    unsigned int FindPointIndex(Point P);
    void ImportFile(char name[], int EXT);
    void ExportFile(char str[], int EXT);
    float StdDevSqr();                              //Returns the square of the standard deviation of the points.
    float Density(Point P, int outdegree, float sqrsigma);
    float Accuracy();
    void SwapClassifcl(int classifclA, int classifclB);
    void CorrectClasses();                          //Swaps the classified classes until maximize the accuracy.
    int GetSplineId1(int id0);
    Spline *GetSpline(int id0, int id1);
    void print();
    //~DataList();
};

class Voxel : public std::vector <int>
{
public:
    vector<float> C;
    bool isNeighboor(Voxel V);
    bool operator == (Voxel V);
};

class VoxelSet : public vector<Voxel>
{
public:
vector<int> lim;
vector<Voxel> Neighboorhood(Voxel V);
vector<Voxel> Neighboorhood(unsigned int i);
void addSpline(Spline S);                           //implementar
void addPoint (Point P);
void addPointSet (PointSet P);
};

}       //end PSet namespace
#endif
