#include "PointSet.h"

namespace PSet
{

/*-----------Point--------------*/

void Point::print()
{
    for(int i = 0; i < DIM; i++) printf("%.2f\t", p[i]);
    cout << endl;
}

Point::Point()
{
    DIM=1;
    cl=0;
    classifcl = 0;
    p.reserve(1);
}

Point::Point(int D)
{
    cl = 0;
    classifcl = 0;
    if(D == 0) DIM = 0;
    else p.reserve(1);
}

void Point::setdim(int n)
{
    p.resize(n);
    for(int i=DIM; i<n; i++)
        p[i] = 0;
    DIM = n;
}

int Point::dim()
{
    return DIM;
}

void Point::free()
{
    delete &p;
}

Point Point::operator - (Point P2)
{
    Point P3;
    int i;
    P3.setdim(DIM);
    for(i=0; i<DIM; i++) P3.p[i] = p[i] - P2.p[i];
    return P3;
}

Point Point::operator + (Point P2)
{
    Point P3;
    int i;
    P3.setdim(DIM);
    for(i=0; i<DIM; i++) P3.p[i] = p[i] + P2.p[i];
    return P3;
}

Point Point::operator * (float x)
{
    Point P3;
    int i;
    P3.setdim(DIM);
    for(i=0; i<DIM; i++) P3.p[i] = p[i]*x;
    return P3;
}

float Point::operator * (Point P2)
{
    float S = 0;
    int i;
    for(i=0; i<DIM; i++) S += p[i]*P2.p[i];
    return S;
}

Point Point::operator ^ (Point P2)
{
    Point P;
    P.setdim(3);
    P[0] = p[1]*P2[2] - p[2]*P2[1];
    P[1] = p[2]*P2[0] - p[0]*P2[2];
    P[2] = p[0]*P2[1] - p[1]*P2[0];
    return P;
}

void Point::operator = (Point P2)
{
    int i;
    setdim(P2.dim());
    for(i=0; i<P2.dim(); i++) p[i] = P2.p[i];
    for(i=P2.dim(); i<DIM; i++) p[i] = 0;
    cl = P2.cl;
    classifcl = P2.classifcl;
}

bool Point::operator == (Point P2)
{
    if(P2.dim() != DIM) return false;
    for(int i=0; i<DIM; i++) if(P2[i] != p[i]) return false;
    return true;
}

void Point::operator += (Point P2)
{
    if(P2.dim() > DIM) setdim(P2.dim());
    for(int i=0; i<P2.dim(); i++) p[i] += P2[i];
}

void Point::operator *= (float f)
{
    for(int i=0; i<DIM; i++) p[i]*=f;
}

float& Point::operator [] (int i)
{
    if(p.size() <= i) setdim(i+1);
    return p[i];
}

Point Point::versor()
{
    Point P;
    int i;
    P.setdim(DIM);
    for(i=0; i<DIM; i++) P[i] = p[i]*(1/abs());
    return P;
}

float Point::sqrabs()
{
    float S = 0;
    for(int i=0; i<DIM; i++) S += p[i]*p[i];
    return S;
}

float Point::abs()
{
    return sqrt(sqrabs());
}


void Point::clear()
{
    for(int i=0; i<DIM; i++)
        p[i] = 0;
}


float Point::maxcoord()
{
    float maxcoord = std::abs(p[0]);
    for(int i=0; i<DIM; i++)
        if(std::abs(p[i]) > maxcoord)
            maxcoord = std::abs(p[i]);
    return maxcoord;
}

void Point::DeleteDimension(int dim)
{
    p.erase(p.begin()+dim);
    dim--;
}

/*---------PointSet---------*/

Point PointSet::ctrlback(unsigned int i)
{
    return ctrl[ctrl.size() - 1 - i];
}

PointSet::PointSet()
{
    DIM=1;
    NUM=0;
    visible = true;
    cl = -1;
    visual_dim = 1;
}

void PointSet::setdim(int n)
{
    DIM = n;
    int i;
    for(i=0; i<NUM; i++) p[i].setdim(n);
    if(n < 3)
        visual_dim = n;
    else
        visual_dim = 3;

}

void PointSet::setnum(int n)
{
    p.resize(n);
    NUM = n;
    setdim(DIM);
}

void PointSet::addnum()
{
    setnum(NUM+1);
}

int PointSet::dim()
{
    return DIM;
}

int PointSet::num()
{
    return NUM;
}

void PointSet::addPoint(Point PT)
{
    addnum();
    p[NUM-1] = PT;
}

void PointSet::erasePoint(Point PT)
{
    for(int i=0; i<NUM; i++) if(p[i] == PT) p.erase(p.begin() + i);
}

void PointSet::erasePoint(unsigned int index)
{
    p.erase(p.begin() + index);
}

void PointSet::setcl(int c)
{
    int i;
    cl = c;
    for(i = 0; i < NUM; i++) p[i].cl = c;
}

PointSet PointSet::operator + (Point P)
{
    PointSet Q;
    for(int i = 0; i < NUM; i++) Q[i] = p[i] + P[i];
    return Q;
}

void PointSet::operator += (Point P)
{
    if(P.dim() > DIM) setdim(P.dim());
    for(int i=0; i<num(); i++) p[i] += P;
}

void PointSet::operator *= (float f)
{
    for(int i=0; i<NUM; i++) p[i] *= f;
}

void PointSet::operator = (PointSet PS)
{
    setnum(PS.num());
    setdim(PS.dim());
    for(int i = 0; i < NUM; i++) p[i] = PS[i];
    type = PS.type;
    cl = PS.cl;
}

Point& PointSet::operator [] (unsigned int i)
{
    if(p.size() <= i) p.resize(i+1);
    return p[i];
}

void PointSet :: AddGaussianDistributionAlongLine(int nsamples, float stddev, Point x1, Point x2)
{
    int i;
    float r;
    r = (float) rand()/RAND_MAX;
    for(i=0; i<nsamples; i++)
        AddGaussianDistribution(1, stddev, x1 + ((x1 - x2)*r));
}

void PointSet::AddGaussianDistributionAlongLine(int nsamples, float stddev, Point x1, Point x2, Point PMAX, Point PMIN)
{
    int i;
    float r;
    r = (float) rand()/RAND_MAX;
    for(i=0; i<nsamples; i++)
        AddGaussianDistribution(1, stddev, x1 + ((x1 - x2)*r), PMAX, PMIN);
}


void PointSet :: AddGaussianDistribution(int nsamples, float stddev, Point x)
{
    int i, j;
    float U, V;
    for(i=0; i<nsamples; i++)
    {
        addnum();
        for(j=0; j<DIM; j++)
            p[NUM - 1][j] = x[j];
        for(j=0; j<DIM; j++)
        {
            U = (float) rand()/RAND_MAX;
            V = (float) rand()/RAND_MAX;
            p[NUM - 1][j] += stddev*sqrt(-2*log(U))*cos(2*PI*V);            //Box-Muller Transform
        }
    }

    ctrl.clear();
    ctrl.push_back(x);

    ctrlfloat[0] = nsamples;
    ctrlfloat[1] = stddev;
}



void PointSet :: AddGaussianDistribution(int nsamples, float stddev, Point x, Point PMAX, Point PMIN)
{
    PointSet PS;
    bool b = 0;
    int i;
    PS.setdim(DIM);
    while(b == false)
    {
        b = true;
        for(i=0; i<DIM; i++) if(PS[0][i] > PMAX[i] || PS[0][i] < PMIN[i])
            {
                b = false;
                break;
            }
        PS.setnum(0);
        PS.AddGaussianDistribution(nsamples,stddev,x);
    }
    addPoint(PS[0]);
}

/*
void PointSet :: AddGaussianDistributionAlongSpline(int nsamples, float stddev, Spline s, float a, float b, int N)
{
    int i;
    float dt, R=0, S, t=a, L;
    dt = (b-a)/N;
    L = s.length(a,b);

    for(i=0; i<nsamples; i++)
    {
        R=0;
        t=a;
        S = L*((float) rand()/RAND_MAX);
        while(R < S)
        {
            R+=(s(t+dt)-s(t)).abs();
            t+=dt;
        }


        AddGaussianDistribution(1,stddev,s(t));
    }
}*/

void PointSet :: AddGaussianDistributionAlongSpline(int nsamples, float stddev, Spline s, float a, float b, int N)
{
    int i;
    float dt, R=0, t=a, L;
    std::vector<float> S;

    dt = (b-a)/N;
    L = s.length(a,b);

    for(i=0;i<nsamples;i++)
        S.push_back(L*((float) rand()/RAND_MAX));

    std::sort(S.begin(),S.end());

    for(i=0;i<nsamples;i++) {
        while(R < S[i]) {
            R+=(s(t+dt)-s(t)).abs();
            t+=dt;
        }
        AddGaussianDistribution(1,stddev,s(t));
    }


}

void PointSet :: AddGaussianDistributionAlongSpline(int nsamples, float stddev, Spline s)
{
    AddGaussianDistributionAlongSpline(nsamples, stddev, s, s.a, s.b, 100000);
    this->S = &s;
}

void PointSet::AddLinearDistribution(int nsamples, float radius, Point x)
{
    float r, lim;
    int i, j;
    Point P;
    P.setdim(DIM);
    for(i=0; i<nsamples; i++)
    {
    do {
        r = (radius*rand())/RAND_MAX;
        for(j=0; j< DIM; j++)
        {
            //lim = sqrt(r*r - S);
                P[j] = r*(2*rand() - 1)/RAND_MAX;
        }

    } while(P*P > radius*radius);


        //P[DIM - 1] = sqrt(r*r - S);

        addPoint(P+x);
    }
    type = PSet::LINEAR;

    ctrl.clear();
    ctrl.push_back(x);

    ctrlfloat[0] = nsamples;
    ctrlfloat[1] = radius;
}

void PointSet:: AddSimpleGaussianDistributionAlongSpline(int nsamples, float stddev, Spline s)
{
    float t = s.a, dt = (s.b - s.a)/nsamples;
    for(int i = 0; i<nsamples; i++)
    {
        AddGaussianDistribution(1, stddev, s(t));
        t+=dt;
    }
    this->S = &s;
}

void PointSet::DeleteDimension(int dim)
{
    int i;
    for(i=0; i<NUM; i++)
        p[i].DeleteDimension(dim);
    DIM--;
}





void PointSet::AddSpline(Spline S, float VoxelSize)
{
    float t=S.a, dt = (S.b - S.a)/1000;            //Quem  dt inicial??
    type = S.type;
    while(t<S.b)
    {
        addPoint(S(t));
        while((S(t+dt) - S(t)).abs() >= VoxelSize)
        {
            dt *= 0.9;
        }
        t += dt;
    }
    addPoint(S(S.b));

    this->S = &S;
}

void PointSet::AddSplineSimple(Spline S)
{
    float t, dt = (S.b - S.a)/1000;
    for(t=S.a; t<S.b; t+=dt) addPoint(S(t));
}

void PointSet::ImportFile(char name[], int EXT)
{
    FILE *file;
    file = fopen(name,"r");
    char str[10000];
    int i, j, offset, accoffset;
    if(EXT == STD)
    {
        for(i=0; !feof(file); i++)
        {
            accoffset = 0;
            setnum(i+1);
            if(i==0) setdim(1);
            fgets(str,10000,file);
            for(j=0; sscanf(str+accoffset," %f%n",&p[i][j],&offset) == 1; j++)
            {
                accoffset += offset;
                if(i==0) setdim(j+2);
            }
        }
        setdim(dim()-1);
    }
    fclose(file);
}

void PointSet::ExportFile(char str[], int EXT)
{
    FILE *file;
    file = fopen(str, "w");
    int i, j;
    if(EXT == STD)
    {
        for(i=0; i<NUM; i++)
        {
            for(j=0; j<DIM - 1; j++)
                fprintf(file, "%f ", p[i][j]);
            fprintf(file, "%f\n",p[i][j]);
        }
    }

    fclose(file);
}


void PointSet::free()
{
    for(int i = 0; i < NUM; i++) p[i].free();
}

void PointSet::print()
{
    int i, j;
    for(i=0; i<NUM; i++)
    {
        for(j=0; j<DIM; j++) printf("%.6f\t", p[i][j]);
        cout << endl;
    }
}

void PointSet::clear()
{
    setnum(0);
    // p.clear();
}


/*-----------fNode--------------*/

fNode::fNode()
{
    left = NULL;
    right = NULL;
}


float fNode::value(float t)
{
    if(right == NULL)
    {
        if(left == NULL) return Operation(t, 0, type, arg);
        else return Operation(left->value(t), 0, type, arg);        //Intil
    }
    if(left == NULL) return Operation(left->value(t), 0, type, arg);
    return Operation(left->value(t), right->value(t), type, arg);
}

float fNode::operator () (float t)
{
    return value(t);
}

void fNode::operator = (fNode fN)
{
    left = fN.left;
    right = fN.right;
    type = fN.type;
    arg = fN.arg;
}

/*-----------AlgFunction--------------*/

void AlgFunction::insert(unsigned int type, vector<float> arg)
{
    fNode fN2;
    fN2.type = type;
    fN2.arg = arg;
    fN.push_back(fN2);
}

void AlgFunction::insert(unsigned int type, float arg)
{
    vector<float> V;
    V.push_back(arg);
    insert(type, V);
}

void AlgFunction::insert(unsigned int type)
{
    fNode fN2;
    fN2.type = type;
    fN.push_back(fN2);
}

void AlgFunction::append(unsigned int x, unsigned int y, bool b)
{
    if(b==0) fN[x].left = &fN[y];
    else fN[x].right = &fN[y];
}

void AlgFunction::append(unsigned int x, AlgFunction F, bool b)
{
    int i, M;
    M = fN.size();
    for(i=0; i<F.fN.size(); i++)
    {
        fN.push_back(F.fN[i]);
        fN.back() = F.fN[i];
    }
    append(x, M, b);
}

void AlgFunction::linsert(unsigned int type)
{
    insert(type);
    append(fN.size() - 1, fN.size() - 2, 0);
}

void AlgFunction::rinsert(unsigned int type)
{
    insert(type);
    for(int i=fN.size()-1; i>=0; i--) if(fN[i].left != NULL) append(i,fN.size()-1, 1);
}

float AlgFunction:: operator () (float t)
{
    return fN[0].value(t);
}

void AlgFunction:: operator = (AlgFunction F)
{
    fN.clear();
    for(unsigned int i=0; i<F.fN.size(); i++) fN.push_back(F.fN[i]);
}

/*-----------Spline-------------*/

Spline::Spline()
{
    type = -1;
}

Point Spline:: operator() (float t)
{
    return value(t);
}

float Spline:: length()
{
    return length(a,b);
}

Point Spline::value(float t)
{
    Point P, Q;
    float p, r, s, dt;
    int i, j, k;
    unsigned int aux;
    vector<Point> U;
    switch(type)
    {
    case -1:
        for(unsigned int i = 0; i < size(); i++)
        {
            //P.setdim(i+1);
            P[i] = (at(i))(t);
        }
        return P;
        break;

    case CIRC_RC:
        P[0] = CtrlPt[0][0] + Ctrl[0]*cos(t);
        P[1] = CtrlPt[0][1] + Ctrl[0]*sin(t);
        P[2] = CtrlPt[0][2];
        return P;
        break;

    case CIRC_CP:
        r = (CtrlPt[1] - CtrlPt[0]).abs();
        P[0] = CtrlPt[0][0] + r*cos(t);
        P[1] = CtrlPt[0][1] + r*sin(t);
        P[2] = CtrlPt[0][2];
        return P;
        break;

    case CIRC_C2P:
        Q.setdim(3);
        Q = (((CtrlPt[1] - CtrlPt[0])^(CtrlPt[2] - CtrlPt[0]))^(CtrlPt[1] - CtrlPt[0]))*(1/(((CtrlPt[1] - CtrlPt[0])^(CtrlPt[2] - CtrlPt[0])^(CtrlPt[1] - CtrlPt[0])).abs()));
        P = CtrlPt[0] + (CtrlPt[1] - CtrlPt[0])*cos(t) + Q*((CtrlPt[1] - CtrlPt[0]).abs()*sin(t));
        return P;
        break;


        // case CIRC_3P:

        //case CIRC_NCP:

    case LINE:
        return (CtrlPt[0] + (CtrlPt[1] - CtrlPt[0])*t);
        break;

    case CUBIC:
        return (CtrlPt[0]*pow((1-t),3) + CtrlPt[1]*(3*t*pow(1-t,2)) + CtrlPt[2]*(3*t*t*(1-t))+CtrlPt[3]*t*t*t);
        break;

    case BEZIER:
        aux = CtrlPt.size();
        P.setdim(aux);
        for(i=0; i<aux; i++) P[i] = 0;
        for(i=0; i<aux; i++)
            P += CtrlPt[i]*(Binomial(aux-1,i)*pow(1-t,aux-1-i)*pow(t,i));
        return P;
        break;

        //case SPIRAL_CRNV:

    case SPIRAL_CAV:

        /*
        r = (CtrlPt[1] - CtrlPt[0]).abs();
        P[0] = CtrlPt[0][0] + r*cos(t);
        P[1] = CtrlPt[0][1] + r*sin(t);
        P[2] = Ctrl[0]*t;
        return P; break;*/

        Q.setdim(3);
        Q = (((CtrlPt[1] - CtrlPt[0])^(CtrlPt[2] - CtrlPt[0]))^(CtrlPt[1] - CtrlPt[0]))*(1/(((CtrlPt[1] - CtrlPt[0])^(CtrlPt[2] - CtrlPt[0])^(CtrlPt[1] - CtrlPt[0])).abs()));
        P = CtrlPt[0] + (CtrlPt[1] - CtrlPt[0])*cos(t) + Q*((CtrlPt[1] - CtrlPt[0]).abs()*sin(t)) +
            ((CtrlPt[1] - CtrlPt[0])^(CtrlPt[2] - CtrlPt[0])).versor()*(Ctrl[0]*t);
        return P;
        break;

        //case SPIRAL_CAB:

    case POLY:
        aux = CtrlPt.size();
        if(aux == 1) return CtrlPt[0];
        Q.setdim(CtrlPt[0].dim());
        for(i=0; i<Q.dim(); i++) Q[i] = 0;
        dt = (b-a)/(aux - 1);

        for(i=0; i<aux; i++)
        {
            p = 1;
            for(j=0; j<aux; j++)
                if(i!=j) p*=(t - j*dt)/(i*dt -j*dt);
            Q+=CtrlPt[i]*p;
        }
        return Q;
        break;

    }
}

float Spline:: length(float u, float v)
{
    const int N = 10000;
    float dt, S=0, t;
    dt = (v-u)/N;
    for(t=u; t<=v; t+=dt) S += (value(t+dt) - value(t)).abs();
    return S;
}

void Spline::Translation(Point Original, Point New)
{
    int i;
    if(type == -1)
    {
        for(i=0; i<size(); i++)
        {
            at(i).insert(PLUS);
            at(i).append(size()-1, 0, 0);
            at(i).insert(CONST, New[i] - Original[i]);
            at(i).append(size()-2, size()-1, 1);
        }
    }
    else for(i=0; i<CtrlPt.size(); i++) CtrlPt[i] += (New - Original);
}


/*-----------Spline::SetAs...----------*/

void Spline::SetAsCircumference(Point center, float radius)
{
    a=0;
    b=2*PI;
    CtrlPt.push_back(center);
    Ctrl.push_back(radius);
    type = CIRC_RC;
}

void Spline::SetAsCircumference(Point center, Point A)
{
    a=0;
    b=2*PI;
    CtrlPt.push_back(center);
    CtrlPt.push_back(A);
    type = CIRC_CP;
}

void Spline::SetAsCircumference(Point C, Point P, Point Q)
{
    a=0;
    b=2*PI;
    CtrlPt.push_back(C);
    CtrlPt.push_back(P);
    CtrlPt.push_back(Q);
    type = CIRC_C2P;
}

void Spline::SetAsCircumference(Point center, float radius, vector<float> normal)
{
    a=0;
    b=2*PI;
    CtrlPt.push_back(center);
    CtrlVector.push_back(normal);
    Ctrl.push_back(radius);
    type = CIRC_NCP;
}

void Spline::SetAsLine(Point A, Point B)
{
    a=0;
    b=1;
    CtrlPt.push_back(A);
    CtrlPt.push_back(B);
    type = LINE;
}

void Spline::SetAsCubicSpline (Point A, Point B)
{
    Point C, D;
    float R, M;
    R = (B - A).abs()/2;
    M = (B[1] - A[1])/(B[0] - A[0]);
    for(int i=2; i<A.dim(); i++)
    {
        C[i] = A[i];
        D[i] = B[i];
    }
    C[0] = A[0] + R*cos(atan(M) + PI/4);
    C[1] = A[1] + R*sin(atan(M) + PI/4);
    D[0] = B[0] + R*cos(atan(M) + 3*PI/4);
    D[1] = B[1] + R*sin(atan(M) + 3*PI/4);
    SetAsCubicSpline(A,B,C,D);
}

void Spline::SetAsCubicSpline (Point A, Point B, Point C, Point D)
{
    a=0;
    b=1;
    CtrlPt.push_back(A);
    CtrlPt.push_back(B);
    CtrlPt.push_back(C);
    CtrlPt.push_back(D);
    type = CUBIC;
}

void Spline::SetAsBezierSpline (vector<Point> V)
{
    a=0;
    b=1;
    CtrlPt = V;
    type = BEZIER;
}

void Spline::SetAsSpiral(Point center, float radius, vector<float> normal, float velocity)
{
    a=0;
    b=2*PI;
    CtrlPt.push_back(center);
    CtrlVector.push_back(normal);
    Ctrl.push_back(radius);
    Ctrl.push_back(velocity);
    type = SPIRAL_CRNV;
}


void Spline::SetAsSpiral(Point center, Point A, float velocity)
{
    a=0;
    b=2*PI;
    CtrlPt.push_back(center);
    CtrlPt.push_back(A);
    Ctrl.push_back(velocity);
    type = SPIRAL_CAV;
}

void Spline::SetAsSpiral(Point center, Point A, Point B, float velocity, int nloops)
{
    a=0;
    b=2*PI*nloops;
    CtrlPt.push_back(center);
    CtrlPt.push_back(A);
    CtrlPt.push_back(B);
    Ctrl.push_back(velocity);
    type = SPIRAL_CAV;
}

void Spline::SetAsSpiral(Point center, Point A, Point B)
{
    a=0;
    b=2*PI;
    CtrlPt.push_back(center);
    CtrlPt.push_back(A);
    CtrlPt.push_back(B);
    type = SPIRAL_CAB;
}

void Spline::SetAsPolynomial(vector<Point> V)
{
    a=0;
    b=1;
    CtrlPt = V;
    type = POLY;
}

void Spline::SetAsCyclicPolynomial(vector<Point> V)
{
    vector<Point> U;
    U = V;
    U.push_back(V[0]);
    SetAsPolynomial(U);
}

/*-----------DataList------------*/

DataList::DataList()
{
    DIM = 1;
    role = LABEL;
    for(int i=0; i<3; i++)
        CurrentDim[i] = 0;
}

void DataList::setdim(int dim)
{
    int i;
    DIM = dim;
    srand(time(NULL));
    for(i=0; i<size(); i++) at(i).setdim(DIM);
    for(i=0; i<3; i++) CurrentDim[i] = 0;
    for(i=0; i<dim; i++)
        CurrentDim[i] = i;
}

int DataList::dim()
{
    return DIM;
}

unsigned int DataList::NumberOfClasses()
{
    /*
    unsigned int i, n = 0;
    int aux = -1;
    for(i=0; i<size(); i++) if(aux != at(i).cl) {
        aux = at(i).cl;
        n++;    }
    return n;*/
    unsigned int i, j;
    std::vector<int> V;
    bool flag;
    for(i=0; i<size(); i++)
    {
        flag = false;
        for(j=0; j<V.size(); j++)
            if(at(i).cl == V[j])
            {
                flag = true;
                break;
            }
        if(flag == false) V.push_back(at(i).cl);
    }

    return V.size();

}

unsigned int DataList::NumberOfPoints()
{
    unsigned int i, n=0;
    for(i=0; i<size(); i++) n += at(i).num();
    return n;
}

void DataList::AddPoint(Point PT)
{
    PointSet PS;
    PS.setdim(DIM);
    PS.addPoint(PT);
    PS.role = role;
    push_back(PS);
}

void DataList::AddPoint(Point PT, int cl)
{
    /*int i;
    bool b = 0;
    for(i=0; i<size(); i++) if(at(i).cl == cl) {at(i).addPoint(PT); b=1;}
    if(b==0) {
        AddPoint(PT);
        back().cl = cl;
        }*/
    AddPoint(PT);
    back().setcl(cl);
}

void DataList::AddPointSet(PointSet PS)
{
    int i;
    for(i=0; i<PS.num(); i++) AddPoint(PS[i], PS.cl);
}

void DataList::AddGaussianDistribution(int nsamples, float stddev, Point x)
{
    PointSet PS;
    PS.type = GAUSSIAN;
    PS.setdim(DIM);
    PS.AddGaussianDistribution(nsamples,stddev,x);
    PS.role = role;
    push_back(PS);
}

void DataList::AddGaussianDistribution(int nsamples, float stddev, Point x, Point PMAX, Point PMIN)
{
    PointSet PS;
    PS.type = GAUSSIAN;
    PS.setdim(DIM);
    PS.AddGaussianDistribution(nsamples,stddev,x,PMAX, PMIN);
    PS.role = role;
    push_back(PS);
}

void DataList::AddGaussianDistribution(int nsamples, float stddev, Point x, int cl)
{
    PointSet PS;
    PS.type = GAUSSIAN;
    PS.setdim(DIM);
    PS.AddGaussianDistribution(nsamples,stddev,x);
    PS.role = role;
    PS.setcl(cl);
    push_back(PS);
}

void DataList::AddGaussianDistributionAlongLine(int nsamples, float stddev, Point x1, Point x2)
{
    PointSet PS;
    PS.type = SPLINEGAUSSIAN;
    PS.setdim(DIM);
    PS.AddGaussianDistributionAlongLine(nsamples, stddev, x1, x2);
    PS.role = role;
    push_back(PS);
}

void DataList::AddGaussianDistributionAlongLine(int nsamples, float stddev, Point x1, Point x2, Point PMAX, Point PMIN)
{
    PointSet PS;
    PS.type = SPLINEGAUSSIAN;
    PS.setdim(DIM);
    PS.AddGaussianDistributionAlongLine(nsamples, stddev, x1, x2, PMAX, PMIN);
    PS.role = role;
    push_back(PS);
}

void DataList::AddGaussianDistributionAlongSpline(int nsamples, float stddev, Spline s, float a, float b, int N)
{
    PointSet PS;
    PS.type = SPLINEGAUSSIAN;
    PS.setdim(DIM);
    PS.AddGaussianDistributionAlongSpline(nsamples, stddev, s, a, b, N);
    PS.role = role;
    push_back(PS);
}

void DataList::AddGaussianDistributionAlongSpline(int nsamples, float stddev, Spline s)
{
    PointSet PS;
    PS.type = SPLINEGAUSSIAN;
    PS.setdim(DIM);
    PS.AddGaussianDistributionAlongSpline(nsamples, stddev, s);
    PS.role = role;
    push_back(PS);
}

void DataList::AddGaussianDistributionAlongSpline(int nsamples, float stddev, Spline s, int cl)
{
    PointSet PS;
    PS.type = SPLINEGAUSSIAN;
    PS.setdim(DIM);
    PS.AddGaussianDistributionAlongSpline(nsamples, stddev, s);
    PS.role = role;
    PS.setcl(cl);
    push_back(PS);
}

void DataList::AddSimpleGaussianDistributionAlongSpline(int nsamples, float stddev, Spline s, int cl)
{
    PointSet PS;
    PS.type = SPLINEGAUSSIAN;
    PS.setdim(DIM);
    PS.role = role;
    float t = s.a, dt = (s.b - s.a)/nsamples;
    for(int i = 0; i<nsamples; i++)
    {
        PS.AddGaussianDistribution(1, stddev, s(t));
        t+=dt;
    }
    PS.setcl(cl);
    push_back(PS);
}

void DataList::AddSpline(Spline S, float VoxelSize)
{
    PointSet PS;
    SList.push_back(S);
    PS.setdim(DIM);
    PS.type = S.type;
    PS.AddSpline(S, VoxelSize);
    PS.role = role;
    push_back(PS);
}

void DataList::AddLinearDistribution(int nsamples, float radius, Point x, int cl)
{
    PointSet PS;
    PS.type = LINEAR;
    PS.setdim(DIM);
    PS.AddLinearDistribution(nsamples,radius,x);
    PS.role = role;
    PS.setcl(cl);
    push_back(PS);
}


void DataList::operator = (DataList DL)
{
    int i;
    clear();
    DIM = DL.dim();
    for(i=0; i<DL.size(); i++) push_back(DL[i]);
    SList = DL.SList;
    temp = DL.temp;
}

void DataList::operator *= (float f)
{
    int i;
    for(i=0; i<size(); i++) at(i)*=f;
}

void DataList::operator += (Point P)
{
    int i;
    if(P.dim() > DIM) setdim(P.dim());
    for(i=0; i<size(); i++) at(i) += P;
}

void DataList::operator += (DataList DL)
{
    int i;
    for(i=0; i<DL.size(); i++) AddPointSet(DL[i]);
}

PointSet DataList::FindPoint(Point P)
{
    int i,j;
    for(i=0; i<size(); i++) for(j=0; j<at(i).num(); j++) if(at(i)[j] == P) return at(i);
    PointSet PS;
    PS.setdim(DIM);
    return PS;
}

unsigned int DataList::FindPointIndex(Point P)
{
    unsigned int i,j;
    for(i=0; i<size(); i++) for(j=0; j<at(i).num(); j++) if(at(i)[j] == P) return i;
    return -1;
}

void DataList::ImportFile(char name[], int EXT)
{
    FILE *file;
    file = fopen(name,"r");
    char str[10000], str2[100];
    int i, j, k=0, offset, accoffset, index_new, index_old=0, npoints, nclusters, Dim;
    PointSet PS;
    clear();
    if(EXT == STD)
    {
        PS.setdim(1);
        for(i=0; !feof(file); i++)
        {
            accoffset = 0;
            PS.setnum(i+1);
            if(i==0) setdim(1);
            fgets(str,10000,file);
            for(j=0; sscanf(str+accoffset," %f%n",&PS[i][j],&offset) == 1; j++)
            {
                accoffset += offset;
                if(i==0) PS.setdim(j+2);
            }
        }
        setdim(dim()-1);
    }

    else if(EXT == SIMPLESTD)
    {
        accoffset = 0;
        fgets(str2, 100, file);
        sscanf(str2, "%d %d", &npoints, &Dim);
        setdim(Dim);
        PS.setdim(Dim);
        PS.setnum(npoints);

        for(i=0;i<npoints;i++)
        {
            accoffset = 0;
            fgets(str, 10000, file);

            for(j=0;j<Dim;j++)
            {
                sscanf(str+accoffset," %f%n", &PS[i][j], &offset);
                accoffset += offset;
            }

            sscanf(str+accoffset," %d %d", &PS[i].cl, &PS[i].classifcl);
        }
        push_back(PS);
    }

    else if(EXT == CLUTO)
    {
        accoffset = 0;
        fgets(str2, 100, file);
        sscanf(str2, "%d %d", &npoints, &nclusters);
        PS.setdim(1);
        for(i=0;i<npoints;i++) {
            accoffset = 0;
            fgets(str, 10000, file);
            for(j=0;sscanf(str+accoffset," %f%n", &PS[i][j], &offset) == 1; j++)
            {
                accoffset += offset;
                if(i==0)
                    PS.setdim(PS.dim()+1);
            }
        }
        push_back(PS);
    }

    else if(EXT == MATLAB)
    {
        fgets(str, 10000, file);
        PS.setdim(1);
        for(i=0;!feof(file);i++)
        {
            accoffset = 0;
            fgets(str, 10000, file);
            for(j=0;sscanf(str+accoffset," %f%n", &PS[i][j], &offset) == 1; j++)
            {
                accoffset += offset;
                if(i==0)
                    PS.setdim(PS.dim()+1);
            }
        }
        push_back(PS);
    }

    fclose(file);
}

void DataList::ExportFile(char str[], int EXT)
{
    FILE *file;

    unsigned int i, j, k, n;
    int l=0;

    if(EXT == STD)
    {
        file = fopen(str, "w");
        for(k=0; k<size(); k++)
            for(i=0; i<at(k).num(); i++)
            {
                fprintf(file, "%d ", k);
                for(j=0; j<at(k).dim(); j++)
                    fprintf(file, "%f ", at(k)[i][j]);
                fprintf(file, "%d %d\n",at(k)[i].cl, at(k)[i].classifcl);
            }
    }

    else if(EXT == SIMPLESTD)
    {
        file = fopen(str, "w");
        n = NumberOfPoints();
        fprintf(file, "%d %d\n", n, DIM);

        for(i=0;i<n;i++) {
            for(j=0;j<DIM;j++)
                fprintf(file, "%f ", pt(i)[j]);
            fprintf(file, "%d %d\n",pt(i).cl, pt(i).classifcl);
        }
    }

    else if(EXT == CLUTO)
    {
        file = fopen(str, "w");
        fprintf(file, "%d %d\n", NumberOfPoints(), DIM);
        for(k=0; k<size(); k++)
            for(i=0; i<at(k).num(); i++)
            {
                for(j=0; j<at(k).dim()-1; j++)
                    fprintf(file, "%f ", at(k)[i][j]);
                fprintf(file, "%f\n", at(k)[i][j]);

            }

    }

    else if(EXT == LIBOPF)
    {
        file = fopen(str, "wb");
        fprintf(file, "%d %d %d\n", NumberOfPoints(), NumberOfClasses(), DIM);
        for(k=0; k<size(); k++)
            for(j=0; j< (unsigned int) at(k).num(); j++)
            {
                fprintf(file, "%d %d ", l, at(k).cl);
                l++;
                for(i=0; i<DIM-1; i++)
                    fprintf(file, "%f ",at(k)[j][i]);
                fprintf(file, "%f\n", at(k)[j][i]);
            }
    }

    else if(EXT == MATLAB)
    {
        file = fopen(str, "w");
        for(k=0; k<size(); k++)
            for(j=0; j< (unsigned int) at(k).num(); j++)
            {
                for(i=0; i<DIM-1; i++)
                    fprintf(file, "%f ",at(k)[j][i]);
                fprintf(file, "%f\n", at(k)[j][i]);
            }
    }

    fclose(file);
}

void DataList::EraseClass(int cl)
{
    int i=0;
    for(i=0; i<size(); i++)
        while(at(i).cl == cl)
        {
            erase(begin()+i);
            if(i == size()) break;
        }

    for(i=0; i<size(); i++)
        if(at(i).cl > cl) at(i).cl--;
}

unsigned int DataList::PointSetIndex(int cl, unsigned int k)
{
    unsigned int i, j = 0;
    for(i=0; i<size(); i++)
    {
        if(at(i).cl == cl) j++;
        if(j == k+1) break;
    }

    return i;

}


void DataList::print()
{
    for(int i=0; i<size(); i++)
    {
        std::cout << "PointSet " << i << ":\n";
        at(i).print();
        std::cout << "\n";
    }
}

/*
Point& DataList::pt(int n)
{
    int i, j, m;
    for(i=0,m=0; i<size(); i++)
        for(j=0; j<at(i).num(); j++,m++)
            if(m == n)
                return (*this)[i][j];
}*/

Point& DataList::pt(int n)
{
    int i,m;
    for(i=0,m=0;i<size();i++)
    {
        if(n < m + at(i).num())
            return (*this)[i][n-m];
        else
            m += at(i).num();
    }
}

void DataList::DeleteDimension(int dim)
{
    int i;
    for(i=0; i<size(); i++)
        at(i).DeleteDimension(dim);
    DIM--;
}

float DataList :: StdDevSqr()
{
    int i, n = NumberOfPoints();
    Point avg;
    float S = 0;
    avg.setdim(DIM);
    for(i=0;i<n;i++)
        avg += pt(i)*((float) 1/n);
    for(i=0;i<n;i++)
        S += (pt(i) - avg)*(pt(i) - avg);
    S /= n;
    return S;
}

float DataList :: Density(Point P, int outdegree, float sqrsigma)
{
    float S = 0;
    int n = NumberOfPoints();
    for(int i=0;i<n;i++)
        S+=exp(-(P - pt(i)).sqrabs()/(2*sqrsigma));

    S /= (outdegree*sqrt(2*PI*sqrsigma));

    return S;
}

float DataList :: Accuracy()
{
    int i, n = NumberOfPoints();
    int right = 0;
    for(i=0;i<n;i++)
        if(pt(i).cl == pt(i).classifcl)
            right++;

    return ((float) ((float) right)/((float) n));
}

void DataList :: SwapClassifcl(int classifclA, int classifclB)
{
    if(classifclA != classifclB) {
    int i, n = NumberOfPoints();
    for(i=0; i<n; i++)
        if(pt(i).classifcl == classifclA)
            pt(i).classifcl = classifclB;
        else if(pt(i).classifcl == classifclB)
            pt(i).classifcl = classifclA;
    }
}

void DataList :: CorrectClasses()
{
    int i, j, bestj, n = NumberOfClasses();
    float acc, maxacc;
    for(i=0;i<n;i++) {
        maxacc = 0;
        for(j=0;j<n;j++) {
            SwapClassifcl(i,j);
            acc = Accuracy();
            SwapClassifcl(i,j);
            if(acc > maxacc) {
                maxacc = acc;
                bestj = j;
            }
        }
        SwapClassifcl(i, bestj);
    }
}

int DataList:: GetSplineId1(int id0)
{
    unsigned int i;
    int n=0;
    for(i=0;i<SList.size();i++)
        if(SList[i].id[0] == id0)
            n++;
    return n;
}

Spline * DataList :: GetSpline(int id0, int id1){
    int i;
    for(i=0;i<SList.size();i++)
        if(SList[i].id[0] == id0 && SList[i].id[1] == id1)
            return (&SList[i]);
}


/*-----------Voxel--------------*/

bool Voxel::isNeighboor(Voxel V)
{
    for(int i = 0; i < V.size(); i++) if(abs(V[i] - at(i)) > 1) return false;
    return true;
}

bool Voxel::operator == (Voxel V)
{
    for(int i = 0; i < V.size(); i++) if(V[i] != at(i)) return false;
    return true;
}

/*-----------VoxelSet--------------*/

vector <Voxel> VoxelSet::Neighboorhood(Voxel V)
{
    vector <Voxel> VX;
    for(unsigned int i = 0; i < size(); i++)
        if(V.isNeighboor(at(i)) == true)
            VX.push_back(at(i));
    return VX;
}

vector <Voxel> VoxelSet::Neighboorhood (unsigned int i)
{
    return Neighboorhood(at(i));
}

void VoxelSet::addPoint(Point P)
{
    Voxel V;
    int i;
    for(i = 0; i < P.dim(); i++) V.push_back(trunc(P[i]+0.5));
    for(i = 0; i < V.size(); i++) if(at(i) == V) return;
    push_back(V);
}

void VoxelSet::addPointSet (PointSet PS)
{
    for(int i = 0; i < PS.num(); i++) addPoint(PS[i]);
}

void VoxelSet::addSpline(Spline S)
{
    float t, dt = (S.b - S.a)/1000;
    for(t = S.a; t<=S.b; t+=dt)
        addPoint(S(t));
}

/*-----------Functions--------------*/

float Operation(float u, float v, unsigned int type, vector<float> arg)
{
    switch(type)
    {
    case CONST:
        if(arg.size() == 0) return 1;
        else return arg[0];
        break;
    case ID:
        if(arg.size() == 0) return u;
        else return arg[0]*u;
        break;
    case SIN:
        if(arg.size() == 0) return sin(u);
        else return sin(u*arg[0]);
        break;
    case COS:
        if(arg.size() == 0) return cos(u);
        else return cos(u*arg[0]);
        break;
    case TAN:
        if(arg.size() == 0) return tan(u);
        else return tan(u*arg[0]);
        break;
    case ASIN:
        if(arg.size() == 0) return asin(u);
        else return asin(u*arg[0]);
        break;
    case ACOS:
        if(arg.size() == 0) return acos(u);
        else return acos(u*arg[0]);
        break;
    case ATAN:
        if(arg.size() == 0) return atan(u);
        else return atan(u*arg[0]);
        break;
    case SINH:
        if(arg.size() == 0) return sinh(u);
        else return sinh(u*arg[0]);
        break;
    case COSH:
        if(arg.size() == 0) return cosh(u);
        else return cosh(u*arg[0]);
        break;
    case TANH:
        if(arg.size() == 0) return tanh(u);
        else return tanh(u*arg[0]);
        break;
    case EXP:
        return exp(u);
        break;
    case LN:
        if(arg.size() == 0) return log(u);
        else return log(u+arg[0]);
        break;
    case POW:
        return pow(u,v);
        break;
    case PLUS:
        return (u+v);
        break;
    case MINUS:
        return (u-v);
        break;
    case TIMES:
        return (u*v);
        break;
    case DIV:
        if(v!=0) return(u/v);
        else return 0;
        break;
    case SQR:
        return (u*u);
    case SQRT:
        return (sqrt(u));
    default:
        return 0;
    }
}

int Binomial(int n, int k)
{
    float P = 1;
    int u;
    u = n - k;
    if(k==0 || u==0) return 1;
    while(n>0)
    {
        P *= (float) n/(k*u);
        n--;
        if(k>1) k--;
        if(u>1) u--;
    }
    return ((int) P);
}

}       //end PSet namespace


