
#include "ClassificationDialog.h"

namespace Classify {

    BEGIN_EVENT_TABLE(ClassificationDialog, BaseDialog)
    EVT_SIZE    (ClassificationDialog::OnSize)
  END_EVENT_TABLE()


  ClassificationDialog :: ClassificationDialog(wxWindow *parent) :
    BaseDialog(parent, (char *)"Classification Methods"){

    wxSize size(400, 500);
    wxColour wxcolor;

    this->SetMinSize(size);
    this->SetSize(size);

    this->selection = 0;
    this->NClusters = 1;
    this->Kmax = 50;
    this->fvol = 0.4;
    this->param = APP->Data.DL.StdDevSqr();
    this->stop_value = 0.1;
    this->SpectralCutValue = 0.5;
    this->outdegree = 5;
    this->pparam = 1;
    this->qparam = 1;
    this->OPFftype = 0;
    this->GFTGraph = 0;
    this->GFTsigma = APP->Data.DL.StdDevSqr();
    this->CorrectWeights = false;
    this->OPFsigma = false;

    int i;
    for(i=0;i<10;i++)
        controlint[i] = 0;
    for(i=0;i<3;i++)
        GFTfunc[i] = 0;

    panel = new BasePanel(this);

    panel->sizer->AddSpacer(20);
    panel->sizer->SetSizeHints(panel);
    panel->sizer->Layout();
    this->AddPanel(panel, 0);

    panel->Show(true);


    this->optPanel = NULL;

    //----------ObjectPanel-------------

    this->clasPanel = this->CreateMethodsPanel();

    this->optPanel = CreateOptionsPanel();

    this->AddPanel((wxPanel *) optPanel);

    this->AddPanel((wxPanel *)clasPanel, 0.4);

    APP->RefreshAll();

  }


  ClassificationDialog::~ ClassificationDialog(){
    if(v_panel_bkg != NULL) free(v_panel_bkg);
    //delete ch;
  }




  void  ClassificationDialog::DestroyOptionsPanel(){
    if(optPanel!=NULL){

/*
      Disconnect( this->ids[0], wxEVT_COMMAND_SPINCTRL_UPDATED,
		  wxSpinEventHandler(ManualElementDialog::OnSpinNPoints),
		  NULL, NULL );
      Disconnect( this->ids[1], wxEVT_COMMAND_SPINCTRL_UPDATED,
		  wxSpinEventHandler(ManualElementDialog::OnSpinStddev),
		  NULL, NULL );*/

      if(panel->sizer->GetItem(optPanel,false)!=NULL)
	panel->sizer->Detach(optPanel);
      if(optPanel->GetSizer() != NULL)
	optPanel->GetSizer()->Clear(true);
      optPanel->Destroy();
      optPanel = NULL;
    }

  }


  wxScrolledWindow * ClassificationDialog::CreateOptionsPanel(){

    int sel;
    wxSize size(250, 1000);

    wxScrolledWindow *swind = new wxScrolledWindow(this, wxID_ANY,
						   wxDefaultPosition,
						   wxDefaultSize,
						   wxHSCROLL | wxVSCROLL,
						   _T("scrolledWindow"));

    swind->SetMinSize(size);

    sel = ch->GetSelection();


    swind->SetScrollRate(5, 5);
    swind->Scroll(0, 0);
    swind->SetVirtualSize(250,1000);

    size.SetHeight(1000);
    size.SetWidth(250);

    wxBoxSizer *vsizer = new wxStaticBoxSizer(wxVERTICAL, swind, _T("Options"));


    if(sel == OPFClustering) {

        wxSpinCtrl *spKmax;
        SpinFloat *spfvol;
        wxStaticText *tKmax = new wxStaticText(swind, -1, _T("K Max"), wxDefaultPosition,
        wxDefaultSize, wxALIGN_LEFT, _T("staticText1"));

        wxStaticText *tfvol = new wxStaticText(swind, -1, _T("Vol/Area/Height Factor"), wxDefaultPosition,
        wxDefaultSize, wxALIGN_LEFT, _T("staticText1"));

        spfvol = new SpinFloat(swind, wxID_ANY, 0.0, 100.0, fvol, 0.1, 5);

        spKmax = new wxSpinCtrl(swind, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize,
        wxSP_ARROW_KEYS, 1, 10000, Kmax, _T("wxKMax"));

        wxStaticText *tftype = new wxStaticText(swind, -1, _T("Factor type"), wxDefaultPosition,
        wxDefaultSize, wxALIGN_LEFT, _T("staticText1"));

        wxArrayString str;
        str.Add(_T("Volume"));
        str.Add(_T("Area"));
        str.Add(_T("Height"));
        wxChoice *chOPF = new wxChoice(swind, wxID_ANY, wxDefaultPosition, wxDefaultSize, str, OPFftype, wxDefaultValidator, _T("wxOPFChoice"));

        wxBoxSizer *hbsKmax;
        hbsKmax = new wxBoxSizer(wxHORIZONTAL);
        hbsKmax->Add(tKmax,  1, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
        hbsKmax->Add(spKmax, 0, wxALIGN_RIGHT);
        vsizer->Add(hbsKmax, 0, wxEXPAND);

        wxBoxSizer *hbsfvol;
        hbsfvol = new wxBoxSizer(wxHORIZONTAL);
        hbsfvol->Add(tfvol,  1, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
        hbsfvol->Add(spfvol, 0, wxALIGN_RIGHT);
        vsizer->Add(hbsfvol, 0, wxEXPAND);

        wxBoxSizer *hbsftype;
        hbsftype = new wxBoxSizer(wxHORIZONTAL);
        hbsftype->Add(tftype, 1, wxALIGN_CENTER|wxALIGN_CENTER_VERTICAL);
        hbsftype->Add(chOPF, 0, wxALIGN_RIGHT);
        vsizer->Add(hbsftype, 0, wxEXPAND);

        Connect(spKmax->GetId(), wxEVT_COMMAND_SPINCTRL_UPDATED,
	     wxCommandEventHandler(ClassificationDialog::OnSpinKmax),
	     NULL, NULL );

        Connect(spfvol->GetId(), wxEVT_COMMAND_SPINCTRL_UPDATED,
	     wxCommandEventHandler(ClassificationDialog::OnSpinfvol),
	     NULL, NULL );

        Connect(chOPF->GetId(), wxEVT_COMMAND_CHOICE_SELECTED,
	     wxCommandEventHandler(ClassificationDialog::OnChoiceOPFftype),
	     NULL, NULL);

    }

    else if(sel == OPFKNNClassify)
    {

    }

    else if(sel == CLUTO) {

        wxArrayString str;
        str.Add(_T("R. Bis."));
        str.Add(_T("rbr"));
        str.Add(_T("Direct"));
        str.Add(_T("Agglo"));
        str.Add(_T("Graph"));
        str.Add(_T("Bagglo"));
        wxChoice *chclMethod, *chsim;
        chclMethod = new wxChoice(swind, wxID_ANY, wxDefaultPosition, wxDefaultSize, str, 0, wxDefaultValidator, _T("wxclMethodChoice"));

        str.clear();
        str.Add(_T("Cosine"));
        str.Add(_T("Corr. Coef."));
        str.Add(_T("Distance"));
        str.Add(_T("Jaccard"));         //Only applicable if clMethod = Graph
        chsim = new wxChoice(swind, wxID_ANY, wxDefaultPosition, wxDefaultSize, str, 0, wxDefaultValidator, _T("wxsimChoice"));

        str.clear();
        str.Add(_T("i1"));
        str.Add(_T("i2"));
        str.Add(_T("e1"));
        str.Add(_T("g1"));
        str.Add(_T("g1p"));
        str.Add(_T("h1"));
        str.Add(_T("h2"));
        str.Add(_T("slink"));
        str.Add(_T("wslink"));
        str.Add(_T("clink"));
        str.Add(_T("wclink"));
        str.Add(_T("upgma"));
        chcrfun = new wxChoice(swind, wxID_ANY, wxDefaultPosition, wxDefaultSize, str, 0, wxDefaultValidator, _T("wxcrfunChoice"));


        wxStaticText *tclMethod = new wxStaticText(swind, -1, _T("CLUTO Method"), wxDefaultPosition,
        wxDefaultSize, wxALIGN_LEFT, _T("staticText1"));
        wxStaticText *tsim = new wxStaticText(swind, -1, _T("Similarity Fun."), wxDefaultPosition,
        wxDefaultSize, wxALIGN_LEFT, _T("staticText1"));
        wxStaticText *tcrfun = new wxStaticText(swind, -1, _T("Criterion Fun."), wxDefaultPosition,
        wxDefaultSize, wxALIGN_LEFT, _T("staticText1"));



        wxSpinCtrl *spNClusters;
        wxStaticText *tNClusters = new wxStaticText(swind, -1, _T("N.Clusters"), wxDefaultPosition,
        wxDefaultSize, wxALIGN_LEFT, _T("staticText1"));

        /*wxBoxSizer *hbsclMethod;
        hbsclMethod = new wxBoxSizer(wxHORIZONTAL);
        hbsclMethod->Add(tclMethod, 1, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
        hbsclMethod->Add(chclMethod, 1, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);*/


        spNClusters = new wxSpinCtrl(swind, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize,
        wxSP_ARROW_KEYS, 1, 10000, NClusters, _T("wxNClusters"));

        wxButton *butDefault = new wxButton(swind, wxID_ANY, _T("Default"), wxDefaultPosition,
		       wxDefaultSize, wxALIGN_LEFT,
		       wxDefaultValidator, _T("butDefault"));

        wxBoxSizer *hbsNClusters;
        hbsNClusters = new wxBoxSizer(wxHORIZONTAL);
        hbsNClusters->Add(tNClusters,  1, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
        hbsNClusters->Add(spNClusters, 0, wxALIGN_RIGHT);

        wxBoxSizer *hbscrfun;
        hbscrfun = new wxBoxSizer(wxHORIZONTAL);
        hbscrfun->Add(chcrfun, 0,  wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
        //hbscrfun->AddSpacer(40);
        hbscrfun->Add(butDefault, 0, wxALIGN_RIGHT);

        vsizer->Add(tclMethod, 0,wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
        vsizer->Add(chclMethod, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
        vsizer->Add(tsim, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
        vsizer->Add(chsim, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
        vsizer->Add(tcrfun, 0,wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
        vsizer->Add(hbscrfun, 0, wxEXPAND);
        vsizer->AddSpacer(10);
        vsizer->Add(hbsNClusters, 0, wxEXPAND);

        Connect(chclMethod->GetId(), wxEVT_COMMAND_CHOICE_SELECTED,
                wxCommandEventHandler(ClassificationDialog::OnChoiceClMethod));

        Connect(chsim->GetId(), wxEVT_COMMAND_CHOICE_SELECTED,
                wxCommandEventHandler(ClassificationDialog::OnChoiceSim));

        Connect(chcrfun->GetId(), wxEVT_COMMAND_CHOICE_SELECTED,
                wxCommandEventHandler(ClassificationDialog::OnChoiceCrFun));

        Connect(spNClusters->GetId(), wxEVT_COMMAND_SPINCTRL_UPDATED,
	     wxCommandEventHandler(ClassificationDialog::OnSpinNClusters),
	     NULL, NULL );

        Connect(butDefault->GetId(), wxEVT_COMMAND_BUTTON_CLICKED,
                wxCommandEventHandler(ClassificationDialog::OnClickDefault));

    }

    else if(sel == SPECTRAL || sel == SPECTRAL_NORMALIZEDCUT) {

        wxArrayString str;
        str.Add(_T("cos"));
        str.Add(_T("p/d"));
        str.Add(_T("p/d²"));
        str.Add(_T("-p*d"));
        str.Add(_T("-p*d²"));
        str.Add(_T("exp(-d²/2p)"));
        wxChoice *chspecsimfun;
        chspecsimfun = new wxChoice(swind, wxID_ANY, wxDefaultPosition, wxDefaultSize, str, 0, wxDefaultValidator, _T("wxSpectralSimFun"));
        wxStaticText *tsimfun = new wxStaticText(swind, -1, _T("Simmetry Func."), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT, _T("staticText1"));

        spfparam_spec = new SpinFloat(swind, wxID_ANY, -10000.0, 10000.0, param, 1, 5);
        wxStaticText *tparam = new wxStaticText(swind, -1, _T("Parameter"), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT, _T("staticTextParam"));
        SpinFloat *spfcutvalue = new SpinFloat(swind, wxID_ANY, 0, 1, SpectralCutValue, 0.01, 5);
        wxStaticText *tcutvalue = new wxStaticText(swind, -1, _T("Cut Value"), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT, _T("staticTextSpecCuValue"));

        wxButton *butDefault = new wxButton(swind, wxID_ANY, _T("Default"), wxDefaultPosition,
                                            wxDefaultSize, wxALIGN_LEFT, wxDefaultValidator, _T("butDefault"));

        wxBoxSizer *hbsparam = new wxBoxSizer(wxHORIZONTAL);
        hbsparam->Add(spfparam_spec, 0,  wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
        hbsparam->Add(butDefault, 0, wxALIGN_RIGHT);

        vsizer->Add(tsimfun, 0, wxALIGN_CENTER_VERTICAL);
        vsizer->Add(chspecsimfun, 0, wxALIGN_CENTER_VERTICAL);
        vsizer->AddSpacer(20);
        vsizer->Add(tparam, 0, wxALIGN_CENTER_VERTICAL);
        vsizer->Add(hbsparam, 0, wxEXPAND);
        vsizer->AddSpacer(20);
        vsizer->Add(tcutvalue, 0, wxALIGN_CENTER_VERTICAL);
        vsizer->Add(spfcutvalue, 0, wxALIGN_CENTER_VERTICAL);

        Connect(spfparam_spec->GetId(), wxEVT_COMMAND_SPINCTRL_UPDATED,
	     wxCommandEventHandler(ClassificationDialog::OnSpinSpectralParam),
	     NULL, NULL );

        Connect(chspecsimfun->GetId(), wxEVT_COMMAND_CHOICE_SELECTED,
                wxCommandEventHandler(ClassificationDialog::OnChoiceSpecSimFun));

        Connect(spfcutvalue->GetId(), wxEVT_COMMAND_SPINCTRL_UPDATED,
	     wxCommandEventHandler(ClassificationDialog::OnSpinSpectralCutValue),
	     NULL, NULL );

        Connect(butDefault->GetId(), wxEVT_COMMAND_BUTTON_CLICKED,
                wxCommandEventHandler(ClassificationDialog::OnClickDefault));

    }

    else if(sel == SPECTRAL_NCLUSTERS)
    {

        wxArrayString str;
        str.Add(_T("cos"));
        str.Add(_T("p/d"));
        str.Add(_T("p/d²"));
        str.Add(_T("-p*d"));
        str.Add(_T("-p*d²"));
        str.Add(_T("exp(-d²/2p²)"));
        wxChoice *chspecsimfun;
        chspecsimfun = new wxChoice(swind, wxID_ANY, wxDefaultPosition, wxDefaultSize, str, 0, wxDefaultValidator, _T("wxSpectralSimFun"));
        wxStaticText *tsimfun = new wxStaticText(swind, -1, _T("Simmetry Func."), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT, _T("staticText1"));

        wxSpinCtrl *spNClusters;
        wxStaticText *tNClusters = new wxStaticText(swind, -1, _T("N.Clusters"), wxDefaultPosition,
        wxDefaultSize, wxALIGN_LEFT, _T("staticText1"));

        spNClusters = new wxSpinCtrl(swind, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize,
        wxSP_ARROW_KEYS, 1, 10000, NClusters, _T("wxNClusters"));

        spfparam_nspec = new SpinFloat(swind, wxID_ANY, -10000.0, 10000.0, param, 0.1, 5);
        wxStaticText *tparam = new wxStaticText(swind, -1, _T("Parameter"), wxDefaultPosition,
                                   wxDefaultSize, wxALIGN_LEFT, _T("staticTextParam"));

        SpinFloat *spfstop_value = new SpinFloat(swind, wxID_ANY, -10000.0, 10000.0, stop_value, 0.1, 5);
        wxStaticText *tstop_value = new wxStaticText(swind, -1, _T("Stop Value"), wxDefaultPosition,
                                   wxDefaultSize, wxALIGN_LEFT, _T("staticTextStopValue"));

        wxButton *butDefault = new wxButton(swind, wxID_ANY, _T("Default"), wxDefaultPosition,
                                            wxDefaultSize, wxALIGN_LEFT, wxDefaultValidator, _T("butDefault"));

        wxBoxSizer *hbsparam = new wxBoxSizer(wxHORIZONTAL);
        hbsparam->Add(spfparam_nspec, 0,  wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
        hbsparam->Add(butDefault, 0, wxALIGN_RIGHT);

        vsizer->Add(tsimfun, 0, wxALIGN_CENTER_VERTICAL);
        vsizer->Add(chspecsimfun, 0, wxALIGN_CENTER_VERTICAL);
        vsizer->AddSpacer(20);
        vsizer->Add(tparam, 0, wxALIGN_CENTER_VERTICAL);
        vsizer->Add(hbsparam, 0, wxEXPAND);
        vsizer->AddSpacer(20);
        vsizer->Add(tNClusters, 0, wxALIGN_CENTER_VERTICAL);
        vsizer->Add(spNClusters, 0, wxALIGN_CENTER_VERTICAL);
        vsizer->AddSpacer(20);
        vsizer->Add(tstop_value, 0, wxALIGN_CENTER_VERTICAL);
        vsizer->Add(spfstop_value, 0, wxALIGN_CENTER_VERTICAL);

        Connect(chspecsimfun->GetId(), wxEVT_COMMAND_CHOICE_SELECTED,
                wxCommandEventHandler(ClassificationDialog::OnChoiceSpecSimFun));

        Connect(spfparam_nspec->GetId(), wxEVT_COMMAND_SPINCTRL_UPDATED,
	     wxCommandEventHandler(ClassificationDialog::OnSpinSpectralParam),
	     NULL, NULL );

        Connect(spNClusters->GetId(), wxEVT_COMMAND_SPINCTRL_UPDATED,
	     wxCommandEventHandler(ClassificationDialog::OnSpinNClusters),
	     NULL, NULL );

        Connect(spfparam_nspec->GetId(), wxEVT_COMMAND_SPINCTRL_UPDATED,
	     wxCommandEventHandler(ClassificationDialog::OnSpinStopValue),
	     NULL, NULL );

	     Connect(butDefault->GetId(), wxEVT_COMMAND_BUTTON_CLICKED,
                wxCommandEventHandler(ClassificationDialog::OnClickDefault));



    }

    else if(sel == KMEANS)
    {

        wxSpinCtrl *spNClusters;
        wxStaticText *tNClusters = new wxStaticText(swind, -1, _T("N.Clusters (K)"), wxDefaultPosition,
        wxDefaultSize, wxALIGN_LEFT, _T("staticText1"));

        spNClusters = new wxSpinCtrl(swind, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize,
        wxSP_ARROW_KEYS, 1, 10000, NClusters, _T("wxNClusters"));

        SpinFloat *spfstopvalue = new SpinFloat(swind, wxID_ANY, -10000.0, 10000.0, stop_value, 1, 5);
        wxStaticText *tstopvalue = new wxStaticText(swind, -1, _T("Stop Value"),
        wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT, _T("staticTextStopValue"));

        vsizer->Add(tNClusters, 0, wxALIGN_CENTER_VERTICAL);
        vsizer->Add(spNClusters, 0, wxALIGN_CENTER_VERTICAL);
        vsizer->AddSpacer(20);
        vsizer->Add(tstopvalue, 0, wxALIGN_CENTER_VERTICAL);
        vsizer->Add(spfstopvalue, 0, wxALIGN_CENTER_VERTICAL);

        Connect(spNClusters->GetId(), wxEVT_COMMAND_SPINCTRL_UPDATED,
	     wxCommandEventHandler(ClassificationDialog::OnSpinNClusters),
	     NULL, NULL );

        Connect(spfstopvalue->GetId(), wxEVT_COMMAND_SPINCTRL_UPDATED,
	     wxCommandEventHandler(ClassificationDialog::OnSpinSpectralParam),
	     NULL, NULL );

    }


    else if(sel == GFT)
    {

        wxSpinCtrl *spGFTNClusters, *spOutdegree;
        wxStaticText *tNClusters = new wxStaticText(swind, -1, _T("N.Clusters (K)"), wxDefaultPosition,
        wxDefaultSize, wxALIGN_LEFT, _T("staticText1"));

        wxStaticText *tOutdegree = new wxStaticText(swind, -1, _T("Outdegree"), wxDefaultPosition,
        wxDefaultSize, wxALIGN_LEFT, _T("staticText2"));

        spGFTNClusters = new wxSpinCtrl(swind, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize,
        wxSP_ARROW_KEYS, 1, 10000, NClusters, _T("wxGFTNClusters"));

        spOutdegree = new wxSpinCtrl(swind, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize,
        wxSP_ARROW_KEYS, 1, 10000, outdegree, _T("wxOutdegree"));

        wxBoxSizer *vbsNClusters;
        vbsNClusters = new wxBoxSizer(wxVERTICAL);
        vbsNClusters->Add(tNClusters, 1, wxEXPAND);
        vbsNClusters->Add(spGFTNClusters, 0, wxEXPAND);

        wxBoxSizer *vbsOutdegree;
        vbsOutdegree = new wxBoxSizer(wxVERTICAL);
        vbsOutdegree->Add(tOutdegree, 1, wxEXPAND);
        vbsOutdegree->Add(spOutdegree, 0, wxEXPAND);

        wxBoxSizer *hbsNCOD;
        hbsNCOD = new wxBoxSizer(wxHORIZONTAL);
        hbsNCOD->Add(vbsNClusters, 0, wxEXPAND);
        hbsNCOD->AddSpacer(20);
        hbsNCOD->Add(vbsOutdegree, 0, wxEXPAND);

        SpinFloat *spfP, *spfQ;
        spfP = new SpinFloat(swind, wxID_ANY, 0.0, 100.0, pparam, 0.1, 5);
        spfQ = new SpinFloat(swind, wxID_ANY, 0.0, 100.0, qparam, 0.1, 5);
        wxStaticText *tP = new wxStaticText(swind, -1, _T("p"), wxDefaultPosition,
        wxDefaultSize, wxALIGN_LEFT, _T("staticTextp"));
        wxStaticText *tQ = new wxStaticText(swind, -1, _T("q"), wxDefaultPosition,
        wxDefaultSize, wxALIGN_LEFT, _T("staticTextq"));

        spfSigma = new SpinFloat(swind, wxID_ANY, 0.0, 100000.0, GFTsigma, 1, 5);
        wxStaticText *tsigma = new wxStaticText(swind, -1, _T("σ"), wxDefaultPosition,
        wxDefaultSize, wxALIGN_LEFT, _T("staticTextp"));


        wxButton *butDefault = new wxButton(swind, wxID_ANY, _T("Default"), wxDefaultPosition,
		       wxDefaultSize, wxALIGN_LEFT,
		       wxDefaultValidator, _T("butDefaultGFT"));



        wxBoxSizer *hbsP;
        hbsP = new wxBoxSizer(wxHORIZONTAL);
        hbsP->Add(tP,  1, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
        hbsP->AddSpacer(20);
        hbsP->Add(spfP, 0, wxALIGN_RIGHT);

        wxBoxSizer *hbsQ;
        hbsQ = new wxBoxSizer(wxHORIZONTAL);
        hbsQ->Add(tQ,  1, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
        hbsQ->AddSpacer(20);
        hbsQ->Add(spfQ, 0, wxALIGN_RIGHT);

        wxBoxSizer *hbssigma;
        hbssigma = new wxBoxSizer(wxHORIZONTAL);
        hbssigma->Add(tsigma, 1, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
        hbssigma->AddSpacer(20);
        hbssigma->Add(spfSigma, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
        hbssigma->Add(butDefault, 0, wxALIGN_RIGHT);


        wxArrayString str;
        str.Add(_T("Average"));
        str.Add(_T("Max Value"));
        str.Add(_T("Min Value"));
        str.Add(_T("Normalized Average"));
        str.Add(_T("Normalized Max Value"));

        wxChoice *chArcDen, *chSubGraphDen, *chArcW, *chGraph;
        chArcDen = new wxChoice(swind, wxID_ANY, wxDefaultPosition, wxDefaultSize, str, GFTfunc[0], wxDefaultValidator, _T("wxchArcDen"));

        str.clear();
        str.Add(_T("Min(Sa,Sb)"));
        str.Add(_T("Min(Max Densities)"));
        str.Add(_T("Normalized Min(Sa,Sb)"));
        str.Add(_T("Normalized Min(Max Dens.)"));
        chSubGraphDen= new wxChoice(swind, wxID_ANY, wxDefaultPosition, wxDefaultSize, str, GFTfunc[1], wxDefaultValidator, _T("wxchSGD"));

        str.clear();
        str.Add(_T("AD/SGD"));
        str.Add(_T("pAD - qSGD"));
        str.Add(_T("SGD*exp(-pAD)"));
        str.Add(_T("min(Sum(max{d(i)-AD, 0}))"));

        chArcW = new wxChoice(swind, wxID_ANY, wxDefaultPosition, wxDefaultSize, str, GFTfunc[2], wxDefaultValidator, _T("wxcrfunChoice"));

        str.clear();
        str.Add(_T("Gabriel"));
        str.Add(_T("Gabriel Knn"));

        chGraph = new wxChoice(swind, wxID_ANY, wxDefaultPosition, wxDefaultSize, str, GFTGraph, wxDefaultValidator, _T("wxcrfunChoice"));

        wxStaticText *tArcDen = new wxStaticText(swind, -1, _T("Arc Density"), wxDefaultPosition,
        wxDefaultSize, wxALIGN_LEFT, _T("staticTextArcDen"));
        wxStaticText *tSubGraphDen = new wxStaticText(swind, -1, _T("SubGraph Density"), wxDefaultPosition,
        wxDefaultSize, wxALIGN_LEFT, _T("staticTextSubGraphDen"));
        wxStaticText *tArcW = new wxStaticText(swind, -1, _T("Arc Weight"), wxDefaultPosition,
        wxDefaultSize, wxALIGN_LEFT, _T("staticTextArcW"));
        wxStaticText *tGraph = new wxStaticText(swind, -1, _T("Graph"), wxDefaultPosition,
        wxDefaultSize, wxALIGN_LEFT, _T("staticTextArcW"));


        wxCheckBox *chboxCorrection;
        chboxCorrection = new wxCheckBox(swind, wxID_ANY, _T("Correct Weights"),
        wxDefaultPosition, wxDefaultSize, CorrectWeights, wxDefaultValidator, _T("WxCheckBox"));

        wxCheckBox *chboxOPFsigmadef;
        chboxOPFsigmadef = new wxCheckBox(swind, wxID_ANY, _T("σ = max{d(s,t)}/3"),
        wxDefaultPosition, wxDefaultSize, OPFsigma, wxDefaultValidator, _T("WxCheckBoxSigmaOPFdef"));

        vsizer->Add(hbsNCOD, 0, wxALIGN_CENTER_VERTICAL);
        vsizer->AddSpacer(20);

        vsizer->Add(hbsP, 0, wxALIGN_CENTER_VERTICAL);
        vsizer->Add(hbsQ, 0, wxALIGN_CENTER_VERTICAL);
        vsizer->Add(hbssigma, 0, wxALIGN_CENTER_VERTICAL);
        vsizer->AddSpacer(20);

        vsizer->Add(tGraph, 0, wxALIGN_CENTER_VERTICAL);
        vsizer->Add(chGraph, 0, wxALIGN_CENTER_VERTICAL);
        vsizer->AddSpacer(10);
        vsizer->Add(tArcDen, 0, wxALIGN_CENTER_VERTICAL);
        vsizer->Add(chArcDen, 0, wxALIGN_CENTER_VERTICAL);
        vsizer->AddSpacer(10);
        vsizer->Add(tSubGraphDen, 0, wxALIGN_CENTER_VERTICAL);
        vsizer->Add(chSubGraphDen, 0, wxALIGN_CENTER_VERTICAL);
        vsizer->AddSpacer(10);
        vsizer->Add(tArcW, 0, wxALIGN_CENTER_VERTICAL);
        vsizer->Add(chArcW, 0, wxALIGN_CENTER_VERTICAL);
        vsizer->AddSpacer(10);
        vsizer->Add(chboxOPFsigmadef, 0, wxALIGN_CENTER_VERTICAL);
        vsizer->Add(chboxCorrection, 0, wxALIGN_CENTER_VERTICAL);
        vsizer->AddSpacer(20);


        Connect(spGFTNClusters->GetId(), wxEVT_COMMAND_SPINCTRL_UPDATED,
	     wxCommandEventHandler(ClassificationDialog::OnSpinNClusters),
	     NULL, NULL );

        Connect(spOutdegree->GetId(), wxEVT_COMMAND_SPINCTRL_UPDATED,
	     wxCommandEventHandler(ClassificationDialog::OnSpinOutdegree),
	     NULL, NULL );

        Connect(spfP->GetId(), wxEVT_COMMAND_SPINCTRL_UPDATED,
	     wxCommandEventHandler(ClassificationDialog::OnSpinPparam),
	     NULL, NULL );

	    Connect(spfQ->GetId(), wxEVT_COMMAND_SPINCTRL_UPDATED,
	     wxCommandEventHandler(ClassificationDialog::OnSpinQparam),
	     NULL, NULL );

	    Connect(chArcDen->GetId(), wxEVT_COMMAND_CHOICE_SELECTED,
	     wxCommandEventHandler(ClassificationDialog::OnChoiceArcDen),
	     NULL, NULL );

	    Connect(chSubGraphDen->GetId(), wxEVT_COMMAND_CHOICE_SELECTED,
	     wxCommandEventHandler(ClassificationDialog::OnChoiceSGDen),
	     NULL, NULL );

	    Connect(chArcW->GetId(), wxEVT_COMMAND_CHOICE_SELECTED,
	     wxCommandEventHandler(ClassificationDialog::OnChoiceArcW),
	     NULL, NULL );

	    Connect(chGraph->GetId(), wxEVT_COMMAND_CHOICE_SELECTED,
	     wxCommandEventHandler(ClassificationDialog::OnChoiceGFTGraph),
	     NULL, NULL );

        Connect(butDefault->GetId(), wxEVT_COMMAND_BUTTON_CLICKED,
                wxCommandEventHandler(ClassificationDialog::OnClickDefault),
        NULL, NULL  );


        Connect(chboxCorrection->GetId(), wxEVT_COMMAND_CHECKBOX_CLICKED,
                wxCommandEventHandler(ClassificationDialog::OnCheckCorrectWeights));

        Connect(chboxOPFsigmadef->GetId(), wxEVT_COMMAND_CHECKBOX_CLICKED,
                wxCommandEventHandler(ClassificationDialog::OnCheckOPFSigma));

        Connect(spfSigma->GetId(), wxEVT_COMMAND_SPINCTRL_UPDATED,
            wxCommandEventHandler(ClassificationDialog::OnSpinGFTsigma),
            NULL, NULL);
    }

    else std::cout << "\nError - Invalid method selection\n";

    //vsizer->Add(but, 0, wxEXPAND);
    vsizer->AddSpacer(10);

    swind->SetSizer(vsizer, true);
    vsizer->SetSizeHints(swind);
    vsizer->Layout();

    swind->SetVirtualSize(250,600);

    wxPaintDC paintDC(swind);
    swind->DoPrepareDC(paintDC);



    return (swind);
  }



  wxScrolledWindow * ClassificationDialog::CreateMethodsPanel(){

    wxSize size(250, 200);
    wxScrolledWindow *swind = new wxScrolledWindow(this, wxID_ANY,
						   wxDefaultPosition,
						   wxDefaultSize,
						   wxHSCROLL | wxVSCROLL,
						   _T("scrolledWindow"));

    swind->SetMinSize(size);

    v_panel_bkg = NULL;

    swind->SetScrollRate(5, 5);
    swind->Scroll(0, 0);
    swind->SetVirtualSize(250,200);

      size.SetHeight(200);
      size.SetWidth(250);

      /*Connect( id, wxEVT_COMMAND_BUTTON_CLICKED,
	       wxCommandEventHandler(ManualElementDialog::OnDelete),
	       NULL, NULL );*/

    wxArrayString str;
    str.Add(_T("OPF Clustering"));
    str.Add(_T("OPF KNN Classify"));
    str.Add(_T("CLUTO"));
    str.Add(_T("Spectral - Average Association"));
    str.Add(_T("Spectral NClusters - Average Cut"));
    str.Add(_T("Spectral - Normalized Cut"));
    str.Add(_T("K Means"));
    str.Add(_T("GFT"));

    ch = new wxChoice(swind, wxID_ANY, wxDefaultPosition, wxDefaultSize, str, 0, wxDefaultValidator, _T("wxChoice"));

    ch->SetSelection(selection);

    Connect(ch->GetId(), wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler(ClassificationDialog::OnChoice));


    wxString *Vtypechoices;
    Vtypechoices = new wxString[2];
    Vtypechoices[0] = _T("Truelabel");
    Vtypechoices[1] = _T("Classified label");

    rbView = new wxRadioBox (swind, wxID_ANY, _T("Data view"), wxDefaultPosition, wxDefaultSize, 2, Vtypechoices,
                           1, wxRA_SPECIFY_COLS, wxDefaultValidator, _T("wxRadioBoxView"));

    rbView->Enable(1, false);

    sbAcc = new wxStatusBar(swind, wxID_ANY, 0, _T("wxStatusBarAcc"));
    sbAcc->SetStatusText(_T("No data was classified"), 0);


    Connect(rbView->GetId(), wxEVT_COMMAND_RADIOBOX_SELECTED, wxCommandEventHandler(ClassificationDialog::OnChoiceViewType));

    wxBoxSizer *vsizer = new wxStaticBoxSizer(wxVERTICAL, swind, _T("Methods"));

    vsizer->Add(ch, 0, wxEXPAND);
    vsizer->AddSpacer(10);
    vsizer->Add(rbView, 0, wxEXPAND);
    vsizer->AddSpacer(10);
    vsizer->Add(sbAcc, 0,  wxALIGN_LEFT);
    vsizer->AddSpacer(10);


    swind->SetSizer(vsizer, true);
    vsizer->SetSizeHints(swind);
    vsizer->Layout();

    swind->SetVirtualSize(250,200);


    wxPaintDC paintDC(swind);
    swind->DoPrepareDC(paintDC);



    return (swind);
  }


  void  ClassificationDialog::DestroyMethodsPanel(){

      /*Disconnect( id, wxEVT_COMMAND_BUTTON_CLICKED,
		  wxCommandEventHandler(ManualElementDialog::OnDelete),
		  NULL, NULL );*/

    if(clasPanel!=NULL){
      if(sizer->GetItem(clasPanel,false)!=NULL)
	sizer->Detach(clasPanel);
      clasPanel->Destroy();
      clasPanel = NULL;
    }
  }


  void  ClassificationDialog :: RefreshDialog(){

    wxSize size(250, 200);
    this->DestroyMethodsPanel();
    this->DestroyOptionsPanel();
    this->SetMinSize(size);
    //this->SetSize(size);

    this->clasPanel = this->CreateMethodsPanel();

    this->optPanel = CreateOptionsPanel();

    this->AddPanel((wxPanel *) optPanel);

    this->AddPanel((wxPanel *)clasPanel, 0.4);



    //this->RefreshOptions();

    this->Fit();
    this->Update();
  }


  void  ClassificationDialog :: RefreshOptions(){

    this->DestroyOptionsPanel();
    optPanel = this->CreateOptionsPanel();

    if(optPanel != NULL){
      this->panel->sizer->Prepend(optPanel, 1, wxEXPAND);
          }
    this->panel->sizer->SetSizeHints(this->panel);
    this->panel->sizer->Layout();

    this->Update();
  }

  void  ClassificationDialog:: OnCancel(wxCommandEvent& event){
        //mod->stop();
  //controlint[0] = -1;

  }

  void  ClassificationDialog :: OnOk(wxCommandEvent& event){

    wxBeginBusyCursor();
    if(selection == OPFClustering) {

    Subgraph *g;
    int i, j, u, jmax;
    float acc=0, max_acc, Hmax=0;
    double Vmax=0;
    g = DLtoSubgraph(APP->Data.DL);
    opf_BestkMinCut(g,1,Kmax);

    switch(OPFftype){
	  case 2:
	    for (i=0; i < g->nnodes; i++)
	      if (g->node[i].dens > Hmax)
		Hmax = g->node[i].dens;
	    opf_ElimMaxBelowH(g, fvol*Hmax);
	    break;
	  case 1:
	    opf_ElimMaxBelowArea(g, (int)(fvol*g->nnodes));
	    break;
	  case 0:
	    for (i=0; i < g->nnodes; i++)
	      Vmax += g->node[i].dens;
	    opf_ElimMaxBelowVolume(g, (int)(fvol*Vmax/g->nnodes));
	    break;
	  default:
	    printf("Error in OPFClustering - Invalid filtering type\n");
	    break;
	  }

    opf_OPFClustering(g);

    ReadFromSubgraph(g, &APP->Data.DL);

    printf("\nOPF: num of clusters = %d\n",g->nlabels);
/*
    g->nlabels = 0;
	  for (i = 0; i < g->nnodes; i++){
	    if (g->node[i].root==i)
	      g->node[i].label = g->node[i].truelabel;
	    else
	      g->node[i].label = g->node[g->node[i].root].truelabel;
	  }

	  for (i = 0; i < g->nnodes; i++){

		  if (g->node[i].label > g->nlabels)
			  g->nlabels = g->node[i].label;
	  }
	  */

    for (i = 0; i < g->nnodes; i++)
	    g->node[i].truelabel = g->node[i].label+1;

    opf_WriteModelFile(g, "classifier.opf");


    //printf("Accuracy: %.2f%%\n", opf_Accuracy(g)*100);

    //DestroySubgraph(&g);


    }

    else if(selection == OPFKNNClassify) {

      Subgraph *gTrain, *gTest;

      gTest = DLtoSubgraph(APP->Data.DL);
      gTrain = opf_ReadModelFile("classifier.opf");

      opf_OPFKNNClassify(gTrain, gTest);

      ReadFromSubgraph(gTest, &APP->Data.DL);

        //DestroySubgraph(&gTrain);
        //DestroySubgraph(&gTest);

    }

    else if(selection == CLUTO) {

    CallCluto(&APP->Data.DL, NClusters, controlint[0], controlint[1], controlint[2]);
    ReadCluto(&APP->Data.DL, NClusters);

    }

    else if(selection == SPECTRAL) {
            if(APP->Data.DL.size() != 0) {
                SpectralClustering(controlint[0], param, SpectralCutValue, &APP->Data.DL);
            }
    }

    else if(selection == SPECTRAL_NCLUSTERS) {
            if(APP->Data.DL.size() != 0)
            SpectralClusteringNClusters(controlint[0], param, NClusters, stop_value, &APP->Data.DL);
    }

    else if(selection == SPECTRAL_NORMALIZEDCUT) {
        if(APP->Data.DL.size() != 0)
            NormalizedCutSpectralClustering(controlint[0], param, SpectralCutValue, &APP->Data.DL);
    }

    else if(selection == KMEANS) {
            KMeansClassify(&APP->Data.DL, NClusters, param);
    }

    else if(selection == GFT) {
            GFT_Clustering(&APP->Data.DL, NClusters, outdegree, pparam, qparam, GFTfunc, GFTGraph, GFTsigma, CorrectWeights, OPFsigma);
    }

    APP->Data.DL.CorrectClasses();
    sbAcc->SetStatusText(wxString::Format(wxT("Accuracy = %.2f%%"),100*APP->Data.DL.Accuracy()), 0);
    rbView->Enable(1, true);
    rbView->SetSelection(1);
    APP->RefreshAll(APP->Data.DL);
    wxEndBusyCursor();
  }


  void  ClassificationDialog :: OnSize(wxSizeEvent& event){
    BaseDialog::OnSize(event);
  }

  void ClassificationDialog ::  OnChoice(wxCommandEvent& event){

  selection = ch->GetSelection();
  RefreshDialog();

  }

  void ClassificationDialog :: OnChoiceClMethod(wxCommandEvent& event){
    controlint[0] = event.GetInt();
  }

  void ClassificationDialog :: OnChoiceSim(wxCommandEvent& event){
    int x = event.GetInt();
    wxObject  *O;
    wxChoice *ch;
    if((x == 2 || x == 3 )&& controlint[0] != 4) {
        wxMessageBox(_T("This simmetry function is only allowed if Method = Graph."));
        O = event.GetEventObject();
        ch = (wxChoice *) O;
        ch->SetSelection(0);

    }
    else
        controlint[1] = x;
  }

  void ClassificationDialog :: OnChoiceCrFun(wxCommandEvent& event){
    controlint[2] = event.GetInt();
  }

  void ClassificationDialog :: OnChoiceSpecSimFun(wxCommandEvent& event){
    controlint[0] = event.GetInt();
  }


  void ClassificationDialog :: OnSpinNClusters(wxCommandEvent& event){
  NClusters = event.GetInt();
  }

  void ClassificationDialog :: OnSpinKmax(wxCommandEvent& event){
  Kmax = event.GetInt();
  }

  void ClassificationDialog :: OnSpinfvol(wxCommandEvent& event){
   wxObject  *O;
   SpinFloat *S;
   O = event.GetEventObject();
   S = (SpinFloat *)O;
   fvol = S->GetValue();
  }

  void ClassificationDialog :: OnClickDefault(wxCommandEvent& event){
  if(selection == CLUTO) {
    if(controlint[0] <= 2) {
        chcrfun->SetSelection(1);
        controlint[2] = 1;}

    else if(controlint[0] == 3 || controlint[0] == 5) {
        chcrfun->SetSelection(11);
        controlint[2] = 11; }
    }
  else if(selection == SPECTRAL || selection == SPECTRAL_NORMALIZEDCUT)
  {
      param = APP->Data.DL.StdDevSqr();
      spfparam_spec->SetValue(param);
  }
  else if(selection == SPECTRAL_NCLUSTERS) {
      param = APP->Data.DL.StdDevSqr();
      spfparam_nspec->SetValue(param);
  }
  else if(selection == GFT)
  {
    GFTsigma = APP->Data.DL.StdDevSqr();
    spfSigma->SetValue(GFTsigma);
  }
}

  void ClassificationDialog :: OnSpinSpectralParam(wxCommandEvent& event)
  {
         wxObject  *O;
         SpinFloat *S;
         O = event.GetEventObject();
         S = (SpinFloat *)O;
         param = S->GetValue();
  }

  void ClassificationDialog :: OnSpinSpectralCutValue(wxCommandEvent& event)
  {
         wxObject  *O;
         SpinFloat *S;
         O = event.GetEventObject();
         S = (SpinFloat *)O;
         SpectralCutValue = S->GetValue();
  }

  void ClassificationDialog :: OnSpinStopValue(wxCommandEvent& event)
  {
         wxObject  *O;
         SpinFloat *S;
         O = event.GetEventObject();
         S = (SpinFloat *)O;
         stop_value = S->GetValue();
  }

  void ClassificationDialog :: OnSpinOutdegree(wxCommandEvent& event)
  {
      outdegree = event.GetInt();
  }

  void ClassificationDialog :: OnChoiceViewType(wxCommandEvent& event)
  {
      int sel = event.GetSelection();
      if(sel == 0)
      {
        APP->UpdateLabel();
        APP->UpdateDensityScene();
      }
      else {

        APP->UpdateLabel();
        APP->UpdateDensityScene(APP->Data.DL);
      }

      APP->Refresh2DCanvas();
      APP->Refresh3DCanvas(true, 1.0);
  }

  void ClassificationDialog :: OnSpinPparam(wxCommandEvent& event)
  {
         wxObject  *O;
         SpinFloat *S;
         O = event.GetEventObject();
         S = (SpinFloat *)O;
         pparam = S->GetValue();
  }

  void ClassificationDialog :: OnSpinQparam(wxCommandEvent& event)
  {
         wxObject  *O;
         SpinFloat *S;
         O = event.GetEventObject();
         S = (SpinFloat *)O;
         qparam = S->GetValue();
  }

  void ClassificationDialog :: OnSpinGFTsigma(wxCommandEvent& event)
  {
         wxObject  *O;
         SpinFloat *S;
         O = event.GetEventObject();
         S = (SpinFloat *)O;
         GFTsigma = S->GetValue();
  }

  void ClassificationDialog :: OnChoiceArcDen(wxCommandEvent& event)
  {
    GFTfunc[0] = event.GetSelection();
  }

  void ClassificationDialog :: OnChoiceSGDen(wxCommandEvent& event)
  {
    GFTfunc[1] = event.GetSelection();
  }

  void ClassificationDialog :: OnChoiceArcW(wxCommandEvent& event)
  {
    GFTfunc[2] = event.GetSelection();
  }

  void ClassificationDialog :: OnChoiceOPFftype(wxCommandEvent& event)
  {
      OPFftype = event.GetSelection();
  }

  void ClassificationDialog :: OnChoiceGFTGraph(wxCommandEvent& event)
  {
      GFTGraph = event.GetSelection();
  }

  void ClassificationDialog :: OnCheckCorrectWeights(wxCommandEvent& event)
  {
    CorrectWeights = !CorrectWeights;
  }
  void ClassificationDialog :: OnCheckOPFSigma(wxCommandEvent& event)
  {
    OPFsigma = !OPFsigma;
  }

} //end Classify namespace


