
#ifndef _MANUALDIALOG_H_
#define _MANUALDIALOG_H_

#include "startnewmodule.h"
#include "modulemanual.h"
#include "painthandler.h"

namespace Manual{

class ManualElementDialog : public BaseDialog{
public:
    ManualElementDialog(wxWindow *parent,
			ModuleManual *mod);
    ~ManualElementDialog();

  void OnDelete(wxCommandEvent& event);


  void OnCancel(wxCommandEvent& event);
  void OnOk(wxCommandEvent& event);

  void OnSize(wxSizeEvent& event);

  void OnSpinStddev(wxSpinEvent& event);
  void OnSpinNPoints(wxSpinEvent& event);
  void OnSpinNLoops(wxSpinEvent& event);
  void OnSpinVelocity(wxSpinEvent& event);
  void OnResort(wxCommandEvent& event);
  void OnSpinLinRadius(wxSpinEvent& event);

  void ChangeSelection(wxPanel *bkg);
  void RefreshDialog();
  void RefreshOptions();

private:
  wxScrolledWindow *CreateElementPanel();
  void              DestroyElementPanel();
  int               FindNumberOfElements();
  char            **FindElementNames();
  int               FindCurrentElementType();
  wxScrolledWindow *CreateOptionsPanel();
  void              DestroyOptionsPanel();
  PSet::PointSet         *GetPointSet();
  int               GetSplineStartIndex();

  ModuleManual *mod;
  BasePanel  *panel;

  int elem_sel;

  //ElementPanel:
  wxScrolledWindow *elemPanel;
  int                  nelems;
  wxPanel            **v_panel_bkg;
  wxBitmapButton     **v_but_trash;

  int ids[100];
  int nids;

  //OptionsPanel:
  wxScrolledWindow *optPanel;

DECLARE_EVENT_TABLE()
  };


//--------------------------------


class ManualDialog : public BaseDialog{
public:
  typedef enum {NAVIGATOR=0,
		PAINT} ModeType;

  ManualDialog(wxWindow *parent,
	       ModuleManual *mod);
  ~ManualDialog();

  void OnChangeMode(wxCommandEvent& event);
  void OnChangeDrawMode(wxCommandEvent& event);
  void OnChangeDrawMode2(wxCommandEvent& event);
  void OnReset(wxCommandEvent& event);
  void OnUndo(wxCommandEvent& event);

  void OnCheckSimpleDistribution(wxCommandEvent& event);

  void OnAddObj(wxCommandEvent& event);
  void OnDelete(wxCommandEvent& event);
  void OnChangeColor(wxCommandEvent& event);
  void OnChangeVisibility(wxCommandEvent& event);

  void OnCancel(wxCommandEvent& event);
  void OnOk(wxCommandEvent& event);
  void OnSize(wxSizeEvent& event);
  void ChangeSelection(wxPanel *bkg);



  ManualDialog::ModeType GetOperationMode();
  Manual::DrawMode GetDrawMode();

  AdjRel   *GetBrush();
  wxCursor *GetBrushCursor(int zoom);
  void      NextBrush();
  void      PrevBrush();

  ManualElementDialog *elem_dialog;

private:
  wxScrolledWindow *CreateObjectPanel();
  void              DestroyObjectPanel();

  int id_but;
  int id_but2;
  int id_but3;
  int id_bp;
  int id_res;
  int id_undo;
  int id_add;
  PaintHandler       *xhandler;
  PaintHandler       *yhandler;
  PaintHandler       *zhandler;
  ModuleManual       *mod;

  BitmapRadioButton  *but;
  BitmapRadioButton  *but2;
  BitmapRadioButton  *but3;
  BrushPicker        *bPicker;
  wxButton           *undo;
  wxButton           *res;
  wxButton           *addobj;
  BasePanel  *panel;

  //ObjectPanel:
  wxScrolledWindow *objPanel;
  wxPanel            **v_panel_bkg;
  AlphaColourButton  **v_but_color;
  BitmapToggleButton **v_but_eye;
  wxBitmapButton     **v_but_trash;

DECLARE_EVENT_TABLE()
  };


//--------------------------------

class ObjBkgPanel : public wxPanel{
public:
  ObjBkgPanel(wxWindow *parent,
	      const wxPoint& pos,
	      const wxSize& size,
	      BaseDialog *dialog,
	      int type);
  ~ObjBkgPanel();
  void OnMouseEvent(wxMouseEvent& event);

private:
  BaseDialog *dialog;
  int type;
DECLARE_EVENT_TABLE()
  };

//--------------------------------

class ObjLabel : public wxStaticText{
public:
  ObjLabel(wxWindow *parent,
	   const wxString& label,
	   BaseDialog *dialog,
	   int type);
  ~ObjLabel();
  void OnMouseEvent(wxMouseEvent& event);

private:
  BaseDialog *dialog;
  int type;
DECLARE_EVENT_TABLE()
  };


} //end Manual namespace

#endif

