#include "painthandler.h"

namespace Manual{

  PaintHandler :: PaintHandler(char axis,
			       ModuleManual *mod){
    this->axis = axis;
    this->mod = mod;
    mode = 0;


  }

  PaintHandler :: ~PaintHandler(){}

  void PaintHandler :: ChangeDrawMode(DrawMode mode)
  {
      this->mode = mode;
      APP->Data.labelDL.temp.clear();
  }

  void PaintHandler :: OnEnterWindow(){
    wxCursor *cursor;
    float zoom = APP->GetZoomLevel();

    cursor = mod->GetBrushCursor(ROUND(zoom));
    APP->Set2DViewCursor(cursor, axis);
  }

  void PaintHandler :: OnLeaveWindow(){
    APP->Set2DViewCursor(wxSTANDARD_CURSOR, axis);
  }


  void PaintHandler :: OnLeftClick(int p){
    PSet::Point x;
    PSet::Spline S;
    PSet::PointSet PS;
    Subgraph *g = NULL;
    PSet::DataList V;
    int i;

    wxBeginBusyCursor();

     PS.setdim(APP->Data.DL.dim());
     x.setdim(APP->Data.DL.dim());
     x = APP->Data.StaticPoint;
     x[APP->Data.dim[0]] = VoxelX(APP->Data.den[0], p);
     x[APP->Data.dim[1]] = VoxelY(APP->Data.den[0], p);
     x[APP->Data.dim[2]] = VoxelZ(APP->Data.den[0], p);
     x.cl = mod->obj_sel;


    switch(mode)
    {

    case ADDSPLINE:


        APP->Data.labelDL.temp.push_back(x);
        S.SetAsPolynomial(APP->Data.labelDL.temp);
        if(APP->Data.labelDL.temp.size() != 1) {
                    PS.AddSpline(S, 1);
                    PS.setcl(mod->obj_sel);
                    PS.type = PSet::POLY;
                    APP->Data.labelDL.back() = PS;
                            }
        else {
                APP->Data.labelDL.AddPoint(x,mod->obj_sel);
                APP->Data.labelDL.back().type = PSet::POLY;
                }


    break;

    case ADDBEZIER:

        APP->Data.labelDL.temp.push_back(x);
        S.SetAsBezierSpline(APP->Data.labelDL.temp);
        if(APP->Data.labelDL.temp.size() != 1) {
                    PS.AddSpline(S, 1);
                    PS.setcl(mod->obj_sel);
                    PS.type = PSet::BEZIER;
                    APP->Data.labelDL.back() = PS;
                            }
        else {
                APP->Data.labelDL.AddPoint(x,mod->obj_sel);
                APP->Data.labelDL.back().type = PSet::BEZIER;
                }
    break;

    case ADDGAUSSIAN:
        APP->Data.DL.AddGaussianDistribution(APP->Control.NPoints, APP->Control.stddev, x);
        APP->Data.DL.back().setcl(mod->obj_sel);
    break;

    case ADDOPF:

    if(APP->Control.path[0] != 0) {
        g = ReadSubgraph(APP->Control.path);
        V = SubgraphtoDL(g);
        V*=10;
        V+=x;
        for(i=0;i<V.size();i++) {
        V[i].type = PSet::GAUSSIAN;
        }
        APP->Data.DL += V;
    }
    break;

    case ADDCIRC:
        if(APP->Data.labelDL.temp.size() < 2) APP->Data.labelDL.temp.push_back(x);
        else
        {
            S.SetAsCircumference(APP->Data.labelDL.temp[0], APP->Data.labelDL.temp[1], x);
            S.id[0] = mod->obj_sel;
            S.id[1] = APP->Data.labelDL.GetSplineId1(S.id[0]);
            APP->Data.labelDL.AddSpline(S, 0.1);
            APP->Data.labelDL.temp.clear();
            APP->Data.labelDL.SList.push_back(S);
            APP->Data.labelDL.back().setcl(mod->obj_sel);
            APP->Control.CurrentSpline = &APP->Data.labelDL.SList.back();

        }
    break;

    case ADDSPIRAL:
        if(APP->Data.labelDL.temp.size() < 2) APP->Data.labelDL.temp.push_back(x);
        else
        {
            S.SetAsSpiral(APP->Data.labelDL.temp[0], APP->Data.labelDL.temp[1], x, APP->Control.Velocity, APP->Control.NLoops);
            S.id[0] = mod->obj_sel;
            S.id[1] = APP->Data.labelDL.GetSplineId1(S.id[0]);
            APP->Data.labelDL.AddSpline(S, 0.1);
            APP->Data.labelDL.back().setcl(mod->obj_sel);
            APP->Data.labelDL.temp.clear();
            APP->Data.labelDL.SList.push_back(S);
            APP->Control.CurrentSpline = &APP->Data.labelDL.SList.back();
        }
        break;

    case ADDLINEAR:
        APP->Data.DL.AddLinearDistribution(APP->Control.NPoints, APP->Control.LinRadius, x, mod->obj_sel);
        APP->Data.DL.back().setcl(mod->obj_sel);
        break;

    case ADDGAUSSIANSPLINE:
    if(APP->Control.CurrentSpline != NULL) {
        if(APP->Control.simple == false)
            APP->Data.DL.AddGaussianDistributionAlongSpline(APP->Control.NPoints, APP->Control.stddev, *APP->Control.CurrentSpline, mod->obj_sel);
        else
            APP->Data.DL.AddSimpleGaussianDistributionAlongSpline(APP->Control.NPoints, APP->Control.stddev, *APP->Control.CurrentSpline, mod->obj_sel);
    }
    break;

    case ADDLINEARSPLINE:
        //APP->Data.DL.AddLinearDistribution(APP->Control.NPoints, APP->Control.LinRadius, x, mod->obj_sel);
        //APP->Data.DL.back().setcl(mod->obj_sel);
        break;

    }

    APP->UpdateLabel();
    APP->UpdateDensityScene();
    APP->Refresh2DCanvas();
    APP->Refresh3DCanvas(true,1.0);
    mod->RefreshDialogs();
    wxEndBusyCursor();
  }

  void PaintHandler :: OnRightClick(int p){


    PSet::Spline S;
    int i;
    wxBeginBusyCursor();
    switch(mode)
    {
    case ADDSPLINE:
        S.SetAsPolynomial(APP->Data.labelDL.temp);
        S.id[0] = mod->obj_sel;
        S.id[1] = APP->Data.labelDL.GetSplineId1(S.id[0]);
        APP->Data.labelDL.pop_back();
        APP->Data.labelDL.AddSpline(S, 1);
        APP->Data.labelDL.back().setcl(mod->obj_sel);
        APP->Data.labelDL.SList.push_back(S);
        APP->Data.labelDL.temp.clear();

        APP->Control.CurrentSpline = &APP->Data.labelDL.SList.back();
    break;

    case ADDBEZIER:
        S.SetAsBezierSpline(APP->Data.labelDL.temp);
        S.id[0] = mod->obj_sel;
        S.id[1] = APP->Data.labelDL.GetSplineId1(S.id[0]);
        APP->Data.labelDL.pop_back();
        APP->Data.labelDL.AddSpline(S, 1);
        APP->Data.labelDL.back().setcl(mod->obj_sel);
        APP->Data.labelDL.SList.push_back(S);
        APP->Data.labelDL.temp.clear();


    break;

    }

    APP->UpdateDensityScene();
    APP->UpdateLabel();
    APP->Refresh2DCanvas();
    APP->Refresh3DCanvas(true,1.0);
    mod->RefreshDialogs();

    wxEndBusyCursor();

}

  void PaintHandler :: OnMiddleClick(int p){

    wxBeginBusyCursor();
    APP->UpdateDensityScene();
    APP->UpdateLabel();
    APP->Refresh2DCanvas();
    APP->Refresh3DCanvas(true,1.0);
    mod->RefreshDialogs();
    wxEndBusyCursor();

    }

  void PaintHandler :: OnMouseWheel(int p, int rotation, int delta){

    //if(rotation>0) mod->NextBrush();
   // else           mod->PrevBrush();

    char msg[1024];

    wxCursor *cursor;
    float zoom = APP->GetZoomLevel();

    cursor = mod->GetBrushCursor(ROUND(zoom));
    APP->Set2DViewCursor(cursor, axis);

    switch(mode)
    {

    case ADDSPIRAL:
    if(rotation>0)              APP->Control.Velocity++;
    else if(APP->Data.Raio>0)   APP->Control.Velocity--;

    sprintf(msg,"APP->Control.Velocity = %.2f",APP->Control.Velocity);

    break;

    default:
    if(rotation>0)              APP->Data.Raio++;
    else if(APP->Data.Raio>0)   APP->Data.Raio--;

    sprintf(msg,"Radius = %d",APP->Data.Raio);
    break;
    }

    wxString wxmsg(msg, wxConvUTF8);
    APP->Window->SetStatusText(wxmsg);

  }

  void PaintHandler :: OnLeftDrag(int p, int q){

    /*AdjRel *A;
    int o;

    o = mod->obj_sel;
    if(o<0) return;

    A = mod->GetBrush();
    //APP->DrawBrushTrace(p,q, o+1, A,axis);
    DestroyAdjRel(&A);
    APP->Refresh2DCanvas();*/
  }

  void PaintHandler :: OnRightDrag(int p, int q){

/*
    PSet::Point x;
    x.setdim(APP->Data.DL.dim());
    int i;
    x[APP->Data.dim[0]] = VoxelX(APP->Data.den[0], p);
    x[APP->Data.dim[1]] = VoxelY(APP->Data.den[0], p);
    x[APP->Data.dim[2]] = VoxelZ(APP->Data.den[0], p);
    /*
    PSet::Spline S;
    //if(APP->Data.DL.SList.size() > 1)
    //{
        //for(i=0; i<3;i++) std::cout << APP->Data.DL.SList.back()(0)[i] << "\t";
        //std::cout << std::endl;
        APP->Data.DL.pop_back();
        APP->Data.DL.SList.back().Translation(APP->Data.DL.SList.back()(0), x);
        //for(i=0; i<3;i++) std::cout << APP->Data.DL.SList.back()(0)[i] << "\t";
        //std::cout << std::endl;
        //APP->Data.DL.AddSpline(APP->Data.DL.SList.back(), 0.1);
        //APP->Data.DL.SList.pop_back();
    //  }

    APP->UpdateLabel();
    APP->UpdateDensityScene();
    APP->Refresh2DCanvas(); */
        }


  void PaintHandler :: OnMouseMotion(int p){
    wxCursor *cursor=NULL;
    char msg[1024];
    static float zoom = -1.0;

    if(zoom!=APP->GetZoomLevel()){
      zoom = APP->GetZoomLevel();
      cursor = mod->GetBrushCursor(ROUND(zoom));
      APP->Set2DViewCursor(cursor, axis);
    }

    switch(mode)
    {
    case ADDSPIRAL:
    sprintf(msg,"APP->Control.Velocity = %.2f",APP->Control.Velocity);
    break;
    default:
    sprintf(msg,"Radius = %d",APP->Data.Raio);
    break;
    }
    wxString wxmsg(msg, wxConvUTF8);
    APP->Window->SetStatusText(wxmsg);
  }

} //end Manual namespace
