
#include "moduleregistration.h"


ModuleRegistration :: ModuleRegistration(){
  int i;

  this->prev = NULL;
  this->ndependencies = 0;
  for(i=0; i<Module::LAST_TYPE; i++){
    this->M[i] = CreateArrayList(20);
    SetArrayListCleanFunc(this->M[i], freeModule);
  }
}


ModuleRegistration :: ~ModuleRegistration(){
  int i;

  for(i=0; i<Module::LAST_TYPE; i++)
    DestroyArrayList(&this->M[i]);
}


void ModuleRegistration :: Register(Module *m){
  AddArrayListElement(this->M[m->GetType()],
		      (void *)m);
}

int ModuleRegistration :: GetNumberModules(Module::ModuleType type){
  return (M[type]->n);
}


Module* ModuleRegistration :: GetModule(Module::ModuleType type,
					int index){
  return (Module *)GetArrayListElement(M[type], index);
}


void ModuleRegistration :: ShowModuleWindow(Module::ModuleType type){
  char title[512],msg[512];
  int cod,id,j;

  if( !StopLastActiveProcessingModule() ) return;

  switch(type){
  case Module::PREPROC:
    strcpy(title, "Transformations"); //Preprocessing");
    break;
  case Module::DRAW:
    strcpy(title, "Draw");
    break;
  case Module::ANALYSIS:
    strcpy(title, "Analysis");
    break;
  case Module::CLASSIFY:
    strcpy(title, "Classify");
    break;
  case Module::VIEW2D:
    strcpy(title, "2D Visualization");
    break;
  case Module::VIEW3D:
    strcpy(title, "3D Visualization");
    break;
  case Module::LAST_TYPE:
    return;
  }

  if(M[type]->n == 0){
    sprintf(msg,"No %s modules available.",title);
    wxString wxstr(msg, wxConvUTF8);
    wxMessageBox(wxstr, _T("Warning"),
		 wxOK | wxICON_EXCLAMATION, APP->Window);
    return;
  }


  id = APP->idManager->AllocID();
  BaseDialog dialog(APP->Window, title);
  ModulePanel panel((wxWindow *)&dialog, id);

  for(j=0; j<M[type]->n; j++)
    panel.AddModule((Module *)GetArrayListElement(M[type], j));

  dialog.AddPanel(&panel);
  cod = dialog.ShowModal();

  if(cod==wxID_OK){
    Module *m = panel.GetSelection();

    if(type==Module::VIEW2D){
      View2DModule *m2d = (View2DModule *)m;
      APP->Set2DViewModule(m2d);
    }
    else if(type==Module::VIEW3D){
      View3DModule *m3d = (View3DModule *)m;
      APP->Set3DViewModule(m3d);
    }
    else{
      this->prev = m;
    }
    m->Start();
 }
  APP->idManager->FreeID(id);
}


bool ModuleRegistration :: StopLastActiveProcessingModule(){
  bool ret;
  if(this->prev!=NULL){
    ret = (this->prev)->Stop();
    if(ret) this->prev = NULL;
    return ret;
  }
  return true;
}


//**** Dependency functions: ***********




