
#define BIADEBUG  1

#include "smdpr.h"

#ifdef _WIN32
#include <windows.h>
#endif

#include <wx/wx.h>

//Modules
#include "register_modules.h"

//#include "xpm/segmentation.xpm"
//#include "xpm/visualization.xpm"
//#include "xpm/reg.xpm"
#include "xpm/curve.xpm"
//#include "xpm/preproc.xpm"
//#include "xpm/eye.xpm"
//#include "xpm/noteye.xpm"
#include "xpm/trash.xpm"
#include "xpm/zoomin.xpm"
#include "xpm/zoomout.xpm"
#include "xpm/brightcontr.xpm"
#include "xpm/brain.xpm"
//#include "xpm/new.xpm"
//#include "xpm/open.xpm"
//#include "xpm/save.xpm"
//#include "xpm/measure.xpm"
//#include "xpm/group.xpm"
#include "xpm/tools.xpm"
#include "xpm/classify.xpm"
#include "xpm/navigator.xpm"
#include "xpm/paint.xpm"
#include "xpm/add.xpm"

#include <string>
using namespace std;

void app_selective_refresh(Voxel v);

#define NPREOBJ 9


static const char *PredefinedObjs[] = {"Brain",
				       "Telencephalon",
				       "Left Hemisphere",
				       "Right Hemisphere",
				       "Brain Envelope",
				       "Cerebellum",
				       "CSF",
				       "White Matter",
                    "Gray Matter"};

Subgraph* DLtoSubgraph(PSet::DataList DL)
{
    unsigned int i, j, k=0, l;
    Subgraph *g = NULL;
    if(DL.size() == 0) return g;
    g = CreateSubgraph((int) DL.NumberOfPoints());
    g->nlabels = (int) DL.NumberOfClasses();
    g->nfeats = (int) DL[0].dim();
    for(i=0;i< DL.size();i++) for(j=0; j< (unsigned int) DL[i].num();j++) {
        g->node[k].feat = AllocFloatArray(g->nfeats);
        g->node[k].position = k;
        g->node[k].truelabel = DL[i].cl+1;
        //g->node[k].label = DL[i][0].classifcl;
    for (l = 0; l < (unsigned int) g->nfeats; l++)
      g->node[k].feat[l] = DL[i][j][l];
    k++;
    }
    return g;
}

PSet::DataList SubgraphtoDL(Subgraph *g)
{
    PSet::DataList DL;
    PSet::Point P;
    int i, j;

    DL.setdim(g->nfeats);
    P.setdim(g->nfeats);
    for(i=0; i<g->nnodes; i++) {
        for(j=0; j<g->nfeats; j++) P[j] = g->node[i].feat[j];
        DL.AddPoint(P, g->node[i].label); }
    return DL;
}

void ReadFromSubgraph(Subgraph *g, PSet::DataList *DL)
{
    int i, n;
    n = DL->NumberOfPoints();
    if(n != g->nnodes)
        std::cout << "\nError in ReadFromSubgraph";
    for(i=0;i<n;i++)
        DL->pt(i).classifcl = g->node[i].label;
}

void AddDensityDistributionAroundPoint(Scene *scn[], int px, int py, int pz, int Radius, int color)
{
    vector<int> V;
    int i, j, k, R, G, B, a;
    R = t0(color);
    G = t1(color);
    B = t2(color);
    //std::cout << R << "\t" << G << "\t" << B << std::endl;

    for(i=-Radius; i<=Radius; i++)
        for(j = -Radius; j<=Radius; j++)
            for(k = -Radius; k<=Radius; k++)
                if(ValidVoxel(scn[0],px+i,py+j,pz+k) && (Radius + 1 - floor(sqrt(i*i+j*j+k*k)) > 0)) {
                    a = Radius + 1 - floor(sqrt(i*i+j*j+k*k));
                    scn[0]->data[VoxelAddress(scn[0], px+i, py+j,pz+k)] += floor(a*(R/255.0)+0.5);
                    scn[1]->data[VoxelAddress(scn[0], px+i, py+j,pz+k)] += floor(a*(G/255.0)+0.5);
                    scn[2]->data[VoxelAddress(scn[0], px+i, py+j,pz+k)] += floor(a*(B/255.0)+0.5);
                    scn[3]->data[VoxelAddress(scn[0], px+i, py+j,pz+k)] += a;


                    }
}

int GetPointSetIndex(PSet::DataList **p, int cl, int sel) {
     int j=0, k=-1;

    while(j< (int) APP->Data.DL.size()) {
        if(APP->Data.DL[j].cl == cl) {
            k++;
            if(k == sel) {
                *p = &APP->Data.DL;
                return j;
            }
        }
        j++;
    }

    j=0;

    while(j < (int) APP->Data.labelDL.size()) {
        if(APP->Data.labelDL[j].cl == cl) {
            k++;
            if(k == sel) {
                *p = &APP->Data.labelDL;
                return j;
            }

        }
        j++;
    }


    return 0;
}

float SimilarityFunction(int simfun, PSet::Point P1, PSet::Point P2, float param){
    float S = 0, f;
    int i;
    PSet::Point P;
    switch(simfun){
    case COSIN:
        f = (P1)*(P2);
        return(f);
        break;
    case INV_DIST:
        return (param/((P1-P2).abs()));
        break;

    case INV_SQRDIST:
            S = (P1-P2)*(P1-P2);
            return(param/S);
            break;
    case MINUS_DIST:
            return(-param*(P1-P2).abs());
            break;
    case MINUS_SQRDIST:
            P = (P1-P2)*(P1-P2);
            for(i=0;i<P1.dim();i++) S+=P[i];
            return(-param*S);
            break;
    case EXP_DIST:
            return(exp(-pow((P1-P2).abs(),2)/(2*param)));
            break;

    default:
            return 1;
    }

}

Eigen :: MatrixXf WriteSimilarityMatrix(int simfun, float param, PSet::DataList DL){

    Eigen::MatrixXf M (DL.NumberOfPoints(), DL.NumberOfPoints());
    int i1, j1, i2, j2, m = 0, n = 0;

    for(i1=0; i1<DL.size();i1++)
        for(j1=0;j1<DL[i1].num();j1++, m++)
            for(i2=0,n=0; i2<DL.size();i2++)
                for(j2=0;j2<DL[i2].num();j2++, n++)
                    M(m,n) = SimilarityFunction(simfun, DL[i1][j1], DL[i2][j2], param);
    return (M);
}

void NormalizedCutSpectralClustering(int simfun, float param, float CutValue, PSet::DataList *DL){

    Eigen::SelfAdjointEigenSolver<Eigen::MatrixXf> SOLVER;
    Eigen::MatrixXf W = WriteSimilarityMatrix(simfun, param, *DL);
    Eigen::MatrixXf D(W.rows(), W.cols()), D2(W.rows(), W.cols());
    int i, j, k, m;
    float f, fmin=1, fmax=0, S=0;

    for(i=0;i<W.rows();i++)
        for(j=0;j<W.cols();j++)
            if(i == j)
            {
                S = 0;
                for(k=0;k<W.cols();k++)
                    if(i!=k)
                        S+=W(i,k);
                D(i,j) = S;
            }
            else D(i,j) = 0;

    D2 = D;
    for(i=0;i<W.rows();i++)
        D2(i,i) = 1/sqrt(D(i,i));

    W = D2*(D-W)*D2;

    SOLVER.compute(W);

        for(i=0;i<SOLVER.eigenvectors().rows();i++) {
         f = SOLVER.eigenvectors()(i,1);
         if(f < fmin) fmin = f;
         if(f > fmax) fmax = f;
        }
    f = fmin+(fmax-fmin)*CutValue;

    for(i=0,m=0;i<DL->size();i++)
        for(j=0;j<(*DL)[i].num();j++,m++) {
            if(SOLVER.eigenvectors()(m,1) < f)
                (*DL)[i][j].classifcl = 0;
            else (*DL)[i][j].classifcl = 1;

        }
}

void SpectralClustering(int simfun, float param, float CutValue, PSet::DataList *DL){

    Eigen::SelfAdjointEigenSolver<Eigen::MatrixXf> SOLVER;
    SOLVER.compute(WriteSimilarityMatrix(simfun, param, *DL));

    int i, j, m;
    float f, fmin=1, fmax=0;

        for(i=0;i<SOLVER.eigenvectors().rows();i++) {
         f = SOLVER.eigenvectors()(i,SOLVER.eigenvectors().cols() - 1);
         if(f < fmin && f>0) fmin = f;
         if(f > fmax && f>0) fmax = f;
        }
    f = fmin+(fmax-fmin)*CutValue;

    for(i=0,m=0;i<DL->size();i++)
        for(j=0;j<(*DL)[i].num();j++,m++) {
            if(SOLVER.eigenvectors()(m,SOLVER.eigenvectors().cols() - 1) < f)
                (*DL)[i][j].classifcl = 0;
            else (*DL)[i][j].classifcl = 1;

        }
}


void SpectralClusteringNClusters(int simfun, float param, int NClusters,  float stop_value, PSet::DataList* DL)
{

    int i, j;
    float sum;
    Eigen::SelfAdjointEigenSolver<Eigen::MatrixXf> SOLVER;
    Eigen::MatrixXf M = WriteSimilarityMatrix(simfun, param, *DL);

    for(i=0;i<M.rows();i++) {
        sum = 0;
        for(j=0;j<M.cols();j++) {
            sum += M(i,j);
            }
        M(i,i) = M(i,i) - sum;
    }

    M = -M;

    SOLVER.compute(M);

    PSet::DataList AuxDL;
    PSet::Point AuxP;
    AuxP.setdim(NClusters);
    AuxDL.setdim(NClusters);
    AuxP.cl = 0;

    for(i=0;i<SOLVER.eigenvectors().rows();i++) {
        for(j=0;j<NClusters;j++)
            AuxP[j] = SOLVER.eigenvectors()(i, j);
        AuxDL.AddPoint(AuxP);
    }

   KMeansClassify(&AuxDL, NClusters, stop_value);

   for(i=0;i<DL->NumberOfPoints();i++)
        DL->pt(i).classifcl = AuxDL.pt(i).classifcl;
}

void CallCluto(PSet::DataList *DL, int nclusters, int clMethod, int sim, int crfun){
    char command[256], aux[32];
    DL->ExportFile("cluto.txt", PSet::CLUTO);
    sprintf(command, "./cluto-2.1.1/Linux/vcluster ");
    switch(clMethod) {
        case CLUTO_RB: strcpy(aux,"-clmethod=rb "); break;
        case CLUTO_RBR: strcpy(aux,"-clmethod=rbr "); break;
        case CLUTO_DIRECT: strcpy(aux, "-clmethod=direct "); break;
        case CLUTO_AGGRO: strcpy(aux, "-clmethod=agglo "); break;
        case CLUTO_GRAPH: strcpy(aux, "-clmethod=graph "); break;
        case CLUTO_BAGGLO: strcpy(aux,"-clmethod=bagglo "); break;
        default: strcpy(aux, ""); break;
    }

   strcat(command, aux);


    switch(sim) {
        case CLUTO_COS: strcpy(aux,"-sim=cos "); break;
        case CLUTO_CORR: strcpy(aux,"-sim=corr "); break;
        case CLUTO_DIST: strcpy(aux, "-sim=dist "); break;
        case CLUTO_JACE: strcpy(aux, "-sim=jacc "); break;
        default: strcpy(aux, ""); break;
    }

    strcat(command, aux);

    switch(crfun){
        case CLUTO_I1: strcpy(aux,"-crfun=i1 "); break;
        case CLUTO_I2: strcpy(aux,"-crfun=i2 "); break;
        case CLUTO_E1: strcpy(aux, "-crfun=e1 "); break;
        case CLUTO_G1: strcpy(aux, "-crfun=g1 "); break;
        case CLUTO_G1P: strcpy(aux, "-crfun=g1p "); break;
        case CLUTO_H1: strcpy(aux,"-crfun=h1 "); break;
        case CLUTO_H2: strcpy(aux,"-crfun=h2 "); break;
        case CLUTO_SLINK: strcpy(aux,"-crfun=slink "); break;
        case CLUTO_WSLINK: strcpy(aux, "-crfun=wslink "); break;
        case CLUTO_CLINK: strcpy(aux, "-crfun=clink "); break;
        case CLUTO_WCLINK: strcpy(aux, "-crfun=wclink "); break;
        case CLUTO_UPGMA: strcpy(aux,"-crfun=upgma "); break;
        default: strcpy(aux, ""); break;
    }

    strcat(command, aux);

   strcat(command, "cluto.txt ");
   sprintf(aux, "%d", nclusters);
   strcat(command, aux);

   std::cout << "\n\n" << command;
    //sprintf(command, "%scluto.txt %d", method, nclusters);
    system(command);
}

void ReadCluto(PSet::DataList *output, int nclusters){

    char str[256];
    FILE *file;
    sprintf(str, "cluto.txt.clustering.%d", nclusters);
    file = fopen(str,"r");
    int i, j, aux;
    for(i=0;i< (int) output->size();i++)
        for(j=0;j<(*output)[i].num();j++) {
            fscanf(file, "%d",&aux);
            (*output)[i][j].classifcl = aux;
        }
}


void KMeansClassify(PSet::DataList *DL, int k, float stop_value)
{
    int i, j, l, n, jmin;
    float error, maxerror  = stop_value + 1, dist, mindist;
    PSet::Point* V = new PSet::Point[k];
    PSet::Point* nextV = new PSet::Point[k];
    int *NNextV = new int[k];

    n = DL->NumberOfPoints();

    for(i=0;i<k;i++) {
        V[i] = DL->pt(i*(n/k));
        nextV[i].setdim(DL->dim());
    }


    while(maxerror > stop_value)
    //for(l=0;l<3;l++)
    {
        maxerror = 0;

        for(i=0;i<k;i++) {
            nextV[i].clear();
            NNextV[i] = 0;}

        for(i=0;i<n;i++)
        {
            jmin = 0;
            DL->pt(i).classifcl = 0;
            mindist = (DL->pt(i) - V[0]).abs();
            for(j=1;j<k;j++)
                {
                    dist = (DL->pt(i) - V[j]).abs();
                    if(dist < mindist) {
                        mindist = dist;
                        jmin = j;
                    }
                }

            DL->pt(i).classifcl = jmin;
            nextV[jmin] += DL->pt(i);
            NNextV[jmin]++;

        }

        for(i=0;i<k;i++) {

            if(NNextV[i] != 0)
            nextV[i]*=((float) 1/NNextV[i]);

        }
        maxerror = (V[0] - nextV[0]).abs();
        V[0] = nextV[0];
        for(i=1;i<k;i++) {
        error = (V[i] - nextV[i]).abs();
        if(error > maxerror)
            maxerror = error;
        V[i] = nextV[i];
        }

    }

}

void GFT_Clustering(PSet::DataList *DL, int NClusters, int outdegree, float pparam, float qparam, int *GFTfunc, int GFTGraph,
                    float sigma, bool CorrectWeights, bool OPFsigma)
{
    char command[512];
    sprintf(command, "./GFTclustering/gft GFTinput.txt GFToutput.txt %d %d %f %f %d %d %d %d %f %d %d", NClusters,
                                            outdegree, pparam, qparam, GFTfunc[0], GFTfunc[1], GFTfunc[2], GFTGraph,
                                            sigma, CorrectWeights, OPFsigma);
    DL->ExportFile("GFTinput.txt", PSet::SIMPLESTD);
    system(command);
    DL->ImportFile("GFToutput.txt", PSet::SIMPLESTD);
}


ModuleRegistration *modManager;
BIAClass *APP;

AppDataControl :: AppDataControl()
{
    Velocity = 3;
    NLoops = 5;
    NPoints = 100;
    LinRadius = 30;
    stddev = 10;
    simple = true;
    CurrentSpline = NULL;
    ClassColor.push_back(0xFFFF00);
    path[0] = 0;

}


//---------------------------------------------------------

enum{
    ID_Quit = wxID_HIGHEST+1,
    ID_About,
    ID_VolInfo,
#ifdef BIADEBUG
    ID_ParamEstimation,
    ID_LoadGrad,
    ID_LoadGradTool,
#endif
    ID_Preproc,
    ID_Draw,
    ID_Analysis,
    ID_Measurements,
    ID_3DVisu,
    ID_Classify,
    ID_LoadVol,
    ID_LoadLabel,
    ID_SaveLabel,
    ID_SaveVolume,
    ID_SaveObject,
    ID_SaveObjectMask,
    ID_CLUTO,
    ID_MATLAB,
    ID_SIMPLESTD,
    ID_STD,
    ID_OPF,
    ID_OPFDAT,
    ID_LoadMark,
    ID_SaveMark,
    ID_LoadVolTool,
    ID_ObjEye,
    ID_Zoomin,
    ID_Zoomout,
    ID_ChangeDrawMarker,
    ID_BriContr,
    ID_BriContrDi,
    ID_ViewOpt,
    ID_View0,
    ID_View1,
    ID_View2,
    ID_View3,
    ID_View4,
    ID_Last

};

//----------------------------------------------------------


BIAClass :: BIAClass(){
  View2DModule *view2D;
  View3DModule *view3D;
  Voxel Cut;
  int n;

  APP = this;
  this->disableAll = NULL;
  this->busycursor = NULL;
  this->busyinfo   = NULL;
  this->LoadPreferences();

  Window = new MyFrame(_T("Synthetic Database Generator for Pattern Recognition"),
		       wxPoint(50,50),wxSize(800,600));

  //APP->measures = CreateMeasures();
  this->InitData();
  idManager  = new DynamicID(ID_Last);
  modManager = new ModuleRegistration();
  app_register_modules(modManager);

  n = modManager->GetNumberModules(Module::VIEW2D);
  if(n==0) Error((char *)"BIAClass",(char *)"View2DModule not loaded");
  view2D = (View2DModule*)modManager->GetModule(Module::VIEW2D, 0);

  n = modManager->GetNumberModules(Module::VIEW3D);
  if(n==0) Error((char *)"BIAClass",(char *)"View3DModule not loaded");
  view3D = (View3DModule*)modManager->GetModule(Module::VIEW3D, 0);

  this->Set2DViewModule(view2D);
  this->Set3DViewModule(view3D);

  Window->Show(TRUE);
  if( (this->Preferences).IsMaximized ){
    Window->Maximize(true);
    Window->UniformView();
  }

  APP->Data.DL.setdim(3);           //Atualização.
  APP->Data.labelDL.setdim(3);      //Atualização.
  APP->Data.Raio = 10;
  APP->Data.dim[0] = 0;
  APP->Data.dim[1] = 1;
  APP->Data.dim[2] = 2;
  APP->Data.StaticPoint.setdim(3);

  APP->Data.DL.role = PSet::DATA;

  Cut.x = APP->Data.w/2;
  Cut.y = APP->Data.h/2;
  Cut.z = APP->Data.nframes/2;
  APP->Set2DViewSlice(Cut);


  APP->Refresh2DCanvas();
  APP->Refresh3DCanvas(true, 1.0);
}


BIAClass :: ~BIAClass(){
  if(labelcolor!=NULL) free(labelcolor);
  if(labelalpha!=NULL) free(labelalpha);
  this->SavePreferences();
}


void BIAClass :: Busy(char *msg){
  wxString wxmsg(msg, wxConvUTF8);

  if(disableAll==NULL)
    disableAll = new wxWindowDisabler(NULL);
  if(busycursor==NULL)
    busycursor = new wxBusyCursor(wxHOURGLASS_CURSOR);
  if(busyinfo!=NULL)
    delete busyinfo;
  if(msg[0]!='\0')
    busyinfo   = new wxBusyInfo(wxmsg, this->Window);
  wxTheApp->Yield();
}

void BIAClass :: Unbusy(){
  if(disableAll!=NULL)
    delete disableAll;
  if(busyinfo!=NULL)
    delete busyinfo;
  if(busycursor!=NULL)
    delete busycursor;
  disableAll = NULL;
  busyinfo   = NULL;
  busycursor = NULL;
  //wxTheApp->Yield();
}

void BIAClass :: StatusMessage(char *msg){
  wxString wxmsg(msg, wxConvUTF8);
  this->Window->SetStatusText(wxmsg);
}


int BIAClass :: GetColorDialog(){
  wxColourDialog *colourDialog;
  wxColourData cdata;
  wxColour c;
  int r,g,b;

  colourDialog = new wxColourDialog(this->Window, NULL);
  if(colourDialog->ShowModal() == wxID_OK){
    cdata = colourDialog->GetColourData();
    c = cdata.GetColour();
    r = c.Red();
    g = c.Green();
    b = c.Blue();
    return triplet(r,g,b);
  }
  return NIL;
}


void BIAClass :: ResetData(){
  if(APP->Data.loaded){
    SetScene(APP->Data.label, 0);
    //this->DelAllSeeds();
    //this->UnmarkAllForRemoval();
  }
}

void BIAClass :: DestroyData(){
  if(APP->Data.loaded){
    DestroyScene(&(APP->Data.den[0]));
    DestroyScene(&(APP->Data.den[1]));
    DestroyScene(&(APP->Data.den[2]));
    DestroyScene(&(APP->Data.den[3]));
    DestroyScene(&(APP->Data.label));

    //Libera tudo que esta em PSL:

    //this->DelAllSeeds();
    //this->UnmarkAllForRemoval();
    APP->Data.loaded = 0;
  }
}

void BIAClass :: SetDataVolume(Scene *scn){
  Voxel Cut;
  this->DestroyData();

  APP->Data.w = scn->xsize;
  APP->Data.h = scn->ysize;
  APP->Data.nframes = scn->zsize;

  if (APP->Data.den[0]!=NULL) DestroyScene(&(APP->Data.den[0]));
  if (APP->Data.den[1]!=NULL) DestroyScene(&(APP->Data.den[1]));
  if (APP->Data.den[2]!=NULL) DestroyScene(&(APP->Data.den[2]));
  if (APP->Data.den[3]!=NULL) DestroyScene(&(APP->Data.den[3]));
  if (APP->Data.label!=NULL) DestroyScene(&(APP->Data.label));

  APP->Data.den[0]  = scn;
  APP->Data.den[1]  = CreateScene(APP->Data.w, APP->Data.h, APP->Data.nframes);
  APP->Data.den[2]  = CreateScene(APP->Data.w, APP->Data.h, APP->Data.nframes);
  APP->Data.den[3]  = CreateScene(APP->Data.w, APP->Data.h, APP->Data.nframes);
  APP->Data.label = CreateScene(APP->Data.w, APP->Data.h, APP->Data.nframes);

  SetVoxelSize(APP->Data.label, scn->dx, scn->dy, scn->dz);
  //APP->ReallocSeedStructures();

  Cut.x = APP->Data.w/2;
  Cut.y = APP->Data.h/2;
  Cut.z = APP->Data.nframes/2;
  APP->Set2DViewSlice(Cut);

  APP->Data.loaded = 1;
}


void BIAClass :: SetDataVolume_NoDestroy(Scene *scn){
  Voxel Cut;
  APP->Data.w = scn->xsize;
  APP->Data.h = scn->ysize;
  APP->Data.nframes = scn->zsize;

  if (APP->Data.den[0]!=NULL) DestroyScene(&(APP->Data.den[0]));
  if (APP->Data.den[1]!=NULL) DestroyScene(&(APP->Data.den[1]));
  if (APP->Data.den[2]!=NULL) DestroyScene(&(APP->Data.den[2]));
  if (APP->Data.den[3]!=NULL) DestroyScene(&(APP->Data.den[3]));
  if (APP->Data.label!=NULL) DestroyScene(&(APP->Data.label));

  APP->Data.den[0] = scn;
  APP->Data.den[1] = CreateScene(APP->Data.w, APP->Data.h, APP->Data.nframes);
  APP->Data.den[2] = CreateScene(APP->Data.w, APP->Data.h, APP->Data.nframes);
  APP->Data.den[3] = CreateScene(APP->Data.w, APP->Data.h, APP->Data.nframes);
  APP->Data.label = CreateScene(APP->Data.w, APP->Data.h, APP->Data.nframes);
  SetVoxelSize(APP->Data.label, scn->dx, scn->dy, scn->dz);
  //APP->ReallocSeedStructures();

  Cut.x = APP->Data.w/2;
  Cut.y = APP->Data.h/2;
  Cut.z = APP->Data.nframes/2;
  APP->Set2DViewSlice(Cut);

  APP->Data.loaded = 1;
}


void BIAClass :: GetViewOptions(ViewOptType *vopt){
  Window->GetViewOptions(vopt);
}


void BIAClass :: Set2DViewModule(View2DModule *mod){
  wxPanel *panel;

  this->mod2D = mod;

  panel = mod->GetViewPanel(Window, 'z');
  Window->SetViewPanel(panel, 0);

  panel = mod->GetViewPanel(Window, 'y');
  Window->SetViewPanel(panel, 1);

  panel = mod->GetViewPanel(Window, 'x');
  Window->SetViewPanel(panel, 2);

  mod->Start();
}


void BIAClass :: Set3DViewModule(View3DModule *mod){
  wxPanel *panel;

  this->mod3D = mod;
  panel = mod->GetViewPanel(Window);
  Window->SetViewPanel(panel, 3);
  mod->Start();
}


void BIAClass :: MakePalette() {
  labelcolor[0] = NIL;
  labelcolor[1] = 0xffff00; //0xccbb99;
  labelcolor[2] = 0x00ff00;
  labelcolor[3] = 0x00ffff;
  labelcolor[4] = 0xff80ff;
  labelcolor[5] = 0xff8000;
  labelcolor[6] = 0x008000;
  labelcolor[7] = 0xee00ff;
  labelcolor[8] = 0xff0000;
  labelcolor[9] = 0x800000;
}


void BIAClass :: InitData() {
  this->Lmax = 1024;
  this->labelcolor = AllocIntArray(this->Lmax);
  this->labelalpha = AllocIntArray(this->Lmax);

  this->MakePalette();
  this->Data.labelname[ 0 ] = 0;
  this->Data.projectfilename[ 0 ] = 0;

  this->Data.w = 250;
  this->Data.h = 250;
  this->Data.nframes = 250;
  this->Data.den[0]   = CreateScene(this->Data.w,
				 this->Data.h,
				 this->Data.nframes);
  this->Data.den[1]   = CreateScene(this->Data.w,
				 this->Data.h,
				 this->Data.nframes);
  this->Data.den[2]   = CreateScene(this->Data.w,
				 this->Data.h,
				 this->Data.nframes);
  this->Data.den[3]   = CreateScene(this->Data.w,
				 this->Data.h,
				 this->Data.nframes);

  this->Data.label = CreateScene(this->Data.w,
				 this->Data.h,
				 this->Data.nframes);

  this->Data.loaded = 1;
}


void BIAClass :: InitPreferences(){
  (this->Preferences).IsMaximized        = true; //false;
  (this->Preferences).color_preproc      = NIL; //0xeeeeff;
  (this->Preferences).color_segmentation = NIL; //0xeeffee;
  (this->Preferences).color_view         = NIL; //0xffffdd;
  (this->Preferences).color_analysis     = NIL; //0xffeeee;
  (this->Preferences).dir_LoadVolume[0]  = '.';
  (this->Preferences).dir_LoadLabel[0]   = '.';
  (this->Preferences).dir_SaveVolume[0]  = '.';
  (this->Preferences).dir_SaveLabel[0]   = '.';
  (this->Preferences).dir_SaveObject[0]  = '.';
  (this->Preferences).dir_LoadMarkers[0] = '.';
  (this->Preferences).dir_SaveMarkers[0] = '.';
  (this->Preferences).ext_LoadVolume     = 0;
  (this->Preferences).ext_LoadLabel      = 0;
  (this->Preferences).ext_SaveVolume     = 0;
  (this->Preferences).ext_SaveLabel      = 0;
  (this->Preferences).ext_SaveObject     = 0;
  (this->Preferences).ext_LoadMarkers    = 0;
  (this->Preferences).ext_SaveMarkers    = 0;
}


void BIAClass :: SavePreferences(){
  FILE *fp=NULL;
  char *value=NULL;
  char file[512];

  value = getenv("HOME");
  if(value==NULL){
    Warning((char *)"Unable to save preferences",(char *)"SavePreferences");
    return;
  }
  sprintf(file,"%s/.bia_conf",value);
  fp = fopen(file,"w");
  if(fp==NULL){
    Warning((char *)"Unable to save preferences",(char *)"SavePreferences");
    return;
  }
  fprintf(fp,"IsMaximized         %d\n",(this->Preferences).IsMaximized);
  fprintf(fp,"color_preproc       %d\n",(this->Preferences).color_preproc);
  fprintf(fp,"color_segmentation  %d\n",(this->Preferences).color_segmentation);
  fprintf(fp,"color_view          %d\n",(this->Preferences).color_view);
  fprintf(fp,"color_analysis      %d\n",(this->Preferences).color_analysis);
  fprintf(fp,"dir_LoadVolume      %s\n",(this->Preferences).dir_LoadVolume);
  fprintf(fp,"dir_LoadLabel       %s\n",(this->Preferences).dir_LoadLabel);
  fprintf(fp,"dir_SaveVolume      %s\n",(this->Preferences).dir_SaveVolume);
  fprintf(fp,"dir_SaveLabel       %s\n",(this->Preferences).dir_SaveLabel);
  fprintf(fp,"dir_SaveObject      %s\n",(this->Preferences).dir_SaveObject);
  fprintf(fp,"dir_LoadMarkers     %s\n",(this->Preferences).dir_LoadMarkers);
  fprintf(fp,"dir_SaveMarkers     %s\n",(this->Preferences).dir_SaveMarkers);
  fprintf(fp,"ext_LoadVolume      %d\n",(this->Preferences).ext_LoadVolume);
  fprintf(fp,"ext_LoadLabel       %d\n",(this->Preferences).ext_LoadLabel);
  fprintf(fp,"ext_SaveVolume      %d\n",(this->Preferences).ext_SaveVolume);
  fprintf(fp,"ext_SaveLabel       %d\n",(this->Preferences).ext_SaveLabel);
  fprintf(fp,"ext_SaveObject      %d\n",(this->Preferences).ext_SaveObject);
  fprintf(fp,"ext_LoadMarkers     %d\n",(this->Preferences).ext_LoadMarkers);
  fprintf(fp,"ext_SaveMarkers     %d\n",(this->Preferences).ext_SaveMarkers);

  fclose(fp);
}


void BIAClass :: LoadPreferences(){
  FILE *fp=NULL;
  char option[512],value[512];
  char *envvalue=NULL;
  char file[512];

  this->InitPreferences();

  envvalue = getenv("HOME");
  if(envvalue==NULL)
    return;

  sprintf(file,"%s/.bia_conf",envvalue);
  fp = fopen(file,"r");
  if(fp==NULL)
    return;

  while(fscanf(fp,"%s  %s\n",option,value)!=EOF){
    if(strcmp(option, "IsMaximized")==0)
      (this->Preferences).IsMaximized = atoi(value);

    else if(strcmp(option, "color_preproc")==0)
      (this->Preferences).color_preproc = atoi(value);
    else if(strcmp(option, "color_segmentation")==0)
      (this->Preferences).color_segmentation = atoi(value);
    else if(strcmp(option, "color_view")==0)
      (this->Preferences).color_view = atoi(value);
    else if(strcmp(option, "color_analysis")==0)
      (this->Preferences).color_analysis = atoi(value);
    else if(strcmp(option, "dir_LoadVolume")==0)
      strcpy((this->Preferences).dir_LoadVolume, value);
    else if(strcmp(option, "dir_LoadLabel")==0)
      strcpy((this->Preferences).dir_LoadLabel, value);
    else if(strcmp(option, "dir_SaveVolume")==0)
      strcpy((this->Preferences).dir_SaveVolume, value);
    else if(strcmp(option, "dir_SaveLabel")==0)
      strcpy((this->Preferences).dir_SaveLabel, value);
    else if(strcmp(option, "dir_SaveObject")==0)
      strcpy((this->Preferences).dir_SaveObject, value);
    else if(strcmp(option, "dir_LoadMarkers")==0)
      strcpy((this->Preferences).dir_LoadMarkers, value);
    else if(strcmp(option, "dir_SaveMarkers")==0)
      strcpy((this->Preferences).dir_SaveMarkers, value);

    else if(strcmp(option, "ext_LoadVolume")==0)
      (this->Preferences).ext_LoadVolume = atoi(value);
    else if(strcmp(option, "ext_LoadLabel")==0)
      (this->Preferences).ext_LoadLabel = atoi(value);
    else if(strcmp(option, "ext_SaveVolume")==0)
      (this->Preferences).ext_SaveVolume = atoi(value);
    else if(strcmp(option, "ext_SaveLabel")==0)
      (this->Preferences).ext_SaveLabel = atoi(value);
    else if(strcmp(option, "ext_SaveObject")==0)
      (this->Preferences).ext_SaveObject = atoi(value);
    else if(strcmp(option, "ext_LoadMarkers")==0)
      (this->Preferences).ext_LoadMarkers = atoi(value);
    else if(strcmp(option, "ext_SaveMarkers")==0)
      (this->Preferences).ext_SaveMarkers = atoi(value);
  }
  fclose(fp);
}


void BIAClass :: AppendOptPanel(wxPanel *panel,
				Module::ModuleType type){
  Window->AppendOptPanel(panel, type);
}

void BIAClass :: PrependOptPanel(wxPanel *panel,
				 Module::ModuleType type){
  Window->PrependOptPanel(panel, type);
}

bool BIAClass :: DetachOptPanel(wxPanel *panel){
  return Window->DetachOptPanel(panel);
}


void BIAClass :: EnableObjWindow(bool enable){
  Window->EnableObjWindow(enable);
}


void BIAClass :: Set2DViewCursor(const wxCursor *cursor, char axis){
  View2DModule *view2D;
  int n;

  n = modManager->GetNumberModules(Module::VIEW2D);
  if(n==0) Error((char *)"BIAClass",(char *)"View2DModule not loaded");
  view2D = (View2DModule*)modManager->GetModule(Module::VIEW2D, 0);
  view2D->SetCursor(cursor, axis);
}


void BIAClass :: Set2DViewInteractionHandler(InteractionHandler *handler,
					     char axis){
  View2DModule *view2D;
  int n;

  n = modManager->GetNumberModules(Module::VIEW2D);
  if(n==0) Error((char *)"BIAClass",(char *)"View2DModule not loaded");
  view2D = (View2DModule*)modManager->GetModule(Module::VIEW2D, 0);
  view2D->SetInteractionHandler(handler, axis);
}


void BIAClass :: SetDefaultInteractionHandler(){
  APP->Set2DViewInteractionHandler(NULL, 'x');
  APP->Set2DViewInteractionHandler(NULL, 'y');
  APP->Set2DViewInteractionHandler(NULL, 'z');
}


void BIAClass :: Set2DViewSlice(Voxel Cut){
  View2DModule *view2D;
  int n;

  n = modManager->GetNumberModules(Module::VIEW2D);
  if(n==0) Error((char *)"BIAClass",(char *)"View2DModule not loaded");
  view2D = (View2DModule*)modManager->GetModule(Module::VIEW2D, 0);
  view2D->SetSliceVoxel(Cut);
  APP->Refresh3DCanvas(false, 1.0);
}


Voxel BIAClass :: Get2DViewSlice(){
  View2DModule *view2D;
  int n;

  n = modManager->GetNumberModules(Module::VIEW2D);
  if(n==0) Error((char *)"BIAClass",(char *)"View2DModule not loaded");
  view2D = (View2DModule*)modManager->GetModule(Module::VIEW2D, 0);
  return view2D->GetSliceVoxel();
}


void BIAClass :: Refresh2DCanvas(){
  this->mod2D->Refresh();
}


void BIAClass :: Refresh2DCanvas(char axis){
  this->mod2D->Refresh(axis);
}


void BIAClass :: Refresh3DCanvas(bool dataChanged,
				 float quality){
  if(this->mod3D!=NULL)
    this->mod3D->Refresh(dataChanged, quality);
}


void BIAClass :: SetRefresh2DHandler(RefreshHandler *handler){
  this->mod2D->SetRefreshHandler(handler);
}


void BIAClass :: Clear2DCanvas(){
  this->mod2D->Clear();
}

void BIAClass :: ChangeDrawMarker(){
  this->mod2D->ChangeDrawMarker();
  this->mod2D->Refresh();
}


CImage * BIAClass :: Copy3DCanvasAsCImage(){
  return this->mod3D->CopyAsCImage();
}


CImage * BIAClass :: Copy2DCanvasAsCImage(char axis){
  return this->mod2D->CopyAsCImage(axis);
}

void BIAClass :: UpdateDensityScene(){

  int j,i;
  PSet::Point x;
  int px,py,pz;
  SetScene(APP->Data.den[0], 0);
  SetScene(APP->Data.den[1], 0);
  SetScene(APP->Data.den[2], 0);
  SetScene(APP->Data.den[3], 0);
  for(i=0; i < (int) APP->Data.DL.size(); i++)
    for(j=0; j < APP->Data.DL[i].num(); j++){
        if(Data.DL[i].visible == false) break;
        x = APP->Data.DL[i][j];

        px = ROUND(x[APP->Data.dim[0]]);
        py = ROUND(x[APP->Data.dim[1]]);
        pz = ROUND(x[APP->Data.dim[2]]);

        if(ValidVoxel(APP->Data.den[0], px,py,pz)) {
            AddDensityDistributionAroundPoint(APP->Data.den, px, py, pz,
                                          APP->Data.Raio,
                                          APP->GetLabelColour(APP->Data.DL[i][j].cl+1));
    }

  }


MaximumValue3(APP->Data.den[0]);
MaximumValue3(APP->Data.den[1]);
MaximumValue3(APP->Data.den[2]);
MaximumValue3(APP->Data.den[3]);



}

void BIAClass :: UpdateClassifiedDensityScene()
{
  int j,i;
  PSet::Point x;
  int px,py,pz;
  SetScene(APP->Data.den[0], 0);
  SetScene(APP->Data.den[1], 0);
  SetScene(APP->Data.den[2], 0);
  SetScene(APP->Data.den[3], 0);
  for(i=0; i < (int) APP->Data.DL.size(); i++)
    for(j=0; j < APP->Data.DL[i].num(); j++){
        if(Data.DL[i].visible == false) break;
        x = APP->Data.DL[i][j];

        px = ROUND(x[APP->Data.dim[0]]);
        py = ROUND(x[APP->Data.dim[1]]);
        pz = ROUND(x[APP->Data.dim[2]]);

        if(ValidVoxel(APP->Data.den[0], px,py,pz)) {
            AddDensityDistributionAroundPoint(APP->Data.den, px, py, pz,
                                          APP->Data.Raio,
                                          APP->GetLabelColour(APP->Data.DL[i][j].classifcl+1));
    }

  }


MaximumValue3(APP->Data.den[0]);
MaximumValue3(APP->Data.den[1]);
MaximumValue3(APP->Data.den[2]);
MaximumValue3(APP->Data.den[3]);

}

void BIAClass :: UpdateDensityScene(PSet::DataList DL){

  int j,i, p;
  PSet::Point x;
  int px,py,pz;
  SetScene(APP->Data.den[0], 0);
  SetScene(APP->Data.den[1], 0);
  SetScene(APP->Data.den[2], 0);
  SetScene(APP->Data.den[3], 0);
  SetScene(APP->Data.label, 0);

  for(i=0; i < (int) DL.size(); i++)
  for(j=0; j < DL[i].num(); j++){
    if(DL[i].visible == false) break;
    x = DL[i][j];

    px = ROUND(x[APP->Data.dim[0]]);
    py = ROUND(x[APP->Data.dim[1]]);
    pz = ROUND(x[APP->Data.dim[2]]);

    if(ValidVoxel(APP->Data.label, px,py,pz)) {
        p = VoxelAddress(APP->Data.label, px,py,pz);
        //if(DL[i].cl == -1)
            APP->Data.label->data[p] = DL[i][j].classifcl + 1;

    }

    if(ValidVoxel(APP->Data.den[0], px,py,pz)) {
        //if(DL[i].cl == -1)
        AddDensityDistributionAroundPoint(APP->Data.den, px, py, pz,
                                          APP->Data.Raio,
                                          APP->GetLabelColour(DL[i][j].classifcl+1));
    }

  }

MaximumValue3(APP->Data.den[0]);
MaximumValue3(APP->Data.den[1]);
MaximumValue3(APP->Data.den[2]);
MaximumValue3(APP->Data.den[3]);

}


void BIAClass :: UpdateLabel(){

  int p,j,i;
  PSet::Point x;
  int px=0,py=0,pz=0;
  Scene *label = APP->Data.label;
  SetScene(label, 0);


    for(i=0; i < (int) APP->Data.labelDL.size(); i++)
    for(j=0; j < APP->Data.labelDL[i].num(); j++){
    if(APP->Data.labelDL[i].visible == false) break;
    x = APP->Data.labelDL[i][j];

    px = ROUND(x[APP->Data.dim[0]]);
    py = ROUND(x[APP->Data.dim[1]]);
    pz = ROUND(x[APP->Data.dim[2]]);


    if(ValidVoxel(APP->Data.label, px,py,pz)) {
        p = VoxelAddress(APP->Data.label, px,py,pz);
        if(APP->Data.labelDL[i].cl == -1)
           APP->Data.label->data[p] = APP->Data.labelDL[i][j].cl + 1;
        else
            APP->Data.label->data[p] = APP->Data.labelDL[i].cl + 1;
        }
    }



   for(j=0; j < (int) APP->Data.labelDL.temp.size();j++)
   {

   x = APP->Data.labelDL.temp[j];
   px = ROUND(x[APP->Data.dim[0]]);
   py = ROUND(x[APP->Data.dim[1]]);
   pz = ROUND(x[APP->Data.dim[2]]);

   if(ValidVoxel(label, px,py,pz)){
   p = VoxelAddress(label, px,py,pz);
   if(APP->Data.labelDL.size() == 0) label->data[p] = 1;
   else label->data[p] = APP->Data.labelDL.back().cl + 1;
    }
   }


  for(i=0; i < (int) APP->Data.DL.size(); i++)
  for(j=0; j < APP->Data.DL[i].num(); j++){
    if(Data.DL[i].visible == false) break;
    x = APP->Data.DL[i][j];

    px = ROUND(x[APP->Data.dim[0]]);
    py = ROUND(x[APP->Data.dim[1]]);
    pz = ROUND(x[APP->Data.dim[2]]);

   if(ValidVoxel(label, px,py,pz)){
   p = VoxelAddress(label, px,py,pz);
   label->data[p] = APP->Data.DL[i].cl + 1; }
    }

}

void BIAClass :: RefreshAll() {

    APP->UpdateLabel();
    APP->UpdateDensityScene();
    APP->Refresh2DCanvas();
    APP->Refresh3DCanvas(true,1.0);
}

void BIAClass :: RefreshAll(PSet::DataList DL) {

    APP->UpdateLabel();
    APP->UpdateDensityScene(DL);
    APP->Refresh2DCanvas();
    APP->Refresh3DCanvas(true,1.0);

}



void BIAClass :: SetBriContr(int B, int C){
  APP->Window->SetBriContr(B, C);
  //this->Refresh2DCanvas();
  //this->Refresh3DCanvas(false, 1.0);
}

void BIAClass :: GetBriContr(int *B, int *C){
  APP->Window->GetBriContr(B, C);
}

void BIAClass :: SetZoomLevel(float zoom){
  this->mod2D->SetZoomLevel(zoom);
  this->mod3D->SetZoomLevel(zoom);
}

float BIAClass :: GetZoomLevel(){
  return this->mod2D->GetZoomLevel();
}



int BIAClass :: ShowObjColorDialog(int *color,
				   char *name){
  int ret,id;
  char title[512];

  sprintf(title,"Choose %s color:",name);
  id = this->idManager->AllocID();
  ObjectPickerDialog dialog(this->Window,
			    id, title,
			    PredefinedObjs, NPREOBJ,
			    false, true, false);
  ret = dialog.ShowModal();
  this->idManager->FreeID(id);
  if(ret == wxID_OK){
    *color = dialog.GetColor();
    return 0;
  }
  else{
    *color = NIL;
    return 1;
  }
}


int BIAClass :: GetLabelColour(int label){
  if(label >= Lmax)
    return NIL;
  return labelcolor[label];
}

int BIAClass :: GetLabelAlpha(int label){
  if(label >= Lmax)
    return NIL;
  return labelalpha[label];
}


void BIAClass :: SetLabelColour(int label,
				int color){
  if(label >= Lmax){
    Lmax = MAX(MAX(2*Lmax, label+1), 1024);
    labelcolor = (int *)realloc(labelcolor,
				sizeof(int)*Lmax);
    labelalpha = (int *)realloc(labelalpha,
				sizeof(int)*Lmax);
  }
  labelcolor[label] = color;
  labelalpha[label] = 76; //30%
}


void BIAClass :: SetLabelColour(int label,
				int color,
				int alpha){
  if(label >= Lmax){
    Lmax = MAX(MAX(2*Lmax, label+1), 1024);
    labelcolor = (int *)realloc(labelcolor,
				sizeof(int)*Lmax);
    labelalpha = (int *)realloc(labelalpha,
				sizeof(int)*Lmax);
  }
  labelcolor[label] = color;
  labelalpha[label] = alpha;
}


void BIAClass::DrawBrush(int p, int label,
			 AdjRel *A, char axis){
  DrawBrushCustom(p,label,A,axis,
		  APP->Data.label);
}


void BIAClass::DrawBrushTrace(int p, int q,
			      int label,
			      AdjRel *A, char axis){
  DrawBrushTraceCustom(p,q,label,A,axis,
		       APP->Data.label);
}


void BIAClass::DrawBrushCustom(int p, int label,
			       AdjRel *A, char axis,
			       Scene *scn){
  Voxel u,v;
  int q,i;

  if(scn->n != (APP->Data.den[0])->n) return;
  v.x = VoxelX(scn, p);
  v.y = VoxelY(scn, p);
  v.z = VoxelZ(scn, p);

  for(i=0;i<A->n;i++){
    if(axis == 'z'){
      u.x = v.x + A->dx[i];
      u.y = v.y + A->dy[i];
      u.z = v.z;
    }
    else if(axis == 'x'){
      u.x = v.x;
      u.y = v.y + A->dy[i];
      u.z = v.z + A->dx[i];
    }
    else if(axis == 'y'){
      u.x = v.x + A->dx[i];
      u.y = v.y;
      u.z = v.z + A->dy[i];
    }

    else {                  //if axis != 'x','y' or 'z', then axis = z;
      u.x = v.x + A->dx[i];
      u.y = v.y + A->dy[i];
      u.z = v.z;
    }

    if(ValidVoxel(scn, u.x,u.y,u.z)){
      q = VoxelAddress(scn,u.x,u.y,u.z);
      scn->data[q] = label;
    }
  }
}


void BIAClass::DrawBrushTraceCustom(int p, int q,
				    int label,
				    AdjRel *A, char axis,
				    Scene *scn){
  Voxel v0,v1,v2;
  int t;

  if(scn->n != (APP->Data.den[0])->n) return;
  v0.x = VoxelX(scn, p);
  v0.y = VoxelY(scn, p);
  v0.z = VoxelZ(scn, p);
  v1.x = VoxelX(scn, q);
  v1.y = VoxelY(scn, q);
  v1.z = VoxelZ(scn, q);
  v2.x = (v0.x+v1.x)/2;
  v2.y = (v0.y+v1.y)/2;
  v2.z = (v0.z+v1.z)/2;
  t = VoxelAddress(scn,v2.x,v2.y,v2.z);

  if ((v2.x==v1.x && v2.y==v1.y && v2.z==v1.z) ||
      (v2.x==v0.x && v2.y==v0.y && v2.z==v0.z)) {
    DrawBrushCustom(p,label,A,axis,scn);
    DrawBrushCustom(q,label,A,axis,scn);
    return;
  }
  DrawBrushTraceCustom(p,t,label,A,axis,scn);
  DrawBrushTraceCustom(t,q,label,A,axis,scn);
}


Scene *BIAClass :: CopyVisibleLabelUnion(){
  int p,n;
  Scene *bin,*label = APP->Data.label;

  if(!APP->Data.loaded) return NULL;

  bin = CreateScene(APP->Data.w,
		    APP->Data.h,
		    APP->Data.nframes);
  n = APP->Data.w*APP->Data.h*APP->Data.nframes;

  for(p=0; p<n; p++){
    if(APP->GetLabelColour(label->data[p])!=NIL)
      bin->data[p] = 1;
    else
      bin->data[p] = 0;
  }
  return bin;
}


Scene *BIAClass :: CopyDistinctVisibleLabels(){
  int p,q,n,color,lb,id=1;
  Scene *label = CopyScene(APP->Data.label);
  Scene *flabel;

  if(!APP->Data.loaded) return NULL;

  flabel = CreateScene(APP->Data.w,
		       APP->Data.h,
		       APP->Data.nframes);
  n = APP->Data.w*APP->Data.h*APP->Data.nframes;

  for(p=0; p<n; p++){
    lb = label->data[p];
    if(lb==NIL) continue;

    color = APP->GetLabelColour(lb);
    if(color!=NIL){

      for(q=0; q<n; q++){
	if(label->data[q]==NIL) continue;
	if(APP->GetLabelColour(label->data[q])==color){
	  flabel->data[q] = id;
	  label->data[q] = NIL;
	}
      }
      id++;
    }
    else
      flabel->data[p] = 0;
  }
  DestroyScene(&label);

  return flabel;
}


int BIAClass :: _GetMarkerID(int mk){
  return (mk>>8); //(int)label/256;
}

int BIAClass :: _GetMarkerLabel(int mk){
  return (mk & 0xff);  //(label % 256);
}

int BIAClass :: _GetMarkerValue(int id, int lb){
  return ((id<<8) | lb);  //(id*256 + (int)lb);
}

//----------------------------------------------------------

class MyApp:public wxApp{
  virtual int  OnExit();
  virtual bool OnInit();
};


BEGIN_EVENT_TABLE(MyFrame, wxFrame)
  EVT_MENU(ID_Quit, MyFrame::OnQuit)
  EVT_MENU(ID_About, MyFrame::OnAbout)
  EVT_MENU(ID_VolInfo, MyFrame::OnVolumeInformation)
  EVT_MENU(ID_LoadVol, MyFrame::OnLoadVolume)
  EVT_MENU(ID_LoadLabel, MyFrame::OnLoadLabel)

  EVT_MENU(ID_STD, MyFrame::OnLoadSTD)
  EVT_MENU(ID_SIMPLESTD, MyFrame::OnLoadSIMPLESTD)
  EVT_MENU(ID_CLUTO, MyFrame::OnLoadCLUTO)
  EVT_MENU(ID_OPF, MyFrame::OnLoadLIBOPF)
  EVT_MENU(ID_OPFDAT, MyFrame::OnLoadLIBOPF)
  EVT_MENU(ID_MATLAB, MyFrame::OnLoadMATLAB)

  EVT_MENU(ID_SaveVolume, MyFrame::OnSaveVolume)
  EVT_MENU(ID_SaveLabel, MyFrame::OnSaveLabel)
  EVT_MENU(ID_SaveObject, MyFrame::OnSaveObject)
  EVT_MENU(ID_SaveObjectMask, MyFrame::OnSaveObjectMask)
  EVT_MENU(ID_LoadMark, MyFrame::OnLoadMarkers)
  EVT_MENU(ID_SaveMark, MyFrame::OnSaveMarkers)
  EVT_MENU(ID_View0, MyFrame::OnSelectView)
  EVT_MENU(ID_View1, MyFrame::OnSelectView)
  EVT_MENU(ID_View2, MyFrame::OnSelectView)
  EVT_MENU(ID_View3, MyFrame::OnSelectView)
  EVT_MENU(ID_View4, MyFrame::OnSelectView)
  EVT_MENU(ID_Zoomin, MyFrame::OnZoomin)
  EVT_MENU(ID_Zoomout, MyFrame::OnZoomout)
  EVT_MENU(ID_ChangeDrawMarker, MyFrame::OnChangeDrawMarker)

  EVT_SIZE(           MyFrame::OnSize)

  EVT_TOOL(ID_Preproc, MyFrame::OnPreprocessing)
  EVT_TOOL(ID_Draw, MyFrame::OnDraw)
  EVT_TOOL(ID_Analysis, MyFrame::OnAnalysis)
  EVT_TOOL(ID_3DVisu, MyFrame::On3DVisualization)
  EVT_TOOL(ID_Classify, MyFrame::OnClassify)
  EVT_TOOL(ID_LoadVolTool, MyFrame::OnLoadVolume)
//EVT_TOOL(ID_Measurements, MyFrame::OnMeasurements)

  EVT_BUTTON(ID_ObjEye, MyFrame::OnShowObjWindow)
  EVT_BUTTON(ID_Zoomin, MyFrame::OnZoomin)
  EVT_BUTTON(ID_Zoomout, MyFrame::OnZoomout)
  EVT_BUTTON(ID_BriContr, MyFrame::OnShowBriContr)
  EVT_CHOICE(ID_ViewOpt, MyFrame::OnChangeViewOptions)
  EVT_COMMAND_SCROLL(ID_BriContrDi, MyFrame::OnChangeBriContr)
//EVT_MOUSEWHEEL(MyFrame::OnChangeBriContr)
END_EVENT_TABLE()

//DECLARE_APP(MyApp)
IMPLEMENT_APP(MyApp)


int  MyApp::OnExit(){
  delete APP;
  return 0;
}


bool MyApp::OnInit(){
  char path[1024];


#ifdef _WIN32
 LPWSTR *szArglist;
    szArglist = CommandLineToArgvW(GetCommandLineW(), &argc);
    int __i;
    if( NULL == szArglist ) {
        wprintf(L"CommandLineToArgvW failed\n");
        return 0;
    } else {
        argv = (wxChar **) malloc(sizeof(char *) * argc);
        for( __i=0; __i<argc; __i++) {
            int tmp = wcslen(szArglist[__i]);
            char *c = (char *) malloc((sizeof(char) * tmp) + 2);
            wcstombs(c, szArglist[__i], tmp);
            argv[__i] = (wxChar *) c;
            argv[__i][tmp]='\0';
        }
    }
    // Free memory allocated for CommandLineToArgvW arguments.
    LocalFree(szArglist);
#endif


  APP = new BIAClass();
  SetTopWindow(APP->Window);

  if(argc==2){
#ifdef _WIN32
    wxString str(argv[1], strlen((char*)argv[1])); //wxSTRING_MAXLEN);
#else
    wxString str(argv[1], wxSTRING_MAXLEN);
#endif
    strcpy(path, str.mb_str());
    /*
    int s=strlen(path);
    if (s>3 && strcasecmp(path + s - 4, ".bia")==0)
      APP->Window->OpenProj(path);
    else
    */
    APP->Window->LoadVolume(path);
  }
  else if(argc==3){
    printf("Usage: bia [project or volume file] [-label label_scene [label_name label_color] ]\n");
    exit(0);
  }
  else if(argc==4){
    wxString str2(argv[2],wxSTRING_MAXLEN);
    wxString aux(_T("-label"));
    if(str2.Cmp(aux)==0){
      wxString str(argv[1], wxSTRING_MAXLEN);
      strcpy(path, str.mb_str());
      APP->Window->LoadVolume(path);
      wxString str3(argv[3], wxSTRING_MAXLEN);
      strcpy(path, str3.mb_str());
      APP->Window->LoadLabel(path,NULL,0);
    }
    else{
      printf("Usage: bia [project or volume file] [-label label_scene [label_name label_color] ]\n");
      exit(0);
    }
  }
  else if(argc==6){
    wxString str2(argv[2],wxSTRING_MAXLEN);
    wxString aux(_T("-label"));
    if(str2.Cmp(aux)==0){
      wxString str(argv[1], wxSTRING_MAXLEN);
      strcpy(path, str.mb_str());
      APP->Window->LoadVolume(path);
      wxString str3(argv[3], wxSTRING_MAXLEN);
      strcpy(path, str3.mb_str());
      wxString str4(argv[4], wxSTRING_MAXLEN);
      char label_name[1024];
      strcpy(label_name, str4.mb_str());
      wxString str5(argv[5], wxSTRING_MAXLEN);
      int label_color=atoi(str5.mb_str());
      APP->Window->LoadLabel(path,label_name,label_color);
    }
    else{
      printf("Usage: bia [project or volume file] [-label label_scene [label_name label_color] ]\n");
      exit(0);
    }
  }
  else if(argc>1){
    printf("Usage: bia [project or volume file] [-label label_scene [label_name label_color] ]\n");
    exit(0);
  }
  return TRUE;
}


//---------------------------------------------------

MyFrame::MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size)
  : wxFrame((wxFrame *)NULL, -1, title, pos, size)
{

  SetIcon(brain_xpm);

  //Menubar
  wxMenu *menuFile = new wxMenu;
  menuFile->AppendSeparator();

#if wxCHECK_VERSION(2,8,10)
  wxMenu *menuImport = new wxMenu;
  menuImport->Append(ID_STD, _T("Standard File\tCTRL+i"));
#ifdef BIADEBUG
  menuImport->Append(ID_SIMPLESTD, _T("Simple Standard File\tCTRL+g"));
#endif
  menuImport->Append(ID_MATLAB, _T("MATLAB\tCTRL+b"));
  menuImport->Append(ID_CLUTO,  _T("CLUTO\tCTRL+m"));
  menuImport->Append(ID_OPF,  _T("LibOPF"));
  wxMenu *menuExport = new wxMenu;
  menuExport->Append(ID_SaveVolume, _T("Export Data...\tCTRL+e"));

  wxMenu *menuAdd = new wxMenu;
  menuAdd->Append(ID_OPFDAT, _T("Add OPF .dat file"));

  //menuExport->Append(ID_SaveLabel, _T("Export Label...\tALT+b"));
  //menuExport->Append(ID_SaveObject, _T("Export Object...\tALT+o"));
  //menuExport->Append(ID_SaveObjectMask, _T("Export Object Mask...\tALT+o"));
  //menuExport->Append(ID_SaveMark,  _T("Export Markers...\tALT+m"));
  menuFile->AppendSubMenu(menuImport,_T("Import..."),_T("Import data"));
  menuFile->AppendSubMenu(menuExport,_T("Export..."),_T("Export data"));
  menuFile->AppendSubMenu(menuAdd,_T("Add Points..."),_T("Add Points from file"));





#else
  menuFile->Append(ID_LoadVol, _T("Import Volume...\tCTRL+i"));
#ifdef BIADEBUG
  menuFile->Append(ID_LoadGrad, _T("Import Gradient...\tCTRL+g"));
#endif
  menuFile->Append(ID_LoadLabel, _T("Import Label...\tCTRL+b"));
  menuFile->Append(ID_LoadMark,  _T("Import Markers...\tCTRL+m"));
  menuFile->Append(ID_SaveVolume, _T("Export Volume...\tCTRL+e"));
  menuFile->Append(ID_SaveLabel, _T("Export Label...\tALT+b"));
  menuFile->Append(ID_SaveObjectMask, _T("Export Object Mask...\tALT+o"));
  menuFile->Append(ID_SaveMark,  _T("Export Markers...\tALT+m"));
#endif

  menuFile->AppendSeparator();
  menuFile->Append(ID_VolInfo,  _T("Volume &Information\tF1"));
#ifdef BIADEBUG
  menuFile->AppendSeparator();
  //menuFile->Append(ID_ParamEstimation, _T("Parameters Estimation\tF2"));
#endif
  menuFile->AppendSeparator();
  menuFile->Append(ID_Quit, _T("E&xit"));


  wxMenu *menuView = new wxMenu;
  menuView->Append(ID_View0, _T("&All views...\tCTRL+0"));
  menuView->Append(ID_View1, _T("View&1...\tCTRL+1"));
  menuView->Append(ID_View2, _T("View&2...\tCTRL+2"));
  menuView->Append(ID_View3, _T("View&3...\tCTRL+3"));
  menuView->Append(ID_View4, _T("View&4...\tCTRL+4"));
  menuView->AppendSeparator();
  menuView->Append(ID_Zoomin, _T("Zoom &In\tCTRL+="));
  menuView->Append(ID_Zoomout, _T("Zoom &Out\tCTRL+-"));
  menuView->AppendSeparator();
  menuView->Append(ID_ChangeDrawMarker, _T("Show/Hide Cursor\tspace"));

  wxMenu *menuHelp = new wxMenu;
  menuHelp->Append(ID_About,  _T("&About...\tF12"));

  wxMenuBar *menuBar = new wxMenuBar;
  menuBar->Append( menuFile,   _T("&File") );
  menuBar->Append( menuView,   _T("&View") );
  menuBar->Append( menuHelp,   _T("&Help") );

  SetMenuBar( menuBar );

  //Toolbar
  //wxBitmap *bmNew   = new wxBitmap(new_xpm);
  //wxBitmap *bmOpen  = new wxBitmap(open_xpm);
  //wxBitmap *bmSave  = new wxBitmap(save_xpm);

  wxBitmap *bmVisual    = new wxBitmap(navigator_xpm);
  wxBitmap *bmAnalysis  = new wxBitmap(curve_xpm);
  //wxBitmap *bmRegistration  = new wxBitmap(reg_xpm);
  //wxBitmap *bmPreproc   = new wxBitmap(preproc_xpm);
  wxBitmap *bmSegment   = new wxBitmap(paint_xpm);
  wxBitmap *bmClassify  = new wxBitmap(classify_xpm);
  wxBitmap *bmTrash  = new wxBitmap(trash_xpm);
  wxBitmap *bmAdd = new wxBitmap(add_xpm);


  //wxBitmap *bmMeasurements  = new wxBitmap(measure_xpm);
  //wxBitmap *bmGroup  = new wxBitmap(group_xpm);

  toolBar = CreateToolBar(wxTB_HORIZONTAL | wxTB_TEXT, -1, _T("toolBar"));

  //********************************
  //wxBitmapButton *butPreproc = new wxBitmapButton(toolBar, ID_Preproc, *bmPreproc, wxDefaultPosition, wxDefaultSize, wxBU_AUTODRAW, wxDefaultValidator, _T("Preprocessing"));

  /*
  LabeledBitmapButton *butPreproc;
  butPreproc = new LabeledBitmapButton(toolBar, ID_Preproc,
				       *bmPreproc, _T("Preproc"));
  //butPreproc->Update();

  wxColour wxcolor;
  SetColor(&wxcolor, 0xffffdd);
  butPreproc->SetBackgroundColour(wxcolor);

  //butPreproc->SetBitmapDisabled(*bmAnalysis);
  //butPreproc->Disable();

  toolBar->AddControl((wxControl*)butPreproc);

  //wxBitmap bmdis = butPreproc->GetBitmapDisabled();
  //bmdis.SaveFile(_T("disa.bmp"), wxBITMAP_TYPE_BMP, NULL);
  */

  //********************************

  toolBar->AddSeparator();

  toolBar->AddTool(ID_Draw, _T("Draw"), *bmSegment,
                   _T("Draw"), wxITEM_NORMAL);
  /*toolBar->AddTool(ID_Preproc, _T("Transformations"), *bmPreproc,
                   _T("Transformations"), wxITEM_NORMAL);*/
  toolBar->AddTool(ID_3DVisu, _T("Visualization"), *bmVisual,
                   _T("Visualization"), wxITEM_NORMAL);
  toolBar->AddTool(ID_Classify, _T("Classify"), *bmClassify,
                   _T("Classify"), wxITEM_NORMAL);
  toolBar->AddTool(ID_Analysis, _T("Analysis"), *bmAnalysis,
                   _T("Analysis"), wxITEM_NORMAL);
  toolBar->AddSeparator();

  /*toolBar->AddTool(ID_Measurements, _T("Measurements"), *bmMeasurements,
                   _T("Measurements"), wxITEM_NORMAL);*/

  toolBar->Realize();


  //Display
  wxBoxSizer *hsizer = new wxBoxSizer(wxHORIZONTAL);

  panel = new wxPanel(this, wxID_ANY, wxDefaultPosition,
		      wxDefaultSize, wxBORDER_NONE);//wxBORDER_SIMPLE);
  //panel->SetBackgroundColour(*wxBLUE);
  splitter = new wxSplitterWindow(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxSP_3D, _T("hsplitter"));

  topWindow = new wxSplitterWindow(splitter, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxSP_3D, _T("top"));
  topWindow->Show(true);

  bottomWindow = new wxSplitterWindow(splitter, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxSP_3D, _T("bottom"));
  bottomWindow->Show(true);

  splitter->SetMinimumPaneSize(20);  // Set this to prevent unsplitting
  splitter->SplitHorizontally( topWindow, bottomWindow );
  splitter->SetSashGravity(0.5);

  Views[0] = NULL;
  Views[1] = NULL;
  Views[2] = NULL;
  Views[3] = NULL;

  vsizer = new wxBoxSizer(wxVERTICAL);

  wxArrayString dimstr;
  dimstr.Add(_T("x"));
  dimstr.Add(_T("y"));
  dimstr.Add(_T("z"));

  chDim[0] = new wxChoice(panel, wxID_ANY, wxDefaultPosition, wxDefaultSize, dimstr, 0, wxDefaultValidator, _T("choiceDimension"));
  wxStaticText *tvisdim = new wxStaticText(panel, -1, _T("Visible"), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT, _T("VisibleDim"));

  wxButton *butAddDimension = new wxBitmapButton(panel, wxID_ANY, *bmAdd, wxDefaultPosition, wxDefaultSize, wxBU_AUTODRAW, wxDefaultValidator, _T("butAddDimension"));
  wxButton *butDeleteDimension = new wxBitmapButton(panel, wxID_ANY, *bmTrash, wxDefaultPosition, wxDefaultSize, wxBU_AUTODRAW, wxDefaultValidator, _T("butDeleteDimension"));
  wxButton *butRenameDimension = new wxButton(panel, wxID_ANY, _T("Rename"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("butRenameDimension"));

  wxBoxSizer *hbsAddDelDim = new wxBoxSizer(wxHORIZONTAL);
  hbsAddDelDim->Add(butAddDimension, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
  hbsAddDelDim->Add(butDeleteDimension, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
  hbsAddDelDim->Add(butRenameDimension, 1, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);

  chDim[1] = new wxChoice(panel, wxID_ANY, wxDefaultPosition, wxDefaultSize, dimstr, 0, wxDefaultValidator, _T("chDim[1]"));
  chDim[2] = new wxChoice(panel, wxID_ANY, wxDefaultPosition, wxDefaultSize, dimstr, 0, wxDefaultValidator, _T("chDim[2]"));
  chDim[3] = new wxChoice(panel, wxID_ANY, wxDefaultPosition, wxDefaultSize, dimstr, 0, wxDefaultValidator, _T("chDim[3]"));

  chDim[2]->SetSelection(1);
  chDim[3]->SetSelection(2);

  wxBoxSizer *hbsVisibleDim = new wxBoxSizer(wxHORIZONTAL);
  hbsVisibleDim->Add(chDim[1], 1, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
  hbsVisibleDim->Add(chDim[2], 1, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
  hbsVisibleDim->Add(chDim[3], 1, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);

  wxStaticBoxSizer *dissizer = new wxStaticBoxSizer(wxVERTICAL, panel, _T("Display"));
  wxStaticBoxSizer *objsizer = new wxStaticBoxSizer(wxVERTICAL, panel, _T("Objects"));
  wxStaticBoxSizer *dimsizer = new wxStaticBoxSizer(wxVERTICAL, panel, _T("Dimensions"));

  wxBitmap *bmeye      = new wxBitmap(tools_xpm); //eye_xpm);
  wxBitmap *bmZoomin   = new wxBitmap(zoomin_xpm);
  wxBitmap *bmZoomout  = new wxBitmap(zoomout_xpm);
  wxBitmap *bmBriContr = new wxBitmap(brightcontr_xpm);

  buteye = new wxBitmapButton(panel, ID_ObjEye, *bmeye, wxDefaultPosition, wxDefaultSize, wxBU_AUTODRAW, wxDefaultValidator, _T("button_eye"));
  objsizer->Add(buteye, 0, wxALIGN_CENTER|wxALIGN_CENTER_VERTICAL);

  BCDialog = new BriContrDialog(this, ID_BriContrDi);
  buZoomin = new wxBitmapButton(panel, ID_Zoomin, *bmZoomin, wxDefaultPosition, wxDefaultSize, wxBU_AUTODRAW, wxDefaultValidator, _T("butZoomin"));
  buZoomout = new wxBitmapButton(panel, ID_Zoomout, *bmZoomout, wxDefaultPosition, wxDefaultSize, wxBU_AUTODRAW, wxDefaultValidator, _T("butZoomout"));
  buBriContr = new wxBitmapButton(panel, ID_BriContr, *bmBriContr, wxDefaultPosition, wxDefaultSize, wxBU_AUTODRAW, wxDefaultValidator, _T("butBriContr"));
  wxBoxSizer *hbsButton    = new wxBoxSizer(wxHORIZONTAL);
  hbsButton->Add(buZoomin,   1, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
  hbsButton->Add(buZoomout,  1, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
  hbsButton->Add(buBriContr, 1, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);

  //wxStaticText *tView = new wxStaticText(panel, -1, _T("View"), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT, _T("staticText"));

  wxString vChoices[3];
  vChoices[0] = _T("Density");
  vChoices[1] = _T("Data Points");
  vChoices[2] = _T("Points & Density");
  chView = new wxChoice(panel, ID_ViewOpt, wxDefaultPosition, wxDefaultSize, 3, vChoices, 0, wxDefaultValidator, _T("choice0"));
  chView->SetSelection(2);

  wxBoxSizer *hbsView = new wxBoxSizer(wxHORIZONTAL);
  //hbsView->Add(tView, 1, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
  hbsView->Add(chView, 0, wxALIGN_RIGHT);


  dissizer->Add(hbsButton, 0, wxEXPAND);
  dissizer->Add(hbsView,   0, wxEXPAND);

  dimsizer->Add(chDim[0], 0, wxEXPAND);
  dimsizer->Add(hbsAddDelDim, 0, wxEXPAND);
  dimsizer->AddSpacer(15);
  dimsizer->Add(tvisdim, 0, wxEXPAND);
  dimsizer->Add(hbsVisibleDim, 0, wxEXPAND);

  vsizer->AddSpacer(15);
  vsizer->Add(dissizer, 0, wxEXPAND);
  vsizer->AddSpacer(5);
  vsizer->Add(objsizer, 0, wxEXPAND);
  vsizer->AddSpacer(15);
  vsizer->Add(dimsizer, 0, wxEXPAND);
  panel->SetSizer(vsizer, true);
  vsizer->SetSizeHints(panel);

  hsizer->Add(panel,    0, wxALIGN_LEFT|wxEXPAND);
  hsizer->Add(splitter, 1, wxEXPAND);
  SetSizer(hsizer, true);
  hsizer->SetSizeHints(this);

  Connect(butAddDimension->GetId(), wxEVT_COMMAND_BUTTON_CLICKED,
            wxCommandEventHandler(MyFrame::OnAddDimension));
  Connect(butDeleteDimension->GetId(), wxEVT_COMMAND_BUTTON_CLICKED,
            wxCommandEventHandler(MyFrame::OnDeleteDimension));
  Connect(butRenameDimension->GetId(), wxEVT_COMMAND_BUTTON_CLICKED,
            wxCommandEventHandler(MyFrame::OnRenameDimension));
  int i;
  for(i=1;i<4;i++)
    Connect(chDim[i]->GetId(), wxEVT_COMMAND_CHOICE_SELECTED,
            wxCommandEventHandler(MyFrame::OnChoiceDimension));


  wxSystemScreenType sst = wxSystemSettings::GetScreenType();
  if (sst==wxSYS_SCREEN_TINY || sst==wxSYS_SCREEN_PDA) {
    SetSize(320, 240);
  }
  else if (sst==wxSYS_SCREEN_SMALL) {
    SetSize(640, 480);
  }
  else if (sst==wxSYS_SCREEN_DESKTOP || sst==wxSYS_SCREEN_NONE) {
    SetSize(800, 600);
  }
  else {
    SetSize(800, 600);
  }

  this->B = 50;
  this->C = 0;
  this->zoom = 1.0;
  vsizer->Layout();

  //Statusbar
  CreateStatusBar();
  SetStatusText( _T("Welcome to SDGPR - Synthetic Database Gerenator for Pattern Recognition!") );
}


MyFrame::~MyFrame(){
  (APP->Preferences).IsMaximized = this->IsMaximized();
}


void MyFrame::AppendOptPanel(wxPanel *panel,
			     Module::ModuleType type){
  //wxWindow *parent = vsizer->GetContainingWindow();
  wxWindow *parent = (wxWindow *)this->panel;
  if(parent==NULL || panel==NULL) return;

  panel->Reparent(parent);
  panel->Show(true);

  if(vsizer->GetItem(panel,false)!=NULL)
    vsizer->Detach(panel);

  this->SetOptPanelBkgColor(panel, type);
  vsizer->Add(panel, 0, wxEXPAND);
  vsizer->SetSizeHints(parent);
  vsizer->Layout();
}


void MyFrame::PrependOptPanel(wxPanel *panel,
			      Module::ModuleType type){
  //wxWindow *parent = vsizer->GetContainingWindow();
  wxWindow *parent = (wxWindow *)this->panel;
  if(parent==NULL || panel==NULL) return;

  panel->Reparent(parent);
  panel->Show(true);

  if(vsizer->GetItem(panel,false)!=NULL)
    vsizer->Detach(panel);

  this->SetOptPanelBkgColor(panel, type);
  vsizer->Prepend(panel, 0, wxEXPAND);
  vsizer->SetSizeHints(parent);
  vsizer->Layout();
}


bool MyFrame::DetachOptPanel(wxPanel *panel){
  //wxWindow *parent = vsizer->GetContainingWindow();
  wxWindow *parent = (wxWindow *)this->panel;
  bool ret;

  if(parent==NULL || panel==NULL) return false;

  panel->Hide();
  ret = vsizer->Detach(panel);
  vsizer->SetSizeHints(parent);
  vsizer->Layout();
  return ret;
}


void MyFrame::EnableObjWindow(bool enable){
  this->buteye->Enable(enable);
}


void MyFrame::SetBriContr(int B, int C){
  BCDialog->SetBCLevel(B, C);
}

void MyFrame::GetBriContr(int *B, int *C){
  BCDialog->GetBCLevel(B, C);
}

void MyFrame::GetViewOptions(ViewOptType *vopt){
  *vopt = (ViewOptType)chView->GetSelection();
}

void MyFrame::SetOptPanelBkgColor(wxPanel *panel,
				  Module::ModuleType type){
  wxColour wxcolor;
  int color;

  switch(type){
  case Module::PREPROC:
    color = (APP->Preferences).color_preproc;
    break;
  case Module::DRAW:
    color = (APP->Preferences).color_segmentation;
    break;
  case Module::ANALYSIS:
    color = (APP->Preferences).color_analysis;
    break;
  case Module::CLASSIFY:
    color = (APP->Preferences).color_analysis;
    break;
  case Module::VIEW2D:
    color = (APP->Preferences).color_view;
    break;
  case Module::VIEW3D:
    color = (APP->Preferences).color_view;
    break;
  default:
    return;
  }
  if(color==NIL) return;
  SetColor(&wxcolor, color);
  panel->SetBackgroundColour(wxcolor);
}


void MyFrame::SetViewPanel(wxPanel *view, int pos){
  wxSplitterWindow *splitter;

  if(pos==0 || pos==1)
    splitter = topWindow;
  else if(pos==2 || pos==3)
    splitter = bottomWindow;
  else return;

  // Set this to prevent unsplitting
  splitter->SetMinimumPaneSize(20);

  view->Reparent(splitter);
  if(Views[pos]!=NULL){
    splitter->ReplaceWindow(Views[pos], view);
    delete Views[pos];
    Views[pos] = view;
    Views[pos]->Show(true);
  }
  else{
    Views[pos] = view;
    Views[pos]->Show(true);
    if(splitter->IsSplit())
      splitter->Unsplit(NULL);

    if(pos%2==0){
      if(Views[pos+1]!=NULL)
	splitter->SplitVertically(view, Views[pos+1]);
      else
	splitter->Initialize(view);
    }
    else{
      if(Views[pos-1]!=NULL)
	splitter->SplitVertically(Views[pos-1], view);
      else
	splitter->Initialize(view);
    }
  }
  splitter->SetSashGravity(0.5);
}


void MyFrame::UniformView(){
  int w,h;
  /*
  this->splitter->Layout();
  this->topWindow->Layout();
  this->bottomWindow->Layout();
  this->splitter->Update();
  this->topWindow->Update();
  this->bottomWindow->Update();
  */
  splitter->GetSize(&w, &h);
  splitter->SetSashPosition(h/2, true);

  topWindow->GetSize(&w, &h);
  topWindow->SetSashPosition(w/2, true);

  bottomWindow->GetSize(&w, &h);
  bottomWindow->SetSashPosition(w/2, true);
}

/*
void MyFrame :: OnShowObjWindow(wxCommandEvent & event){
  static ObjectDialog *dialog = NULL;

  if(!APP->Data.loaded) return;

  if(APP->GetNumberOfObjs()==0){
    wxMessageBox(_T("No objects available."), _T("Warning"),
		 wxOK | wxICON_EXCLAMATION, this);
    return;
  }
  else{
    //if(dialog!=NULL)
    //  delete dialog;
    dialog = new ObjectDialog(this);
    dialog->ShowModal();
    wxTheApp->Yield();
  }
}
*/

void MyFrame :: OnShowObjWindow(wxCommandEvent & event){
  //static ObjectDialog *dialog = NULL;

  if(!APP->Data.loaded) return;

  /*
  if(APP->GetNumberOfObjs()==0){
    wxMessageBox(_T("No objects available."), _T("Warning"),
		 wxOK | wxICON_EXCLAMATION, this);
    return;
  }
  else{
    //if(dialog!=NULL)
    //  delete dialog;
    dialog = new ObjectDialog(this);
    wxSize size(500, 400);
    dialog->SetSize(wxDefaultCoord, 400);
    dialog->SetMaxSize(size);
    dialog->ShowModal();
    wxTheApp->Yield();
  }
  */
}


void MyFrame::OnZoomin(wxCommandEvent& WXUNUSED(event)){
  if(!APP->Data.loaded) return;

  if(this->zoom>=15.9999) return;
  this->zoom *= 1.25;
  APP->SetZoomLevel(this->zoom);
}


void MyFrame::OnZoomout(wxCommandEvent& WXUNUSED(event)){
  if(!APP->Data.loaded) return;

  if(this->zoom<=0.5001) return;
  this->zoom *= 0.8;
  APP->SetZoomLevel(this->zoom);
}

void MyFrame::OnChangeDrawMarker(wxCommandEvent& WXUNUSED(event)){
  if(!APP->Data.loaded) return;

  APP->ChangeDrawMarker();
}


void MyFrame::OnChangeBriContr(wxScrollEvent& event){
  APP->Refresh2DCanvas();
  APP->Refresh3DCanvas(false, 1.0);
}


void MyFrame::OnShowBriContr(wxCommandEvent& WXUNUSED(event)){
  BCDialog->ShowWindow();
}

void MyFrame::OnChangeViewOptions(wxCommandEvent& event){
  APP->Refresh2DCanvas();
  APP->Refresh3DCanvas(false, 1.0);
}


void MyFrame::OnSaveVolume(wxCommandEvent & event){
  char *dir = (APP->Preferences).dir_SaveVolume;
  int s,filter;

  static const
    wxChar
    *FILETYPES = _T( "Standard File|*.txt|"
                    "Simple Standard File|*.simple"
		     "OPF Format(*.opf)|*.opf|"
		     "MatLab Format (*.mat)|*.mat|"
		     "CLUTO Format (*.cluto)|*.cluto|"
		     "GNUPLOT Format (*.gnuplot)|*.gnuplot|"
		     );

  if(!APP->Data.loaded){
    wxMessageBox(_T("There is no volume loaded!"), _T("No Data"), wxOK | wxICON_EXCLAMATION, this);
    return;
  }

  wxFileDialog *saveFileDialog =
    new wxFileDialog ( this,
		       _T("Save Data"),
		       _T(""),
		       _T(""),
		       FILETYPES,
		       wxSAVE|wxOVERWRITE_PROMPT,
		       wxDefaultPosition);

  saveFileDialog->SetFilterIndex((APP->Preferences).ext_SaveVolume);
  if(dir[0]!='\0'){
    wxString wxdir(dir, wxConvUTF8);
    saveFileDialog->SetDirectory(wxdir);
  }
  if(saveFileDialog->ShowModal() == wxID_OK){
    wxBusyCursor wait;
    SetStatusText(_T("Saving file ")+saveFileDialog->GetFilename());

    (APP->Preferences).ext_SaveVolume = saveFileDialog->GetFilterIndex();

    char path[512];
    strcpy( dir,  saveFileDialog->GetDirectory().ToAscii() );
    strcpy( path, saveFileDialog->GetPath().ToAscii() );

    // Set the extension if missing
    s = strlen(path);
    filter = saveFileDialog->GetFilterIndex();
    APP->Data.DL.ExportFile(path, filter);

  SetStatusText(_T("Saving file ")+saveFileDialog->GetFilename()+_T("... Done"));
  }
}

void MyFrame::OnSaveLabel(wxCommandEvent & event){
  char *dir = (APP->Preferences).dir_SaveLabel;
  Scene *flabel;
  int s,filter;

  static const
    wxChar
    *FILETYPES = _T( "Supported Formats (*.scn;*.hdr;*.scn.bz2)|*.scn;*.hdr;*.scn.bz2|"
		     "Scene files (*.scn)|*.scn|"
		     "Analyze files (*.hdr)|*.hdr|"
		     "Compressed Scene files (*.scn.bz2)|*.scn.bz2|"
		     "Nifti-1 files (*.nii)|*.nii|"
		     "Compressed Nifti-1 files (*.nii.gz)|*.nii.gz|"
		     "All files|*.*"
		     );

  if(!APP->Data.loaded){
    wxMessageBox(_T("There is no volume loaded!"), _T("No Data"), wxOK | wxICON_EXCLAMATION, this);
    return;
  }

  wxString wxlabelname((APP->Data).labelname, wxConvUTF8);

  wxFileDialog *saveFileDialog =
    new wxFileDialog ( this,
		       _T("Save Label"),
		       _T(""),
		       wxlabelname,
		       FILETYPES,
		       wxSAVE|wxOVERWRITE_PROMPT,
		       wxDefaultPosition);

  saveFileDialog->SetFilterIndex((APP->Preferences).ext_SaveLabel);
  if(dir[0]!='\0'){
    wxString wxdir(dir, wxConvUTF8);
    saveFileDialog->SetDirectory(wxdir);
  }
  if(saveFileDialog->ShowModal() == wxID_OK){
    wxBusyCursor wait;
    SetStatusText(_T("Saving file ")+saveFileDialog->GetFilename());

    (APP->Preferences).ext_SaveLabel = saveFileDialog->GetFilterIndex();

    char path[512];
    strcpy( dir,  saveFileDialog->GetDirectory().ToAscii() );
    strcpy( path, saveFileDialog->GetPath().ToAscii() );
    //flabel = CopyScene(APP->Data.label);
    //flabel = APP->CopyVisibleLabelUnion();
    flabel = APP->CopyDistinctVisibleLabels();
    CloneNiftiHeader( APP->Data.den[0], flabel );

    s = strlen(path);
    filter = saveFileDialog->GetFilterIndex();
    if ((s<4 || strcasecmp(path + s - 4, ".scn")!=0) && filter==1 )
      strcat(path,".scn");
    else if ((s<4 || strcasecmp(path + s - 4, ".hdr")!=0) && filter==2 )
      strcat(path,".hdr");
    else if ((s<8 || strcasecmp(path + s - 8, ".scn.bz2")!=0) && filter==3 )
      strcat(path,".scn.bz2");
    else if ((s<4 || strcasecmp(path + s - 4, ".nii")!=0) && filter==4 )
      strcat(path,".nii");
    else if ((s<7 || strcasecmp(path + s - 7, ".nii.gz")!=0) && filter==5 )
      strcat(path,".nii.gz");

    if (strcasecmp(path + s - 4, ".scn")==0 || strcasecmp(path + s - 4, ".hdr")==0 || strcasecmp(path + s - 4, ".img")==0 || strcasecmp(path + s - 8, ".scn.bz2")==0 || strcasecmp(path + s - 4, ".nii")==0 || strcasecmp(path + s - 7, ".nii.gz")==0)
      WriteScene(flabel, path);
    else {
      wxMessageBox(_T("Unsupported extension!"), _T("Error"), wxOK | wxICON_EXCLAMATION, this);
      DestroyScene(&flabel);
      return;
    }

    DestroyScene(&flabel);

    SetStatusText(_T("Saving file ")+saveFileDialog->GetFilename()+_T("... Done"));
  }
}

void MyFrame::OnSaveObject(wxCommandEvent & event){
  APP->Data.DL.ExportFile("DL.txt",PSet::STD);
  APP->Data.labelDL.ExportFile("labelDL.txt",PSet::STD);
}


void MyFrame::OnSaveObjectMask(wxCommandEvent & event){

  APP->Data.DL.ExportFile("DLOPF.txt",PSet::LIBOPF);


  /*
  char *dir = (APP->Preferences).dir_SaveObject;
  int s,filter;

  static const
    wxChar
    *FILETYPES = _T( "Supported Formats (*.scn;*.hdr;*.scn.bz2)|*.scn;*.hdr;*.scn.bz2|"
		     "Scene files (*.scn)|*.scn|"
		     "Analyze files (*.hdr)|*.hdr|"
		     "Compressed Scene files (*.scn.bz2)|*.scn.bz2|"
		     "Nifti-1 files (*.nii)|*.nii|"
		     "Compressed Nifti-1 files (*.nii.gz)|*.nii.gz|"
		     "All files|*.*"
		     );

  if(!APP->Data.loaded){
    wxMessageBox(_T("There is no volume loaded!"), _T("No Data"), wxOK | wxICON_EXCLAMATION, this);
    return;
  }

  wxFileDialog *saveFileDialog =
    new wxFileDialog ( this,
		       _T("Save Object"),
		       _T(""),
		       _T(""),
		       FILETYPES,
		       wxSAVE|wxOVERWRITE_PROMPT,
		       wxDefaultPosition);

  saveFileDialog->SetFilterIndex((APP->Preferences).ext_SaveObject);
  if(dir[0]!='\0'){
    wxString wxdir(dir, wxConvUTF8);
    saveFileDialog->SetDirectory(wxdir);
  }

  SegmObject *obj = APP->GetObjByDialog();

  if(saveFileDialog->ShowModal() == wxID_OK){
    wxBusyCursor wait;
    SetStatusText(_T("Saving file ")+saveFileDialog->GetFilename());

    (APP->Preferences).ext_SaveObject = saveFileDialog->GetFilterIndex();

    char path[512];

    strcpy( dir,  saveFileDialog->GetDirectory().ToAscii() );
    strcpy( path, saveFileDialog->GetPath().ToAscii() );


    s = strlen(path);
    filter = saveFileDialog->GetFilterIndex();
    if ((s<4 || strcasecmp(path + s - 4, ".scn")!=0) && filter==1 )
      strcat(path,".scn");
    else if ((s<4 || strcasecmp(path + s - 4, ".hdr")!=0) && filter==2 )
      strcat(path,".hdr");
    else if ((s<8 || strcasecmp(path + s - 8, ".scn.bz2")!=0) && filter==3 )
      strcat(path,".scn.bz2");
    else if ((s<4 || strcasecmp(path + s - 4, ".nii")!=0) && filter==4 )
      strcat(path,".nii");
    else if ((s<7 || strcasecmp(path + s - 7, ".nii.gz")!=0) && filter==5 )
      strcat(path,".nii.gz");

    if (strcasecmp(path + s - 4, ".scn")==0 || strcasecmp(path + s - 4, ".hdr")==0 || strcasecmp(path + s - 4, ".img")==0 || strcasecmp(path + s - 8, ".scn.bz2")==0 || strcasecmp(path + s - 4, ".nii")==0 || strcasecmp(path + s - 7, ".nii.gz")==0){
      Scene *scn1 = CreateScene(APP->Data.den->xsize,APP->Data.den->ysize,APP->Data.den->zsize);
      CloneNiftiHeader( APP->Data.den, scn1 );
      CopyBMap2SceneMask(scn1,obj->mask);
      WriteScene(scn1,path);
      DestroyScene(&scn1);
    }
    else {
      wxMessageBox(_T("Unsupported extension!"), _T("Error"), wxOK | wxICON_EXCLAMATION, this);
      return;
    }


    SetStatusText(_T("Saving file ")+saveFileDialog->GetFilename()+_T("... Done"));

  }
  */
}





void MyFrame::OnSaveMarkers(wxCommandEvent & event){
  /*
  char *dir = (APP->Preferences).dir_SaveMarkers;
  int p,n,s,filter,nseeds,id,lb,x,y,z;
  FILE *fp;

  static const
    wxChar
    *FILETYPES = _T( "Text files (.TXT)|*.txt|"
		     "All files|*.*"
		     );

  if(!APP->Data.loaded){
    wxMessageBox(_T("There is no volume loaded!"), _T("No Data"), wxOK | wxICON_EXCLAMATION, this);
    return;
  }

  wxFileDialog *saveFileDialog =
    new wxFileDialog ( this,
		       _T("Save Markers"),
		       _T(""),
		       _T(""),
		       FILETYPES,
		       wxSAVE|wxOVERWRITE_PROMPT,
		       wxDefaultPosition);

  saveFileDialog->SetFilterIndex((APP->Preferences).ext_SaveMarkers);
  if(dir[0]!='\0'){
    wxString wxdir(dir, wxConvUTF8);
    saveFileDialog->SetDirectory(wxdir);
  }
  if(saveFileDialog->ShowModal() == wxID_OK){
    wxBusyCursor wait;
    SetStatusText(_T("Saving file ")+saveFileDialog->GetFilename());

    (APP->Preferences).ext_SaveMarkers = saveFileDialog->GetFilterIndex();

    char path[512];
    strcpy( dir,  saveFileDialog->GetDirectory().ToAscii() );
    strcpy( path, saveFileDialog->GetPath().ToAscii() );

    s = strlen(path);
    filter = saveFileDialog->GetFilterIndex();
    if( (s<3 || strcasecmp(path + s - 3, "txt")!=0) && filter==0 )
      strcat(path,".txt");


    nseeds = 0;
    n = APP->Data.w*APP->Data.h*APP->Data.nframes;
    for(p=0; p<n; p++){
      if(APP->IsSeed(p))
	nseeds++;
    }

    fp = fopen(path,"w");
    if (!fp){
      wxMessageBox(_T("Error opening file for writing!"), _T("Error"), wxOK | wxICON_EXCLAMATION, this);
      return;
    }
    fprintf(fp,"%d\n",nseeds);

    for(p=0; p<n; p++){
      if(APP->IsSeed(p)){
	lb = APP->GetSeedLabel(p);
	id = APP->GetSeedId(p);

	x = VoxelX(APP->Data.label, p);
	y = VoxelY(APP->Data.label, p);
	z = VoxelZ(APP->Data.label, p);
	fprintf(fp,"%d %d %d %d %d\n",x,y,z,id,lb);
      }
    }
    fclose(fp);

    SetStatusText(_T("Saving file ")+saveFileDialog->GetFilename()+_T("... Done"));
  }
  */
}


void MyFrame::OnLoadMarkers(wxCommandEvent & event)
{
  /*
  char *dir = (APP->Preferences).dir_LoadMarkers;
  int i,p,nseeds,x,y,z,id,lb;
  FILE *fp;

  static const
    wxChar
    *FILETYPES = _T( "Text files (.TXT)|*.txt|"
		     "All files|*.*"
		     );

  if(!APP->Data.loaded){
    wxMessageBox(_T("There is no volume loaded!"), _T("No Data"), wxOK | wxICON_EXCLAMATION, this);
    return;
  }

  wxFileDialog *openFileDialog =
    new wxFileDialog ( this,
		       _T("Load Markers"),
		       _T(""),
		       _T(""),
		       FILETYPES,
		       wxOPEN,
		       wxDefaultPosition);

  openFileDialog->SetFilterIndex((APP->Preferences).ext_LoadMarkers);
  if(dir[0]!='\0'){
    wxString wxdir(dir, wxConvUTF8);
    openFileDialog->SetDirectory(wxdir);
  }
  if(openFileDialog->ShowModal() == wxID_OK){
    wxBusyCursor wait;
    SetStatusText(_T("Loading ")+openFileDialog->GetFilename());

    (APP->Preferences).ext_LoadMarkers = openFileDialog->GetFilterIndex();

    char path[512];
    strcpy( dir,  openFileDialog->GetDirectory().ToAscii() );
    strcpy( path, openFileDialog->GetPath().ToAscii() );

    fp = fopen(path,"r");
    if (!fp){
      wxMessageBox(_T("Error opening file!"), _T("Error"), wxOK | wxICON_EXCLAMATION, this);
      return;
    }

    APP->ResetData();

    fscanf(fp," %d \n",&nseeds);
    for(i=0;i<nseeds;i++) {
      fscanf(fp," %d %d %d %d %d \n",&x,&y,&z,&id,&lb);
      if(ValidVoxel(APP->Data.label,x,y,z)){
	p = VoxelAddress(APP->Data.label,x,y,z);
	APP->AddSeed(p, lb, id);
      }
    }
    fclose(fp);

    SetStatusText(_T("Loading ")+openFileDialog->GetFilename()+_T("... Done"));
    APP->Refresh2DCanvas();
    APP->Refresh3DCanvas(true, 1.0);
  }
  */
}



void MyFrame::OnLoadVolume(wxCommandEvent & event){

  wxFileDialog *openFileDialog =
    new wxFileDialog ( this,
		       _T("Open file"),
		       _T(""),
		       _T(""),
		       wxFileSelectorDefaultWildcardStr,
		       wxOPEN,
		       wxDefaultPosition);

  if(openFileDialog->ShowModal() == wxID_OK){
    wxWindowDisabler disableAll;
    wxBusyInfo busy(_T("Please wait, working..."));
    wxBusyCursor wait;
    SetStatusText(_T("Loading ")+openFileDialog->GetFilename());
    wxTheApp->Yield();
    //printf("%s\n",openFileDialog->GetFilename().ToAscii());
    //printf("%s\n",openFileDialog->GetDirectory().ToAscii());
    (APP->Preferences).ext_LoadVolume = openFileDialog->GetFilterIndex();

    char path[512];
    //strcpy( dir,  openFileDialog->GetDirectory().ToAscii() );
    strcpy( path, openFileDialog->GetPath().ToAscii() );
    //strcpy(APP->Data.volumename, openFileDialog->GetFilename().ToAscii());
    LoadVolume(path);
    SetStatusText(_T("Loading ")+openFileDialog->GetFilename()+_T("... Done"));

    wxCursor *cross=NULL;
    float zoom = APP->GetZoomLevel();
    cross = CrossCursor(ROUND(zoom));
    //wxCROSS_CURSOR
    APP->Set2DViewCursor(cross, 'x');
    APP->Set2DViewCursor(cross, 'y');
    APP->Set2DViewCursor(cross, 'z');
  }
}



void MyFrame::LoadVolume(char *filename){
  Scene *scn=NULL;


  //APP->DestroyData();

  //MyRemoveDirectory(APP->Data.volumename, filename);

  if(APP->Control.LoadedFileExtension == PSet::OPFDAT)
    strcpy(APP->Control.path,filename);
  else
    APP->Data.DL.ImportFile(filename, APP->Control.LoadedFileExtension);

    return;
/*
  char *aux = (char*) calloc(1024,sizeof(char));
  if (strcmp(APP->Data.projectfilename,"")==0)
    sprintf(aux, "BIA - Brain Image Analyzer - %s (Untitled Project)", filename);
  else
    sprintf(aux, "BIA - Brain Image Analyzer - %s", APP->Data.projectfilename);
  wxString wxfilename(aux, wxConvUTF8);
  free(aux);
  SetTitle( wxfilename );
*/
  //APP->SetDataVolume(scn);
  //APP->Data.loaded = 1;
  APP->RefreshAll();
  /*APP->Refresh2DCanvas();
  APP->Refresh3DCanvas(true, 1.0);
  APP->Refresh2DCanvas();
  APP->Refresh3DCanvas(true, 1.0);*/
}

  void MyFrame::OnLoadSTD(wxCommandEvent & event) {APP->Control.LoadedFileExtension = PSet::STD; MyFrame::OnLoadVolume(event);}
  void MyFrame::OnLoadSIMPLESTD(wxCommandEvent & event) {APP->Control.LoadedFileExtension = PSet::SIMPLESTD; MyFrame::OnLoadVolume(event);}
  void MyFrame::OnLoadCLUTO(wxCommandEvent & event) {APP->Control.LoadedFileExtension = PSet::CLUTO; MyFrame::OnLoadVolume(event);}
  void MyFrame::OnLoadLIBOPF(wxCommandEvent & event) {APP->Control.LoadedFileExtension = PSet::LIBOPF; MyFrame::OnLoadVolume(event);}
  void MyFrame::OnLoadOPFDAT(wxCommandEvent & event) {APP->Control.LoadedFileExtension = PSet::OPFDAT; MyFrame::OnLoadVolume(event);}
  void MyFrame::OnLoadMATLAB(wxCommandEvent & event) {APP->Control.LoadedFileExtension = PSet::MATLAB; MyFrame::OnLoadVolume(event);}

void MyFrame::OnLoadLabel(wxCommandEvent & event){
  char *dir = (APP->Preferences).dir_LoadLabel;
  static const
    wxChar
    *FILETYPES = _T( "Scene files (*.scn)|*.scn|"
		     "Analyze files (*.hdr)|*.hdr|"
		     "Compressed Scene files (*.scn.bz2)|*.scn.bz2|"
		     "Nifti-1 files (*.nii)|*.nii|"
		     "Compressed Nifti-1 files (*.nii.gz)|*.nii.gz|"
		     "All files|*.*"
		     );

  if(!APP->Data.loaded){
    wxMessageBox(_T("There is no volume loaded!"),
		 _T("No Data"), wxOK | wxICON_EXCLAMATION, this);
    return;
  }

  if( !modManager->StopLastActiveProcessingModule() ) return;

  wxFileDialog *openFileDialog =
    new wxFileDialog ( this,
		       _T("Load Label"),
		       _T(""),
		       _T(""),
		       FILETYPES,
		       wxOPEN,
		       wxDefaultPosition);

  openFileDialog->SetFilterIndex((APP->Preferences).ext_LoadLabel);
  if(dir[0]!='\0'){
    wxString wxdir(dir, wxConvUTF8);
    openFileDialog->SetDirectory(wxdir);
  }
  if(openFileDialog->ShowModal() == wxID_OK){
    wxWindowDisabler disableAll;
    wxBusyInfo busy(_T("Please wait, working..."));
    wxBusyCursor wait;
    SetStatusText(_T("Loading ")+openFileDialog->GetFilename());
    wxTheApp->Yield();
    //printf("%s\n",openFileDialog->GetFilename().ToAscii());
    //printf("%s\n",openFileDialog->GetDirectory().ToAscii());
    (APP->Preferences).ext_LoadLabel = openFileDialog->GetFilterIndex();

    char path[512];
    strcpy( dir,  openFileDialog->GetDirectory().ToAscii() );
    strcpy( path, openFileDialog->GetPath().ToAscii() );
    //strcpy(APP->Data.volumename, openFileDialog->GetFilename().ToAscii());
    LoadLabel(path,NULL,0);
    SetStatusText(_T("Loading ")+openFileDialog->GetFilename()+_T("... Done"));
    strcpy(APP->Data.labelname, openFileDialog->GetFilename().ToAscii());
  }
}


void MyFrame::LoadLabel(char *filename, char *name, int color){
  /*
  Scene *label;
  int s,p,n,lb,l,Lmax;
  char oname[512];
  int *hist=NULL;
  if (name!=NULL) strcpy(oname,name);

  if(!APP->Data.loaded)
    return;

  s = strlen(filename);
  if( ( strcasecmp(filename + s - 7, "scn.bz2") == 0 ) || ( strcasecmp(filename + s - 3, "scn") == 0 ) || ( strcasecmp(filename + s - 3, "hdr") == 0 ) || ( strcasecmp(filename + s - 3, "img") == 0 ) || ( strcasecmp(filename + s - 3, "nii") == 0 ) || ( strcasecmp(filename + s - 6, "nii.gz") == 0 ) ) {
    label = ReadScene( filename );
  }
  else{
    wxMessageBox(_T("Unable to load volume data. Invalid file format!"),
		 _T("Load Label Error"), wxOK | wxICON_EXCLAMATION, this);
    return;
  }

  if(label->xsize!=APP->Data.w ||
     label->ysize!=APP->Data.h ||
     label->zsize!=APP->Data.nframes){
    DestroyScene(&label);
    wxMessageBox(_T("Incompatible volume data."),
		 _T("Load Label Error"), wxOK | wxICON_EXCLAMATION, this);
    return;
  }

  APP->DelAllSeeds();
  n = APP->Data.w*APP->Data.h*APP->Data.nframes;
  Lmax = MaximumValue3(label);
  hist = AllocIntArray(Lmax+1);
  for(p=0; p<n; p++){
    lb = label->data[p];
    hist[lb]++;
  }

  for(l=1; l<=Lmax; l++){
    if(hist[l]==0) continue;

    if(name==NULL && Lmax==1){
      if(APP->ShowNewObjDialog(&color, oname))
	continue;
    }
    else{
      sprintf(oname,"Label%02d",l);
      color = randomColor(); //0xFFFF00;
    }
    SetScene(APP->Data.label, 0);
    for(p=0; p<n; p++){
      lb = label->data[p];
      if(l==lb)
	(APP->Data.label)->data[p] = lb;
    }
    APP->SetLabelColour(0, NIL);
    APP->SetLabelColour(l, color);
    APP->AddObj(oname, color);
  }

  for(l=1; l<=Lmax; l++){
    sprintf(oname,"Label%02d",l);
    APP->SetObjVisibility(oname, true);
  }

  free(hist);
  DestroyScene(&label);
  APP->DrawSegmObjects();
  */
}


void MyFrame::OnQuit(wxCommandEvent& WXUNUSED(event)){
  Close(TRUE);
}


void MyFrame::OnAbout(wxCommandEvent& WXUNUSED(event)){
  static const char *lic_unicamp = "RESTRICTED FOR RESEARCH AND EDUCATIONAL USAGE";

  char z[2048];

  sprintf(z,"SDGPR\nVersion 1.0/Distribution: unrestricted\n\n");

  strcat(z,
	 "Synthetic Base Generator for Pattern Recognition\n\n"\
	 "(C) 2013-2014 Carlos A. Prete Jr. and Paulo A.V. Miranda\n"\
	 "Developed by Carlos A. Prete Jr. and Paulo A.V. Miranda\n"\
	 "Website: http://www.vision.ime.usp.br/~pmiranda/"\
         "Contact: pavmbr@yahoo.com.br or carlos.prete@usp.br");

  strcat(z, lic_unicamp);

  wxString wxz(z, wxConvUTF8);
  wxMessageBox(wxz, _T("About SDBPR"), wxOK | wxICON_INFORMATION, this);
}



void MyFrame::OnVolumeInformation(wxCommandEvent& WXUNUSED(event)){/*
  char z[1024];
  char textline[512];
  float dx,dy,dz,volume;
  int n,bpp,levels,Imax,Imin;

  if(!APP->Data.loaded){
    wxMessageBox(_T("There is no volume loaded."),
		 _T("Volume Information"), wxOK | wxICON_EXCLAMATION, this);
  }
  else{
    sprintf(z,"Filename: %s\n\n",APP->Data.volumename);

    dx = (APP->Data.den[0])->dx;
    dy = (APP->Data.den[0])->dy;
    dz = (APP->Data.den[0])->dz;
    sprintf(textline,"Voxel size: %.6f x %.6f x %.6f = %.6f units^3\n\n",dx,dy,dz,dx*dy*dz);
    strcat(z, textline);

    n = APP->Data.w*APP->Data.h*APP->Data.nframes;
    volume = (float)n*dx*dy*dz;
    sprintf(textline,"Volume size: %d x %d x %d = %d voxels\n",APP->Data.w,APP->Data.h,APP->Data.nframes,n);
    strcat(z, textline);
    sprintf(textline,"\t\t(%.2f units^3)\n\n",volume);
    strcat(z, textline);

    Imax = MaximumValue3(APP->Data.den[0]);
    Imin = MinimumValue3(APP->Data.den[0]);
    bpp  = 8; //GetRadiometricRes3(APP->Data.den);
    levels = (int)powf(2.0, (float)bpp);
    sprintf(textline,"Radiometric resolution: %d levels (%d bits)\n",levels,bpp);
    strcat(z, textline);

    sprintf(textline,"Maximum Value: %d, Minimum Value: %d\n",Imax,Imin);
    strcat(z, textline);

    wxString wxz(z, wxConvUTF8);
    wxMessageBox(wxz, _T("Volume Information"), wxOK | wxICON_INFORMATION, this);
  }*/
}



void MyFrame::OnSelectView(wxCommandEvent & event){
  //printf("event  id: %d\n",event.GetId());
  int viewtype;
  int w,h,min;
  viewtype = event.GetId();

  if(viewtype == ID_View0) this->UniformView();

  else {
    min = splitter->GetMinimumPaneSize();
    splitter->GetSize(&w, &h);
    if(viewtype < ID_View3) {
      splitter->SetSashPosition(h - min - 1, true);

      min = topWindow->GetMinimumPaneSize();
      topWindow->GetSize(&w, &h);
      if(viewtype == ID_View1)
	topWindow->SetSashPosition(w - min - 1, true);
      else
	topWindow->SetSashPosition(min, true);
    }

    else {
      splitter->SetSashPosition(min, true);

      min = bottomWindow->GetMinimumPaneSize();
      bottomWindow->GetSize(&w, &h);
      if(viewtype == ID_View3)
	bottomWindow->SetSashPosition(w - min - 1, true);
      else
	bottomWindow->SetSashPosition(min, true);
    }
  }
}


void MyFrame::OnPreprocessing(wxCommandEvent& event){

  if(!APP->Data.loaded){
    wxMessageBox(_T("There is no volume loaded."),
		 _T("Preprocessing"), wxOK | wxICON_EXCLAMATION, this);
    return;
  }

  modManager->ShowModuleWindow(Module::PREPROC);
}


void MyFrame::OnDraw(wxCommandEvent& event){

  if(!APP->Data.loaded){
    wxMessageBox(_T("There is no volume loaded."),
		 _T("Draw"), wxOK | wxICON_EXCLAMATION, this);
    return;
  }

  modManager->ShowModuleWindow(Module::DRAW);
}


void MyFrame::OnAnalysis(wxCommandEvent& event){

  if(!APP->Data.loaded){
    wxMessageBox(_T("There is no volume loaded."),
		 _T("Analysis"), wxOK | wxICON_EXCLAMATION, this);
    return;
  }

  modManager->ShowModuleWindow(Module::ANALYSIS);
}


void MyFrame::OnClassify(wxCommandEvent& event){

  if(!APP->Data.loaded){
    wxMessageBox(_T("There is no volume loaded."),
		 _T("Classify"), wxOK | wxICON_EXCLAMATION, this);
    return;
  }


  modManager->ShowModuleWindow(Module::CLASSIFY);

}


void MyFrame::On3DVisualization(wxCommandEvent& event){

  if(!APP->Data.loaded){
    wxMessageBox(_T("There is no volume loaded."),
		 _T("3D Visualization"), wxOK | wxICON_EXCLAMATION, this);
    return;
  }

  modManager->ShowModuleWindow(Module::VIEW3D);
}

void MyFrame::OnAddDimension(wxCommandEvent& event){
    wxTextEntryDialog *Namer;
    Namer = new wxTextEntryDialog(this, _T("Type the Dimension's name"), _T("Name Dimension"), wxEmptyString,
                                    wxTextEntryDialogStyle, wxDefaultPosition);
    if(Namer->ShowModal() == wxID_OK) {
        for(int i=0;i<4;i++) {
            chDim[i]->Append(Namer->GetValue());
        }

     chDim[0]->SetSelection(chDim[0]->GetCount()-1);

     APP->Data.DL.setdim(APP->Data.DL.dim()+1);
     APP->Data.labelDL.setdim(APP->Data.labelDL.dim()+1);
     APP->Data.StaticPoint.setdim(APP->Data.StaticPoint.dim() + 1);
     APP->Data.StaticPoint[APP->Data.StaticPoint.dim() - 1] = APP->Data.w/2;
    }

}

void MyFrame::OnRenameDimension(wxCommandEvent& event){
    wxTextEntryDialog *Renamer;
    Renamer = new wxTextEntryDialog(this, _T("Type the new name"), _T("Rename Dimension"), chDim[0]->GetString(chDim[0]->GetSelection()),
                                    wxTextEntryDialogStyle, wxDefaultPosition);

    Renamer->ShowModal();
    for(int i=0;i<4;i++)
        chDim[i]->SetString(chDim[0]->GetSelection(), Renamer->GetValue());
}

void MyFrame::OnDeleteDimension(wxCommandEvent& event){
    unsigned int sel = chDim[0]->GetSelection();
    unsigned int aux;
    wxString str = _T("Delete ");
    str.Append(chDim[0]->GetString(sel));
    str.Append(_T("?"));
     if(wxMessageBox(str, _T("Delete Dimension"), wxYES | wxNO, this) == wxYES) {

        for(int i=0;i<4;i++) {
            aux = chDim[i]->GetSelection();
            chDim[i]->Delete(sel);
            if(aux < chDim[i]->GetCount())
                chDim[i]->SetSelection(aux);
            else
                chDim[i]->SetSelection(aux - 1);
            }

        APP->Data.DL.DeleteDimension(sel);
        APP->Data.labelDL.DeleteDimension(sel);
        APP->Data.StaticPoint.DeleteDimension(sel);
        if(sel == chDim[0]->GetCount()) sel--;
        chDim[0]->SetSelection(sel);
    //APP->Data.StaticPoint.setdim(APP->Data.StaticPoint.dim() - 1);

     }
}


void MyFrame::OnChoiceDimension(wxCommandEvent& event) {

    for(int i=0;i<3;i++) {
        APP->Data.dim[i] = chDim[i+1]->GetSelection();
        APP->Data.DL.CurrentDim[i] = APP->Data.dim[i];
        APP->Data.labelDL.CurrentDim[i] = APP->Data.dim[i];
    }

    APP->RefreshAll();

}







//--------------------------------------------------------

void app_selective_refresh(Voxel v){
  Voxel Cut;

  if (!APP->Data.loaded) return;

  Cut = APP->Get2DViewSlice();

  /*
  if(Cut.x == v.x){
    app_draw_frame(1);
    Views[1]->canvas->Refresh();
  }
  if(Cut.y == v.y){
    app_draw_frame(2);
    Views[2]->canvas->Refresh();
  }
  if(Cut.z == v.z){
    app_draw_frame(0);
    Views[0]->canvas->Refresh();
  }
  */
  APP->Refresh2DCanvas();
}


//----------------------------------------

/*
ObjectDialog :: ObjectDialog(wxWindow *parent)
  : BaseDialog(parent, (char *)"Object Dialog"){
  SegmObject *segm;
  wxColour wxcolor;
  int i,n,id;

  wxScrolledWindow *swind = new wxScrolledWindow(this, -1, wxDefaultPosition, wxDefaultSize, wxHSCROLL | wxVSCROLL, _T("scrolledWindow"));

  sizer = new wxBoxSizer(wxVERTICAL);

  wxBitmap *bmeye   = new wxBitmap(eye_xpm);
  wxBitmap *bmneye  = new wxBitmap(noteye_xpm);
  wxBitmap *bmtrash = new wxBitmap(trash_xpm);

  n = this->n = APP->GetNumberOfObjs();
  if(n>0){
    v_but_color = (AlphaColourButton **)malloc(sizeof(AlphaColourButton *)*n);
    v_but_eye = (BitmapToggleButton **)malloc(sizeof(BitmapToggleButton *)*n);
    v_but_trash = (wxBitmapButton **)malloc(sizeof(wxBitmapButton *)*n);
  }
  for(i=0; i<n; i++){
    segm = APP->GetObjByIndex(i);

    wxPanel *bkg = new wxPanel(swind, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL, _T("panel_bkg"));
    if(i%2==0)
      SetColor(&wxcolor, 0xffffff);
    else
      SetColor(&wxcolor, 0xffffdd);
    bkg->SetBackgroundColour(wxcolor);

    id = APP->idManager->AllocID();
    v_but_color[i] = new AlphaColourButton(bkg, wxID_ANY);
    v_but_eye[i] = new BitmapToggleButton(bkg, wxID_ANY,
					  bmneye, bmeye);
    v_but_trash[i] = new wxBitmapButton(bkg, id, *bmtrash, wxDefaultPosition, wxDefaultSize, wxBU_AUTODRAW, wxDefaultValidator, _T("trash"));

    v_but_color[i]->SetValue(segm->color);
    v_but_color[i]->SetAlpha(segm->alpha);
    v_but_eye[i]->SetValue(segm->visibility);

    Connect( id, wxEVT_COMMAND_BUTTON_CLICKED,
	     wxCommandEventHandler(ObjectDialog::OnDelete),
	     NULL, NULL );

    wxString *wxstr = new wxString(segm->name, wxConvUTF8);
    wxStaticText *tname = new wxStaticText(bkg, -1, *wxstr, wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT, _T("staticText"));

    wxBoxSizer *hsizer = new wxBoxSizer(wxHORIZONTAL);

    hsizer->Add(v_but_eye[i],   0, wxALIGN_LEFT|wxEXPAND);
    hsizer->Add(v_but_trash[i], 0, wxALIGN_RIGHT|wxEXPAND);
    hsizer->Add(v_but_color[i], 0, wxALIGN_LEFT|wxEXPAND);
    hsizer->AddSpacer(10);
    hsizer->Add(tname,          1, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);

    bkg->SetSizer(hsizer, true);
    hsizer->SetSizeHints(bkg);
    hsizer->Layout();

    sizer->Add(bkg, 0, wxEXPAND);
  }

  swind->SetSizer(sizer, true);
  sizer->SetSizeHints(swind);
  sizer->Layout();

  this->AddPanel((wxPanel *)swind);
}
*/


ObjectDialog :: ObjectDialog(wxWindow *parent)
  : BaseDialog(parent, (char *)"Object Dialog"){
  /*
  SegmObject *segm;
  wxColour wxcolor;
  int i,n,id,id2;
  int w,h,wm=200;
  wxSize size(350, 420);
  wxPanel *bkg;

  wxScrolledWindow *swind = new wxScrolledWindow(this, -1,
						 wxDefaultPosition,
						 wxDefaultSize,
						 wxHSCROLL | wxVSCROLL,
						 _T("scrolledWindow"));

  swind->SetMinSize(size);
  wxBitmap *bmeye   = new wxBitmap(eye_xpm);
  wxBitmap *bmneye  = new wxBitmap(noteye_xpm);
  wxBitmap *bmtrash = new wxBitmap(trash_xpm);

  n = this->n = APP->GetNumberOfObjs();
  if(n>0){
    v_panel_bkg = (wxPanel **)malloc(sizeof(wxPanel *)*n);
    v_but_color = (AlphaColourButton **)malloc(sizeof(AlphaColourButton *)*n);
    v_but_eye = (BitmapToggleButton **)malloc(sizeof(BitmapToggleButton *)*n);
    v_but_trash = (wxBitmapButton **)malloc(sizeof(wxBitmapButton *)*n);
  }
  else{
    v_panel_bkg = NULL;
    v_but_color = NULL;
    v_but_eye   = NULL;
    v_but_trash = NULL;
  }

  swind->SetScrollRate(5, 5);
  swind->Scroll(0, 0);
  swind->SetVirtualSize(200,10+40*n);

  for(i=0; i<n; i++){
    segm = APP->GetObjByIndex(i);

    size.SetHeight(wxDefaultCoord);
    size.SetWidth(200);
    bkg = new wxPanel(swind, wxID_ANY,
		      wxPoint(10,10+40*i),//wxDefaultPosition,
		      wxDefaultSize, wxTAB_TRAVERSAL, _T("panel_bkg"));
    bkg->SetMinSize(size);

    if(i%2==0)
      SetColor(&wxcolor, 0xffffff);
    else
      SetColor(&wxcolor, 0xffffdd);
    bkg->SetBackgroundColour(wxcolor);

    v_panel_bkg[i] = bkg;
    v_but_color[i] = new AlphaColourButton(bkg, wxID_ANY);
    id2 = APP->idManager->AllocID();
    v_but_eye[i] = new BitmapToggleButton(bkg, id2, bmneye, bmeye);
    id = APP->idManager->AllocID();
    v_but_trash[i] = new wxBitmapButton(bkg, id, *bmtrash, wxDefaultPosition, wxDefaultSize, wxBU_AUTODRAW, wxDefaultValidator, _T("trash"));

    v_but_color[i]->SetValue(segm->color);
    v_but_color[i]->SetAlpha(segm->alpha);
    v_but_eye[i]->SetValue(segm->visibility);

    Connect( id, wxEVT_COMMAND_BUTTON_CLICKED,
	     wxCommandEventHandler(ObjectDialog::OnDelete),
	     NULL, NULL );
    Connect( id2, wxEVT_COMMAND_BUTTON_CLICKED,
	     wxCommandEventHandler(ObjectDialog::OnChange),
	     NULL, NULL );

    wxString *wxstr = new wxString(segm->name, wxConvUTF8);
    wxStaticText *tname = new wxStaticText(bkg, -1, *wxstr, wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT, _T("staticText"));
    size.SetHeight(wxDefaultCoord);
    size.SetWidth(200);
    tname->SetMinSize(size);
    wxBoxSizer *hsizer = new wxBoxSizer(wxHORIZONTAL);

    hsizer->Add(v_but_eye[i],   0, wxALIGN_LEFT|wxEXPAND);
    hsizer->Add(v_but_trash[i], 0, wxALIGN_RIGHT|wxEXPAND);
    hsizer->Add(v_but_color[i], 0, wxALIGN_LEFT|wxEXPAND);
    hsizer->AddSpacer(10);
    hsizer->Add(tname,          1, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);

    bkg->SetSizer(hsizer, true);
    hsizer->SetSizeHints(bkg);
    hsizer->Layout();

    bkg->GetSize(&w, &h);
    if(w>wm) wm = w;
  }
  for(i=0; i<n; i++){
    size.SetHeight(wxDefaultCoord);
    size.SetWidth(wm);
    (v_panel_bkg[i])->SetSize(size);
  }
  swind->SetVirtualSize(wm,10+MAX(40*n,40));


  wxPaintDC paintDC(swind);
  swind->DoPrepareDC(paintDC);

  delete bmeye;
  delete bmneye;
  delete bmtrash;

  this->AddPanel((wxPanel *)swind);
  */
}


ObjectDialog :: ~ObjectDialog(){
  /*
  int i,n,id,id2;

  n = this->n;
  for(i=0; i<n; i++){
    id = v_but_trash[i]->GetId();
    id2 = v_but_eye[i]->GetId();
    Disconnect( id, wxEVT_COMMAND_BUTTON_CLICKED,
		wxCommandEventHandler(ObjectDialog::OnDelete),
		NULL, NULL );
    Disconnect( id2, wxEVT_COMMAND_BUTTON_CLICKED,
		wxCommandEventHandler(ObjectDialog::OnChange),
		NULL, NULL );
    APP->idManager->FreeID(id);
  }
  if(n>0){
    if(v_panel_bkg!=NULL) free(v_panel_bkg);
    if(v_but_color!=NULL) free(v_but_color);
    if(v_but_eye!=NULL)   free(v_but_eye);
    if(v_but_trash!=NULL) free(v_but_trash);
    v_panel_bkg = NULL;
    v_but_color = NULL;
    v_but_eye   = NULL;
    v_but_trash = NULL;
  }
  */
}


void ObjectDialog :: OnDelete(wxCommandEvent& event){
  /*
  SegmObject *segm;
  char msg[1024];
  int i,n,id;

  id = event.GetId();

  n = this->n;
  for(i=0; i<n; i++){
    if(id == v_but_trash[i]->GetId())
      break;
  }
  if(i<n){
    segm = APP->GetObjByIndex(i);
    sprintf(msg,"Do you really wish to delete the %s?",segm->name);

    wxString wxmsg(msg, wxConvUTF8);
    wxMessageDialog *dialog = new wxMessageDialog(this, wxmsg, _T("Delete Confirmation"), wxYES_NO | wxICON_QUESTION, wxDefaultPosition);
    if(dialog->ShowModal() == wxID_YES){
      APP->DelObjByIndex(i);
      APP->DrawSegmObjects();
      BaseDialog::OnOk(event);
    }
  }
  */
}


void ObjectDialog :: OnCancel(wxCommandEvent& event){
  BaseDialog::OnCancel(event);
}

void ObjectDialog :: OnOk(wxCommandEvent& event){
  /*
  SegmObject *segm;
  int i,n;

  n = APP->GetNumberOfObjs();
  for(i=0; i<n; i++){
    segm = APP->GetObjByIndex(i);

    segm->color = v_but_color[i]->GetValue();
    segm->alpha = v_but_color[i]->GetAlpha();
    segm->visibility = v_but_eye[i]->GetValue();
  }
  APP->DrawSegmObjects();
  */
  BaseDialog::OnOk(event);
}


void ObjectDialog :: OnChange(wxCommandEvent& event){
  /*
  SegmObject *segm;
  int i,n;
  n = APP->GetNumberOfObjs();
  for(i=0; i<n; i++){
    segm = APP->GetObjByIndex(i);
    segm->color = v_but_color[i]->GetValue();
    segm->alpha = v_but_color[i]->GetAlpha();
    segm->visibility = v_but_eye[i]->GetValue();
  }
  APP->DrawSegmObjects();
  */
}


//----------------------------------------

ObjectPickerDialog::ObjectPickerDialog(wxWindow *parent,
				       wxWindowID id,
				       char *title,
				       const char *choices[],
				       int n,
				       bool readonly,
				       bool getcolor,
				       bool getname)
  : BaseDialog(parent, title){
  long style;
  int i;
  wxString *wxchoices;
  wxString wxstr;
  BasePanel  *panel;

  panel = new BasePanel(this);
  panel->Show(true);

  if(getcolor){
    wxStaticText *tColor = new wxStaticText(panel, -1, _T("Color"), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT, _T("staticText1"));
    butColor = new ColourButton(panel, id);
    wxBoxSizer *hbsColor = new wxBoxSizer(wxHORIZONTAL);
    hbsColor->Add(tColor, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    hbsColor->Add(butColor, 0, wxALIGN_LEFT);
    panel->sizer->Add(hbsColor, 0, wxEXPAND);
  }
  else
    butColor = NULL;

  if(getname){
    if(n>0){
      wxchoices = new wxString[n];
      for(i=0; i<n; i++)
	wxchoices[i] = wxString::FromAscii(choices[i]);
      wxstr = wxString::FromAscii(choices[0]);
    }
    else
      wxchoices = NULL;

    if(readonly){
      style = wxCB_READONLY;
      if(n>0) wxstr = wxString::FromAscii(choices[0]);
    }
    else{
      style = wxCB_DROPDOWN;
      wxstr = _T("");
    }

    wxStaticText *tName = new wxStaticText(panel, -1, _T("Name"), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT, _T("staticText2"));
    texName = new wxComboBox(panel, id, wxstr, wxDefaultPosition, wxDefaultSize, n, wxchoices, style, wxDefaultValidator, _T("comboBox"));
    if (readonly && n>0) texName->SetSelection(0);
    wxBoxSizer *hbsName = new wxBoxSizer(wxHORIZONTAL);
    hbsName->Add(tName, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);
    hbsName->Add(texName, 1, wxALIGN_LEFT|wxEXPAND);
    panel->sizer->Add(hbsName, 0, wxEXPAND);
  }
  else
    texName = NULL;

  panel->sizer->SetSizeHints(panel);
  panel->sizer->Layout();

  this->AddPanel(panel);
}


ObjectPickerDialog::~ObjectPickerDialog(){}


void ObjectPickerDialog::GetName(char *name){
  if(texName!=NULL){
    wxString wxstr = texName->GetValue();
    //strcpy(name, wxstr.mb_str());
    strcpy(name, wxstr.ToAscii());
  }
  else
    name[0] = '\0';
}


int  ObjectPickerDialog::GetColor(){
  if(butColor!=NULL)
    return butColor->GetValue();
  else
    return NIL;
}


//-------------------------------------------


// PopUpView :: PopUpView(Scene *scn, char *title)
//   : BaseDialog(APP->Window, title){
//   View2DModule*view2D;
//   wxSize size(200, 200);
//   int n;

//   this->view_scn = scn;
//   n = modManager->GetNumberModules(Module::VIEW2D);
//   if(n==0) Error((char *)"PopUpView",(char *)"View2DModule not loaded");
//   view2D = (View2DModule*)modManager->GetModule(Module::VIEW2D, 0);

//   wxPanel *opanel = new wxPanel(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxNO_BORDER, _T("panel0"));
//   this->id_ori = APP->idManager->AllocID();
//   wxString oriChoices[3];
//   oriChoices[0] = _T("Axial");
//   oriChoices[1] = _T("Coronal");
//   oriChoices[2] = _T("Sagittal");
//   chOrientation = new wxChoice(opanel, id_ori, wxDefaultPosition, wxDefaultSize, 3, oriChoices, 0, wxDefaultValidator, _T("choice0"));
//   chOrientation->SetSelection(0);
//   this->current = 0;

//   wxSizer *osizer = new wxBoxSizer(wxHORIZONTAL);
//   opanel->SetSizer(osizer, true);
//   osizer->Add(chOrientation, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL);
//   osizer->SetSizeHints(opanel);
//   osizer->Layout();

//   sizer->Prepend(opanel, 0, wxBOTTOM);
//   sizer->PrependSpacer(2);
//   sizer->SetSizeHints(this);

//   this->view[0] = view2D->GetCustomViewPanel(this, scn, 'z');
//   this->view[1] = view2D->GetCustomViewPanel(this, scn, 'y');
//   this->view[2] = view2D->GetCustomViewPanel(this, scn, 'x');
//   //this->AddPanel(this->view[this->current]);
//   this->AddPanel(this->view[0]);
//   this->AddPanel(this->view[1]);
//   this->AddPanel(this->view[2]);
//   sizer->Hide(this->view[1], true);
//   sizer->Hide(this->view[2], true);
//   sizer->Layout();
//   this->SetMinSize(size);

//   Connect( id_ori, wxEVT_COMMAND_CHOICE_SELECTED,
// 	   wxCommandEventHandler(PopUpView::OnChangeSliceOrientation),
// 	   NULL, NULL );

//   this->ShowWindow();
// }


// PopUpView::~PopUpView(){
//   View2DModule*view2D;
//   int n;

//   n = modManager->GetNumberModules(Module::VIEW2D);
//   if(n==0) Error((char *)"PopUpView",(char *)"View2DModule not loaded");
//   view2D = (View2DModule*)modManager->GetModule(Module::VIEW2D, 0);
//   view2D->DelViewPanel(&(this->view[0]));
//   view2D->DelViewPanel(&(this->view[1]));
//   view2D->DelViewPanel(&(this->view[2]));

//   Disconnect( id_ori, wxEVT_COMMAND_CHOICE_SELECTED,
// 	      wxCommandEventHandler(PopUpView::OnChangeSliceOrientation),
// 	      NULL, NULL );
//   APP->idManager->FreeID(id_ori);
// }


// void PopUpView::OnChangeSliceOrientation(wxCommandEvent& event){
//   wxSize size = this->GetSize();

//   sizer->Hide(this->view[this->current], true);
//   //this->DetachPanel(this->view[this->current]);

//   this->current = chOrientation->GetSelection();
//   //this->AddPanel(this->view[this->current]);
//   sizer->Show(this->view[this->current], true, true);
//   sizer->Layout();
//   this->SetSize(size);
// }








