
#ifndef _BIA_H_
#define _BIA_H_


#include "gui.h"
extern "C" {
#include "ift.h"
#include "code_c.h"
//#include "bzlib.h"
}

//#include "gft.c"

#include "code_cpp.h"

#include "Eigen/Eigen/Eigen"

#include "module.h"

typedef enum {VIEWOPT_DENSITY=0,
	      VIEWOPT_POINTS,
	      VIEWOPT_DENSITY_POINTS} ViewOptType;

enum{CLUTO_RB=0, CLUTO_RBR, CLUTO_DIRECT, CLUTO_AGGRO, CLUTO_GRAPH, CLUTO_BAGGLO};
enum{CLUTO_COS, CLUTO_CORR, CLUTO_DIST, CLUTO_JACE};
enum{CLUTO_I1, CLUTO_I2, CLUTO_E1, CLUTO_G1, CLUTO_G1P, CLUTO_H1, CLUTO_H2, CLUTO_SLINK,
     CLUTO_WSLINK, CLUTO_CLINK, CLUTO_WCLINK, CLUTO_UPGMA};
enum{COSIN=0, INV_DIST, INV_SQRDIST, MINUS_DIST, MINUS_SQRDIST, EXP_DIST};

Subgraph* DLtoSubgraph(PSet::DataList DL);
PSet::DataList SubgraphtoDL(Subgraph *g);
void ReadFromSubgraph(Subgraph *g, PSet::DataList *DL);

void AddDensityDistributionAroundPoint(Scene *scn[], int px, int py, int pz, int Radius, int color);
int GetPointSetIndex(PSet::DataList **p, int cl, int sel);           //returns the index of the selected PointSet and (*p) will point to the referring DataList.
void CallCluto(PSet::DataList *DL, int nclusters, int clMethod, int sim, int crfun);
void ReadCluto(PSet::DataList *output, int nclusters);
float SimilarityFunction(int simfun, PSet::Point P1, PSet::Point P2, float param);
Eigen :: MatrixXf WriteSimilarityMatrix(int simfun, float param, PSet::DataList DL);
void SpectralClustering(int simfun, float param, float CutValue, PSet::DataList* DL);
void SpectralClusteringNClusters(int simfun, float param, int NClusters, float stop_value, PSet::DataList *DL);
void NormalizedCutSpectralClustering(int simfun, float param, float CutValue, PSet::DataList *DL);
float SimpleAccuracy(PSet::DataList DL);
void KMeansClassify(PSet::DataList *DL, int k, float stop_value);
void GFT_Clustering(PSet::DataList *DL, int NClusters, int outdegree, float pparam, float qparam,
                    int *GFTfunc, int GFTGraph, float sigma, bool CorrectWeights, bool OPFsigma);


typedef struct _AppPreferencesType{
  bool IsMaximized;
  char dir_LoadVolume[512];
  char dir_LoadLabel[512];
  char dir_SaveVolume[512];
  char dir_SaveLabel[512];
  char dir_SaveObject[512];
  char dir_LoadMarkers[512];
  char dir_SaveMarkers[512];

  int  ext_LoadVolume;
  int  ext_LoadLabel;
  int  ext_SaveVolume;
  int  ext_SaveLabel;
  int  ext_SaveObject;
  int  ext_LoadMarkers;
  int  ext_SaveMarkers;

  int color_preproc;
  int color_segmentation;
  int color_view;
  int color_analysis;
} AppPreferencesType;



typedef struct _appDataType{
  char         volumename[512];
  char         labelname[512];
  char         projectfilename[512];
  int          w,h,nframes;

  PSet::DataList DL;
  PSet::DataList labelDL;

  std::vector<PSet::Spline> SList;

  PSet::Point StaticPoint;

  //Scene        *den;
  Scene        *den[4];
  Scene        *label;
  int Raio;                 //Variável temporária, será removida pois deve ser armazenada na Handler.
  unsigned int dim[3];


  int          loaded; // is there any image loaded?
} AppDataType;

typedef struct AppDataControl{


    int NPoints;
    float stddev, Velocity, NLoops, LinRadius;
    bool simple;
    std::vector<int> ClassColor;
    std::vector< std::vector<int> > SplineControl;
    PSet::Spline *CurrentSpline;
    char path[512];

    int LoadedFileExtension;


    AppDataControl();

} AppDataControl;


//--------------------------------------------------------

class MyFrame: public wxFrame{
public:

  MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size);
  ~MyFrame();
  void SetViewPanel(wxPanel *view, int pos);
  void UniformView();
  void AppendOptPanel(wxPanel *panel, Module::ModuleType type);
  void PrependOptPanel(wxPanel *panel, Module::ModuleType type);
  bool DetachOptPanel(wxPanel *panel);
  void LoadVolume(char *filename);
  void LoadLabel(char *filename,char *name, int color);
  void LoadScnGradient(char *filename);
  void OnLoadVolume(wxCommandEvent & event);
  void OnLoadLabel(wxCommandEvent & event);
  void OnLoadScnGradient(wxCommandEvent & event);
  void OnLoadSTD(wxCommandEvent & event);
  void OnLoadSIMPLESTD(wxCommandEvent & event);
  void OnLoadCLUTO(wxCommandEvent & event);
  void OnLoadLIBOPF(wxCommandEvent & event);
  void OnLoadMATLAB(wxCommandEvent & event);
  void OnLoadOPFDAT(wxCommandEvent & event);


  void OnSaveVolume(wxCommandEvent & event);
  void OnSaveLabel(wxCommandEvent & event);
  void OnSaveObject(wxCommandEvent & event);
  void OnSaveObjectMask(wxCommandEvent & event);
  void OnLoadMarkers(wxCommandEvent & event);
  void OnSaveMarkers(wxCommandEvent & event);
  void OnQuit(wxCommandEvent& event);
  void OnAbout(wxCommandEvent& event);
  void OnVolumeInformation(wxCommandEvent& event);
  void OnPreprocessing(wxCommandEvent& event);
  void OnDraw(wxCommandEvent& event);
  void OnAnalysis(wxCommandEvent& event);
  void On3DVisualization(wxCommandEvent& event);
  void OnClassify(wxCommandEvent& event);
  void OnShowObjWindow(wxCommandEvent & event);
  void OnZoomin(wxCommandEvent& event);
  void OnZoomout(wxCommandEvent& event);
  void OnChangeDrawMarker(wxCommandEvent& event);
  void OnChangeBriContr(wxScrollEvent& event);
  void OnShowBriContr(wxCommandEvent& event);
  void OnSelectView(wxCommandEvent & event);
  void OnChangeViewOptions(wxCommandEvent& event);
  void OnChoiceDimension(wxCommandEvent& event);

  void OnRenameDimension(wxCommandEvent& event);
  void OnAddDimension(wxCommandEvent& event);
  void OnDeleteDimension(wxCommandEvent& event);
  void EnableObjWindow(bool enable);
  void SetBriContr(int B, int C);
  void GetBriContr(int *B, int *C);
  void GetViewOptions(ViewOptType *vopt);


private:
  void SetOptPanelBkgColor(wxPanel *panel,
			   Module::ModuleType type);

  wxToolBar      *toolBar;

  wxPanel        *panel;
  wxBoxSizer     *vsizer;
  wxSplitterWindow *splitter;
  wxSplitterWindow *topWindow;
  wxSplitterWindow *bottomWindow;
  wxPanel *Views[4];

  wxBitmapButton *buteye;
  wxBitmapButton *buZoomin;
  wxBitmapButton *buZoomout;
  wxBitmapButton *buBriContr;
  wxChoice       *chView, *chDim[4];
  BriContrDialog *BCDialog;
  float zoom;
  int   B,C;

  DECLARE_EVENT_TABLE()
};

//----------------------------------------------------------

class BIAClass{
public:
  AppPreferencesType Preferences;
  AppDataType        Data;
  AppDataControl     Control;
  MyFrame           *Window;
  DynamicID         *idManager;

  BIAClass();
  ~BIAClass();

  //***** Busy/Message functions: ****

  void Busy(char *msg);
  void Unbusy();
  void StatusMessage(char *msg);
  int  GetColorDialog();

  //***** Data functions: **********
  void InitData();
  void ResetData();
  void DestroyData();
  void SetDataVolume(Scene *scn);
  void SetDataVolume_NoDestroy(Scene *scn);
  int  ShowImageModalityDialog();

  //***** View functions: *********
  void GetViewOptions(ViewOptType *vopt);
  void Set2DViewModule(View2DModule *mod);
  void Set3DViewModule(View3DModule *mod);
  void Refresh2DCanvas();
  void Refresh2DCanvas(char axis);
  void Refresh3DCanvas(bool dataChanged, float quality);
  void SetRefresh2DHandler(RefreshHandler *handler);
  void Clear2DCanvas();
  void ChangeDrawMarker();
  CImage *Copy3DCanvasAsCImage();
  CImage *Copy2DCanvasAsCImage(char axis);

  //void DrawSegmObjects();
  void UpdateDensityScene();
  void UpdateClassifiedDensityScene();
  void UpdateDensityScene(PSet::DataList DL);


  void UpdateLabel();

  void RefreshAll();
  void RefreshAll(PSet::DataList DL);

  void EnableObjWindow(bool enable);
  void Set2DViewCursor(const wxCursor *cursor, char axis);
  void SetDefaultInteractionHandler();
  void Set2DViewInteractionHandler(InteractionHandler *handler,
				   char axis);
  void  Set2DViewSlice(Voxel Cut);
  Voxel Get2DViewSlice();
  void  SetBriContr(int B, int C);
  void  GetBriContr(int *B, int *C);
  void  SetZoomLevel(float zoom);
  float GetZoomLevel();

  //***** OptPanel functions: ********

  void AppendOptPanel(wxPanel *panel,
		      Module::ModuleType type);
  void PrependOptPanel(wxPanel *panel,
		       Module::ModuleType type);
  bool DetachOptPanel(wxPanel *panel);

  //**** Label functions: ************

  int   GetLabelColour(int label);
  int   GetLabelAlpha(int label);
  void  SetLabelColour(int label,
		       int color);
  void  SetLabelColour(int label,
		       int color,
		       int alpha);
  void  DrawBrush(int p, int label,
		  AdjRel *A, char axis);
  void  DrawBrushTrace(int p, int q,
		       int label,
		       AdjRel *A, char axis);
  void  DrawBrushCustom(int p, int label,
			AdjRel *A, char axis,
			Scene *scn);
  void  DrawBrushTraceCustom(int p, int q,
			     int label,
			     AdjRel *A, char axis,
			     Scene *scn);
  Scene *CopyVisibleLabelUnion();
  Scene *CopyDistinctVisibleLabels();

  //**** Object functions: ***********

  int         ShowObjColorDialog(int *color,
				 char *name);

  //**** Dependency functions: ***********


private:
  //***** Busy/Message data: ****
  wxWindowDisabler *disableAll;
  wxBusyCursor     *busycursor;
  wxBusyInfo       *busyinfo;

  int          *labelcolor;
  int          *labelalpha;
  int           Lmax; //Label maximum.

  int           _GetMarkerID(int mk);
  int           _GetMarkerLabel(int mk);
  int           _GetMarkerValue(int id, int lb);

  View2DModule *mod2D;
  View3DModule *mod3D;
  void InitPreferences();
  void SavePreferences();
  void LoadPreferences();
  void MakePalette();
};

//----------------------------------------------------------

class ObjectDialog : public BaseDialog{
public:
  ObjectDialog(wxWindow *parent);
  ~ObjectDialog();
  void OnCancel(wxCommandEvent& event);
  void OnOk(wxCommandEvent& event);
  void OnDelete(wxCommandEvent& event);
  void OnChange(wxCommandEvent& event);


private:
  wxBoxSizer *sizer;
  int n;
  wxPanel            **v_panel_bkg;
  AlphaColourButton  **v_but_color;
  BitmapToggleButton **v_but_eye;
  wxBitmapButton     **v_but_trash;
};

//----------------------------------------------------------

class ObjectPickerDialog : public BaseDialog{
public:
  ObjectPickerDialog(wxWindow   *parent,
		     wxWindowID  id,
		     char       *title,
		     const char *choices[], int n,
		     bool        readonly,
		     bool        getcolor,
		     bool        getname);
  ~ObjectPickerDialog();
  void GetName(char *name);
  int  GetColor();

  ColourButton *butColor;
  wxComboBox   *texName;
private:
};

//---------------------------------------------------------

#endif

